Package: DiscreteTests
Type: Package
Title: Vectorised Computation of P-Values and Their Supports for
        Several Discrete Statistical Tests
Version: 0.5.0
Date: 2026-08-26
Authors@R: c(
    person("Florian", "Junge", role = c("cre", "aut"),
           email = "diso.fbmn@h-da.de",
           comment = c(ORCID = "0009-0001-6856-6938")),
    person("Christina", "Kihn", role = "aut"),
    person("Sebastian", "Döhler", role = "ctb",
           comment = c(ORCID = "0000-0002-0321-6355")),
    person("Guillermo", "Durand", role = "ctb",
           comment = c(ORCID = "0000-0003-4056-5631"))
  )
Description: Provides vectorised functions for computing p-values of various
  common discrete statistical tests, as described e.g. in Agresti (2002)
  <doi:10.1002/0471249688>, including their distributions. Exact and
  approximate computation methods are provided. For exact ones, several
  procedures of determining two-sided p-values are included, which are
  outlined in more detail in Hirji (2006) <doi:10.1201/9781420036190>.
License: GPL-3
Encoding: UTF-8
Language: en-GB
Imports: Rcpp, R6, checkmate, lifecycle, cli, tibble, withr
LinkingTo: Rcpp
URL: https://github.com/DISOhda/DiscreteTests
BugReports: https://github.com/DISOhda/DiscreteTests/issues
Config/roxygen2/version: 8.1.0
NeedsCompilation: yes
Packaged: 2026-08-26 14:07:16 UTC; fjunge
Author: Florian Junge [cre, aut] (ORCID:
    <https://orcid.org/0009-0001-6856-6938>),
  Christina Kihn [aut],
  Sebastian Döhler [ctb] (ORCID: <https://orcid.org/0000-0002-0321-6355>),
  Guillermo Durand [ctb] (ORCID: <https://orcid.org/0000-0003-4056-5631>)
Maintainer: Florian Junge <diso.fbmn@h-da.de>
Repository: CRAN
Date/Publication: 2026-08-26 15:10:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(summary,DiscreteTestResults)
export(DiscreteTestResults)
export(DiscreteTestResultsSummary)
export(binom.test.pv)
export(binom_test_pv)
export(fisher.test.pv)
export(fisher_test_pv)
export(homogeneity_test_pv)
export(mann_whitney_test_pv)
export(mcnemar.test.pv)
export(mcnemar_test_pv)
export(perm_test_pv)
export(poisson.test.pv)
export(poisson_test_pv)
export(sign_test_pv)
export(wilcox_test_pv)
import(Rcpp)
importFrom(R6,R6Class)
importFrom(checkmate,
  assert_character,
  assert_data_frame,
  assert_int,
  assert_integerish,
  assert_list,
  assert_logical,
  assert_numeric,
  assert_r6,
  assert_subset,
  qassert,
  qassertr
)
importFrom(cli,
  ansi_collapse,
  cli_abort,
  cli_end,
  cli_h1,
  cli_h2,
  cli_h3,
  cli_li,
  cli_text,
  cli_ul,
  cli_verbatim,
  cli_warn,
  col_blue,
  col_green,
  col_red,
  qty
)
importFrom(lifecycle,
  deprecate_stop,
  deprecated
)
importFrom(stats,
  dbinom,
  dhyper,
  dnorm,
  dpois,
  dsignrank,
  pchisq,
  pnorm,
  qnorm,
  qpois,
  setNames,
  stepfun
)
importFrom(tibble,
  add_column,
  as_tibble,
  rowid_to_column
)
importFrom(withr,with_options)
useDynLib(DiscreteTests)

#' @name poisson_test_pv
#'
#' @title
#' Poisson Test
#'
#' @description
#' `poisson_test_pv()` performs an exact or approximate Poisson test about the
#' rate parameter of a Poisson distribution. In contrast to
#' [`stats::poisson.test()`], it is vectorised, only calculates *p*-values and
#' offers a normal approximation of their computation. Furthermore, it is
#' capable of returning the discrete *p*-value supports, i.e. all observable
#' *p*-values under a null hypothesis. Multiple tests can be evaluated
#' simultaneously. In two-sided tests, several procedures of obtaining the
#' respective *p*-values are implemented.
#'
#' **Note**: Please do not use the older `poisson.test.pv()` anymore! It is now
#' defunct and will be removed in a future version.\cr
#' `r lifecycle::badge('superseded')`
#'
#' @param x        integer vector giving the number of events.
#' @param lambda   non-negative numerical vector of hypothesised rate(s).
#'
#' @template param
#' @templateVar alternative TRUE
#' @templateVar ts.method TRUE
#' @templateVar exact TRUE
#' @templateVar correct TRUE
#' @templateVar simple.output TRUE
#'
#' @details
#' The parameters `x`, `lambda` and `alternative` are vectorised. They are
#' replicated automatically to have the same lengths. This allows multiple null
#' hypotheses to be tested simultaneously.
#'
#' Since the Poisson distribution itself has an infinite support, so do the
#' *p*-values of exact Poisson tests. Therefore, supports only contain
#' *p*-values that are not rounded off to 0.
#'
#' @template details_two_sided
#'
#' @template return
#'
#' @seealso
#' [`stats::poisson.test()`], [`binom_test_pv()`]
#'
#' @references
#' Blaker, H. (2000). Confidence curves and improved exact confidence intervals
#'   for discrete distributions. *Canadian Journal of Statistics*,
#'   **28**(4), pp. 783-798. \doi{10.2307/3315916}
#'
#' Hirji, K. F. (2006). *Exact analysis of discrete data*. New York: Chapman
#'   and Hall/CRC. pp. 55-83. \doi{10.1201/9781420036190}
#'
#' @examples
#' # Constructing
#' k <- c(4, 2, 2, 14, 6, 9, 4, 0, 1)
#' lambda <- c(3, 2, 1)
#'
#' # Exact two-sided p-values ("blaker") and their supports
#' results_ex  <- poisson_test_pv(k, lambda, ts_method = "blaker")
#' print(results_ex)
#' results_ex$get_pvalues()
#' results_ex$get_pvalue_supports()
#'
#' # Normal-approximated one-sided p-values ("less") and their supports
#' results_ap  <- poisson_test_pv(k, lambda, "less", exact = FALSE)
#' print(results_ap)
#' results_ap$get_pvalues()
#' results_ap$get_pvalue_supports()
#'
#' @importFrom stats qnorm
#' @importFrom checkmate assert_integerish qassert
#' @export
poisson_test_pv <- function(
  x,
  lambda = 1,
  alternative = "two.sided",
  ts_method = "minlike",
  exact = TRUE,
  correct = TRUE,
  simple_output = FALSE
) {
  # plausibility checks of input parameters
  qassert(x, "A+")
  assert_integerish(x, lower = 0)
  x <- round(x)
  len_x <- length(x)

  qassert(lambda, "N+[0,)")
  len_l <- length(lambda)

  qassert(exact, "B1")
  if(!exact) qassert(correct, "B1")

  ts_method <- match.arg(
    tolower(ts_method),
    c("minlike", "blaker", "absdist", "central")
  )

  len_a <- length(alternative)
  for(i in seq_len(len_a)){
    alternative[i] <- match.arg(
      tolower(alternative[i]),
      c("two.sided", "less", "greater")
    )
    if(exact && alternative[i] == "two.sided")
      alternative[i] <- ts_method
  }

  qassert(simple_output, "B1")

  # replicate inputs to same length
  len_g <- max(len_x, len_l, len_a)
  if(len_x < len_g) x <- rep_len(x, len_g)
  if(len_l < len_g) lambda <- rep_len(lambda, len_g)
  if(len_a < len_g) alternative <- rep_len(alternative, len_g)

  # determine unique parameter sets
  params   <- unique(data.frame(lambda, alternative))
  lambda_u <- params$lambda
  alt_u    <- params$alternative
  len_u    <- length(lambda_u)

  # prepare output
  res <- numeric(len_g)
  if(!simple_output) {
    supports <- vector("list", len_u)
    indices  <- vector("list", len_u)
  }

  # begin computations
  for(i in seq_len(len_u)) {
    idx_supp <- which(lambda_u[i] == lambda & alt_u[i] == alternative)
    N <- max(x[idx_supp])

    if(exact) {
      # generate all probabilities under current lambda
      d <- generate_poisson_probs(lambda_u[i])
      # difference between number of probabilities and largest desired x-value
      len_diff <- N - length(d) + 1
      # add 0-probabilities, if necessary
      if(len_diff > 0) d <- c(d, rep(0, len_diff))
      # compute p-value support
      pv_supp <- support_exact(
        alternative = alt_u[i],
        probs = d,
        expectations = abs(seq_along(d) - 1 - lambda_u[i])
      )
    } else {
      # find quantile with (approx.) smallest probability > 0
      q <- -floor(qnorm(2^-1074, -lambda_u[i], sqrt(lambda_u[i])))
      # greatest observation
      M <- max(q, N)
      # compute p-value support
      if(lambda_u[i] == 0){
        pv_supp <- switch(
          EXPR      = alt_u[i],
          less      = rep(1, M + 1),
          greater   = c(1, rep(0, M)),
          two.sided = c(1, rep(0, M))
        )
      } else {
        # compute p-value support
        pv_supp <- support_normal(
          alternative = alt_u[i],
          x = 0:M,
          mean = lambda_u[i],
          sd = sqrt(lambda_u[i]),
          correct = correct
        )
      }
    }

    # store results and support
    res[idx_supp] <- pv_supp[x[idx_supp] + 1]
    if(!simple_output) {
      supports[[i]] <- unique(sort(pv_supp))
      indices[[i]]  <- idx_supp
    }
  }

  # create output object
  out <- if(!simple_output) {
    dnames <- sapply(match.call(), deparse1)

    DiscreteTestResults$new(
      test_name = "Poisson test",
      inputs = list(
        observations = data.frame(
          `Number of events` = x,
          check.names = FALSE
        ),
        parameters = NULL,
        nullvalues = data.frame(
          `event rate` = lambda,
          check.names = FALSE
        ),
        computation = Filter(
          function(df) !all(is.na(df)),
          data.frame(
            alternative = alternative,
            exact = exact,
            distribution = ifelse(exact, "Poisson", "normal"),
            #distribution.mean = if(exact) NA_real_ else lambda,
            #distribution.sd = if(exact) rep(NA_real_, len_g) else sqrt(lambda),
            `continuity correction` = if(exact) NA else correct,
            check.names = FALSE
          )
        )
      ),
      statistics = NULL,
      p_values = res,
      pvalue_supports = supports,
      support_indices = indices,
      data_name = dnames["x"]
    )
  } else res

  # return results
  return(out)
}

#' @rdname poisson_test_pv
#' @export
#' @importFrom lifecycle deprecate_stop
poisson.test.pv <- function(
    x,
    lambda = 1,
    alternative = "two.sided",
    ts.method = "minlike",
    exact = TRUE,
    correct = TRUE,
    simple.output = FALSE
) {
  deprecate_stop("0.2", "poisson.test.pv()", "poisson_test_pv()")
}

#' @name mcnemar_test_pv
#'
#' @title
#' McNemar's Test for Count Data
#'
#' @description
#' Performs McNemar's chi-square test or an exact variant to assess the symmetry
#' of rows and columns in a 2-by-2 contingency table. In contrast to
#' [`stats::mcnemar.test()`], it is vectorised, only calculates *p*-values and
#' offers their exact computation. Furthermore, it is capable of returning the
#' discrete *p*-value supports, i.e. all observable *p*-values under a null
#' hypothesis. Multiple tables can be analysed simultaneously. In two-sided
#' tests, several procedures of obtaining the respective *p*-values are
#' implemented. It is a special case of the [binomial test][binom_test_pv()].
#'
#' **Note**: Please do not use the older `mcnemar.test.pv()` anymore! It is now
#' defunct and will be removed in a future version.\cr
#' `r lifecycle::badge('superseded')`
#'
#' @param x   integer vector with four elements, a 2-by-2 matrix or an integer
#'            matrix (or data frame) with four columns where each line
#'            represents a 2-by-2 table to be tested.
#'
#' @template param
#' @templateVar alternative TRUE
#' @templateVar exact TRUE
#' @templateVar correct TRUE
#' @templateVar simple.output TRUE
#'
#' @details
#' The parameters `x` and `alternative` are vectorised. They are replicated
#' automatically, such that the number of `x`'s rows is the same as the length
#' of `alternative`. This allows multiple null hypotheses to be tested
#' simultaneously. Since `x` is coerced to a matrix (if necessary) with four
#' columns, it is replicated row-wise.
#'
#' It can be shown that McNemar's test is a special case of the binomial test.
#' In contrast to [`binom_test_pv()`], `mcnemar_test_pv()` does not allow
#' specifying exact two-sided p-value calculation procedures. The reason is that
#' McNemar's exact test always tests for a probability of 0.5, in which case all
#' these exact two-sided p-value computation methods yield exactly the same
#' results.
#'
#' @template return
#'
#' @references
#' Agresti, A. (2002). *Categorical data analysis*. Second Edition. New York:
#'   John Wiley & Sons. pp. 411–413. \doi{10.1002/0471249688}
#'
#' @seealso
#' [`stats::mcnemar.test()`], [`binom_test_pv()`]
#'
#' @examples
#' # Constructing
#' S1 <- c(4, 2, 2, 14, 6, 9, 4, 0, 1)
#' S2 <- c(0, 0, 1, 3, 2, 1, 2, 2, 2)
#' N1 <- rep(148, 9)
#' N2 <- rep(132, 9)
#' F1 <- N1 - S1
#' F2 <- N2 - S2
#' df <- data.frame(S1, F1, S2, F2)
#'
#' # Exact p-values and their supports
#' results_ex  <- mcnemar_test_pv(df)
#' print(results_ex)
#' results_ex$get_pvalues()
#' results_ex$get_pvalue_supports()
#'
#' # Chi-square-approximated p-values and their supports
#' results_ap  <- mcnemar_test_pv(df, exact = FALSE)
#' print(results_ap)
#' results_ap$get_pvalues()
#' results_ap$get_pvalue_supports()
#'
#' @importFrom checkmate assert_integerish qassert
#' @importFrom cli cli_abort
#' @export
mcnemar_test_pv <- function(
  x,
  alternative = "two.sided",
  exact = TRUE,
  correct = TRUE,
  simple_output = FALSE
) {
  # plausibility checks of input parameters

  # define error message for malformed x
  error_msg_x <- paste("'x' must either be a 2-by-2 matrix,",
                       "a four-element vector or a four-column matrix")

  #  if x is a vector, make it a matrix with one row
  if(is.vector(x) && is.atomic(x))
    x <- t(x)
  # if x is a data frame, make it a matrix
  if(is.data.frame(x))
    x <- as.matrix(x)
  # when x is a matrix, it must satisfy some conditions
  if(is.matrix(x)) {
    # stop immediately, if dimensions are violated
    if(any(dim(x) != 2) && ncol(x) != 4 && nrow(x) != 4)
      cli_abort(error_msg_x)
    # check if all values are non-negative and close to integer
    assert_integerish(x, lower = 0, min.len = 4)
    # coerce to integer
    x <- round(x)
    mode(x) <- "integer"
    # 2-by-2 matrices are transformed to single-row matrix
    if(all(dim(x) == 2)) {
      x <- matrix(as.vector(x), 1, 4,
        dimnames = list(NULL,
          make.names(paste(rep(colnames(x), rep(2, 2)), rownames(x)))
        )
      )
    }
  } else cli_abort(error_msg_x)

  # lengths
  len_x <- nrow(x)
  len_a <- length(alternative)
  len_g <- max(len_x, len_a)

  # check alternatives
  for(i in seq_len(len_a)){
    alternative[i] <- match.arg(
      tolower(alternative[i]),
      c("two.sided", "less", "greater")
    )
  }

  # enlarge input parameters if necessary
  if(len_x < len_g)
    x <- matrix(rep_len(as.vector(t(x)), 4 * len_g), len_g, 4, TRUE)
  if(len_a < len_g) alternative <- rep_len(alternative, len_g)

  # check exactness and continuity correction arguments
  qassert(exact, "B1")
  if(!exact) qassert(correct, "B1")

  # check output type argument
  qassert(simple_output, "B1")

  # test parameters
  b <- x[, 2]
  n <- b + x[, 3]
  p <- rep(0.5, len_g)

  # compute test results
  #res <- binom_test_pv(b, n, 0.5, alternative, "central", exact, correct, simple_output)
  res <- binom_test_int(b, n, p, alternative, exact, correct, simple_output)

  # create output object
  out <- if(!simple_output) {
    dnames <- sapply(match.call(), deparse1)
    if(is.null(colnames(x)))
      colnames(x) <- paste0("x", c("[1, 1]", "[2, 1]", "[1, 2]", "[2, 2]"))

    DiscreteTestResults$new(
      test_name = "McNemar's test",
      inputs = c(
        list(
          observations = as.data.frame(x),
          parameters = NULL,
          nullvalues = data.frame(
            `counter-diagonal values proportion` = p,
            check.names = FALSE
          ),
          computation = Filter(
            function(df) !all(is.na(df)),
            data.frame(
              alternative = alternative,
              exact = exact,
              distribution = ifelse(exact, "binomial", "normal"),
              #distribution.size = if(exact) n else NA_integer_,
              #distribution.mean = if(exact) NA_real_ else n * 0.5,
              #distribution.sd = if(exact) NA_real_ else sqrt(n * 0.25),
              `continuity correction` = if(exact) NA else correct,
              `counter-diagonal sum` = n,
              check.names = FALSE
            )
          )
        )
      ),
      statistics = NULL,
      p_values = res$pv,
      pvalue_supports = res$sup,
      support_indices = res$idx,
      data_name = dnames["x"]
    )
  } else res

  # return results
  return(out)
}

#' @rdname mcnemar_test_pv
#' @export
#' @importFrom lifecycle deprecate_stop
mcnemar.test.pv <- function(
    x,
    alternative = "two.sided",
    exact = TRUE,
    correct = TRUE,
    simple.output = FALSE
) {
  deprecate_stop("0.2", "mcnemar.test.pv()", "mcnemar_test_pv()")
}

#' @title
#' Discrete Test Results Class
#'
#' @description
#' This is the class used by the statistical test functions of this package for
#' returning not only p-values, but also the supports of their distributions and
#' the parameters of the respective tests. Objects of this class are obtained by
#' setting the `simple.output` parameter of a test function to `FALSE` (the
#' default). All data members of this class are private to avoid inconsistencies
#' by deliberate or inadvertent changes by the user. However, the results can be
#' read by public methods.
#'
#' @examples
#' ## one-sided binomial test
#' #  parameters
#' x <- 2:4
#' n <- 5
#' p <- 0.4
#' m <- length(x)
#' #  support (same for all three tests) and p-values
#' support <- sapply(0:n, function(k) binom.test(k, n, p, "greater")$p.value)
#' pv <- support[x + 1]
#' #  DiscreteTestResults object
#' res <- DiscreteTestResults$new(
#'   # string with name of the test
#'   test_name = "Exact binomial test",
#'   # list of data frames
#'   inputs = list(
#'     observations = data.frame(
#'       `number of successes` = x,
#'       # no name check of column header to have a speaking name for 'print'
#'       check.names = FALSE
#'     ),
#'     parameters = data.frame(
#'       # parameter 'n', needs to be replicated to length of 'x'
#'       `number of trials` = rep(n, m),
#'       # no name check of column header to have a speaking name for 'print'
#'       check.names = FALSE
#'     ),
#'     nullvalues = data.frame(
#'       # here: only one null value, 'p'; needs to be replicated to length of 'x'
#'       `probability of success` = rep(p, m),
#'       # no name check of column header to have a speaking name for 'print'
#'       check.names = FALSE
#'     ),
#'     computation = data.frame(
#'       # mandatory parameter 'alternative', needs to be replicated to the length of 'x'
#'       alternative = rep("greater", m),
#'       # mandatory exactness information, replicated to the length of 'alternative'
#'       exact = rep(TRUE, m),
#'       # mandatory distribution information, replicated to the length of 'alternative'
#'       distribution = rep("binomial", m)
#'     )
#'   ),
#'   # test statistics (not needed, since observation itself is the statistic)
#'   statistics = NULL,
#'   # numerical vector of p-values
#'   p_values = pv,
#'   # list of supports (here: only one support); values must be sorted and unique
#'   pvalue_supports = list(unique(sort(support))),
#'   # list of indices that indicate which p-value/hypothesis each support belongs to
#'   support_indices = list(1:m),
#'   # name of input data variables
#'   data_name = "x, n and p"
#' )
#'
#' #  print results without supports
#' print(res)
#' #  print results with supports
#' print(res, supports = TRUE)
#'
#' @importFrom R6 R6Class
#' @importFrom checkmate qassert
#' @importFrom checkmate assert_character assert_subset
#' @importFrom checkmate assert_int assert_integerish assert_numeric
#' @importFrom checkmate assert_logical
#' @importFrom checkmate assert_data_frame assert_list
#' @importFrom cli cli_end cli_li cli_ul
#' @importFrom cli ansi_collapse cli_h1 cli_h2 cli_h3 cli_text cli_verbatim qty
#' @importFrom cli col_blue col_green col_red
#' @importFrom cli cli_abort
#' @export
DiscreteTestResults <- R6Class(
  "DiscreteTestResults",

  ## public ----

  public = list(
    #' @description
    #' Creates a new `DiscreteTestResults` object.
    #'
    #' @param test_name         single character string with the name of the
    #'                          test(s).
    #' @param inputs            named list of **exactly four named** elements
    #'                          containing the observations, test parameters and
    #'                          hypothesised null values **as data frames or**
    #'                          **lists**; the names of these list fields must
    #'                          be `observations`, `parameters`, `nullvalues`
    #'                          and `computation`. See details for further
    #'                          information about the requirements for these
    #'                          fields.
    #' @param statistics        data frame containing the tests' statistics;
    #'                          `NULL` is allowed and recommended, e.g. if the
    #'                          observed values themselves are the statistics.
    #' @param p_values          numeric vector of the p-values calculated by
    #'                          each hypothesis test.
    #' @param pvalue_supports   list of **unique** numeric vectors containing
    #'                          all p-values that are observable under the
    #'                          respective hypothesis; each value of `p_values`
    #'                          must occur in its respective p-value support.
    #' @param support_indices   list of numeric vectors containing the test
    #'                          indices that indicates to which individual
    #'                          testing scenario each unique parameter set and
    #'                          each unique support belongs.
    #' @param data_name         single character string with the name of the
    #'                          variable that contains the observed data.
    #'
    #' @details
    #' The fields of the `inputs` have the following requirements:
    #' \describe{
    #'   \item{`$observations`}{data frame or list of vectors that comprises of
    #'                          the observed data; if it is a matrix, it must be
    #'                          converted to a data frame; must not be `NULL`,
    #'                          only numerical and character values are
    #'                          allowed.}
    #'   \item{`$nullvalues`}{data frame that holds the hypothesised values
    #'                        of the tests, e.g. the rate parameters for Poisson
    #'                        tests; must not be `NULL`, only numerical values
    #'                        are allowed.}
    #'   \item{`$parameters`}{data frame that may contain additional parameters
    #'                        of each test (e.g. numbers of Bernoulli trials for
    #'                        binomial tests). Only numerical, character or
    #'                        logical values are permitted; `NULL` is allowed,
    #'                        too, e.g. if there are no additional parameters.}
    #'   \item{`$computation`}{data frame that consists of details about the
    #'                         p-value computation, e.g. if they were calculated
    #'                         exactly, the used distribution etc. It **must**
    #'                         include mandatory columns named `exact`,
    #'                         `alternative` and `distribution`. Any additional
    #'                         information may be added, like the marginals for
    #'                         Fisher's exact test etc., but only numerical,
    #'                         character or logical values are allowed.}
    #' }
    #'
    #' All data frames must have the same number of rows. Their column names are
    #' used by the `print()` method for producing text output, therefore they
    #' should be informative, i.e. short and (if necessary) non-syntactic,
    #' like e.g. `` `number of success` ``.
    #'
    #' The mandatory column `exact` of the data frame `computation` must be
    #' logical, while the values of `alternative` must be one of `"greater"`,
    #' `"less"`, `"two.sided"`, `"minlike"`, `"blaker"`, `"absdist"` or
    #' `"central"`. The `distribution` column must hold character strings that
    #' identify the distribution under the null hypothesis, e.g. `"normal"`. All
    #' the columns of this data frame are used by the `print()` method, so their
    #' names should also be informative and (if necessary) non-syntactic.
    #'
    initialize = function(
      test_name,
      inputs,
      statistics,
      p_values,
      pvalue_supports,
      support_indices,
      data_name
    ) {
      # ensure that test name is a single character string
      qassert(x = test_name, rules = "S1")

      # ensure that inputs are given as data frames or lists
      assert_list(
        x = inputs,
        types = c("data.frame", "list", "null"),
        len = 4L,
        names = "named"
      )

      # ensure that input list elements are properly named
      if(!all(
        c("observations", "parameters", "nullvalues", "computation") %in%
          names(inputs)
      ))
        cli_abort(paste(
          "Names of fields in list 'inputs' must be 'observations',",
          "'parameters', 'nullvalues' and 'computation'."
        ))

      # ensure that observations are in a proper format depending on type
      if(is.data.frame(inputs$observations)) {
        # ensure observations are in a data frame of numerical, character or
        #   logical vectors
        assert_data_frame(
          x = inputs$observations,
          types = c("numeric", "character", "logical"),
          any.missing = FALSE,
          min.cols = 1L,
          min.rows = 1L
        )
        # overall number of tests, i.e. observations that were tested
        len <- nrow(inputs$observations)
      } else
        if(is.list(inputs$observations)) {
          # ensure observations are in a list containing numerical or character
          #   vectors
          assert_list(
            x = inputs$observations,
            types = c("numeric", "character", "logical", "list"),
            any.missing = FALSE,
            min.len = 1L
          )
          # if observations are a list, they must contain sample tuples
          if(is.list(inputs$observations)) {
            # all list elements must have the same data type
            type <- unique(sapply(inputs$observations, mode))
            if(length(type) > 1L)
              cli_abort("All observations must have the same data type")
            # if type is list: multiple samples; else: single samples
            if(type == "list") {
              len <- unique(sapply(inputs$observations, length))
              if(length(len) > 1L)
                cli_abort("All sample lists must have the same length")

              type <- character(0)
              for(i in seq_along(inputs$observations))
                type <- unique(c(type, sapply(inputs$observations[[i]], mode)))
              if(length(type) > 1L)
                cli_abort("All samples must have the same data type")
            } else {
              # overall number of tests that were performed
              len <- length(inputs$observations)
            }
            if(!(type %in% c("numeric", "character", "logical")))
              cli_abort("All samples must be numeric, character or logical")
          }
        } else cli_abort("'observations' must be a list or a data frame")

      # ensure that the parameters are in a data frame with as many rows as
      #   there are observations and that it contains only numerical, character
      #   or logical vectors
      assert_data_frame(
        x = inputs$parameters,
        types = c("numeric", "character", "logical"),
        any.missing = TRUE,
        nrows = len,
        null.ok = TRUE
      )

      # ensure that all null (i.e. hypothesised) values are in a numeric data
      #   frame
      assert_data_frame(
        x = inputs$nullvalues,
        types = "numeric",
        any.missing = FALSE,
        nrows = len,
      )

      # ensure that the computation information is given in a data frame with as
      #   many rows as there are observations and that it contains only
      #   numerical, character or logical vectors
      assert_data_frame(
        x = inputs$computation,
        types = c("numeric", "character", "logical"),
        any.missing = TRUE,
        nrows = len
      )

      # ensure that alternatives exist and that they are strings
      if(exists("alternative", inputs$computation)) {
        assert_character(
          x = inputs$computation$alternative,
          any.missing = FALSE
        )
      } else cli_abort("'computation' must have an 'alternative' column")

      # ensure that all alternatives are from the 'usual' ones
      assert_subset(
        x = inputs$computation$alternative,
        choices = c("greater", "less", "two.sided", "minlike",
                    "blaker", "central", "absdist"),
        empty.ok = FALSE
      )

      # ensure that information about exactness exists and that it is logical
      if(exists("exact", inputs$computation)) {
        assert_logical(
          x = inputs$computation$exact,
          any.missing = FALSE
        )
      } else cli_abort("'computation' must have an 'exact' column")

      # ensure that information about the null distribution exists and that it
      #   is text
      if(exists("distribution", inputs$computation)) {
        assert_character(
          x = inputs$computation$distribution,
          any.missing = FALSE
        )
      } else cli_abort("'computation' must have a 'distribution' column")

      # # ensure that information about potential continuity correction exists
      # #   and that it is logical (NA allowed for exact computation)
      # if(exists("correct", inputs$computation)) {
      #   assert_logical(
      #     x = inputs$computation$correct,
      #     any.missing = TRUE
      #   )
      # } else cli_abort("'computation' must have a 'correct' column")

      # ensure that the statistics are in a data frame that contains only
      #   numerical or character vectors and that it has as many rows as there
      #   are observations (NULL is allowed if there are no statistics)
      assert_data_frame(
        x = statistics,
        types = c("numeric", "character"),
        nrows = len,
        null.ok = TRUE
      )

      # ensure that vector of p-values is numeric with probabilities in [0, 1]
      assert_numeric(
        x = p_values,
        lower = 0L,
        upper = 1L,
        any.missing = FALSE,
        len = len
      )

      # ensure that list of support values is a list
      assert_list(
        x = pvalue_supports,
        types = "numeric",
        any.missing = FALSE,
        min.len = 1L,
        max.len = len
      )

      for(i in seq_along(pvalue_supports)){
        # ensure each list item contains sorted vectors of probabilities in
        #   [0, 1]
        assert_numeric(
          x = pvalue_supports[[i]],
          lower = 0L,
          upper = 1L,
          any.missing = FALSE,
          min.len = 1L,
          sorted = TRUE
        )
      }

      # set of p-value indices for checking of correct indices in supports list
      idx_set <- seq_len(len)

      # ensure that list of support indices is a list
      assert_list(
        x = support_indices,
        types = "numeric",
        any.missing = FALSE,
        min.len = 1L
      )

      for(i in seq_along(support_indices)){
       # ensure indices are integerish vectors (and coerce them)
        support_indices[[i]] <- assert_integerish(
          x = support_indices[[i]],
          lower = 1L,
          upper = len,
          any.missing = FALSE,
          min.len = 1L,
          unique = TRUE,
          coerce = TRUE
        )

        # remove indices of current list item from check set
        idx_set <- setdiff(idx_set, support_indices[[i]])

        # ensure that each p-value is taken from its respective distribution
        if(!all(p_values[support_indices[[i]]] %in% c(0, pvalue_supports[[i]])))
          cli_abort(paste(
            "All observed p-values must occur in their respective distribution"
          ))
      }

      # ensure check set is empty
      if(length(idx_set))
        cli_abort("All support set indices must be unique")

      # ensure that data variable name is a character vector or NULL
      qassert(x = data_name, c("S+", "0"))

      # assign inputs
      private$test_name       <- test_name
      private$inputs          <- inputs
      private$statistics      <- statistics
      private$p_values        <- p_values
      private$pvalue_supports <- pvalue_supports
      private$support_indices <- support_indices
      private$data_name       <- data_name

      # add row names in input data (if present) to p-values
      names(private$p_values) <- rownames(inputs$observations)
    },

    #' @description
    #' Returns the computed p-values.
    #'
    #' @param named  single logical value that indicates whether the vector is
    #'               to be returned as a named vector (if names are present)
    #'
    #' @return
    #' A numeric vector of the p-values of all null hypotheses.
    #'
    get_pvalues = function(named = TRUE) {
      if(named)
        return(private$p_values) else
          return(as.numeric(private$p_values))
    },

    #' @description
    #' Return the list of the test inputs.
    #'
    #' @param unique   single logical value that indicates whether only unique
    #'                 combinations of parameter sets and null values are to be
    #'                 returned. If `unique = FALSE` (the default), the returned
    #'                 data frames may contain duplicate sets.
    #'
    #' @return
    #' A list of four elements. The first one contains a data frame with the
    #' observations for each tested null hypothesis, while the second is another
    #' data frame with additional parameters (if any, e.g. `n` in case of a
    #' binomial test) that were passed to the respective test's function. The
    #' third list field holds the hypothesised null values (e.g. `p` for
    #' binomial tests). The last list element contains computational details,
    #' e.g. test `alternative`s, the used `distribution` etc. If
    #' `unique = TRUE`, only unique combinations of parameters, null values and
    #' computation specifics are returned, but observations remain unchanged
    #' (i.e. they are never unique).
    #'
    get_inputs = function(unique = FALSE) {
      lst <- private$inputs[c(
        "observations", "parameters", "nullvalues", "computation"
      )]
      if(unique) {
        np  <- ncol(lst$parameters)
        nn  <- ncol(lst$nullvalues)
        nc  <- ncol(lst$computation)
        id1 <- seq_len(np)
        id2 <- seq_len(np + nn)
        id3 <- seq_len(np + nn + nc)
        df  <- unique(cbind(lst$parameters, lst$nullvalues, lst$computation))
        lst$parameters  <- df[id1]
        lst$nullvalues  <- df[setdiff(id2, id1)]
        lst$computation <- df[setdiff(id3, id2)]
      }
      return(lst)
    },

    #' @description
    #' Returns the test statistics.
    #'
    #' @return
    #' A numeric `data.frame` with one column containing the test statistics.
    #'
    get_statistics = function() {
      return(private$statistics)
    },

    #' @description
    #' Returns the *p*-value supports, i.e. all observable p-values under the
    #' respective null hypothesis of each test.
    #'
    #' @param unique   single logical value that indicates whether only unique
    #'                 p-value supports are to be returned. If `unique = FALSE`
    #'                 (the default), the returned supports may be duplicated.
    #'
    #' @return
    #' A list of numeric vectors containing the supports of the p-value null
    #' distributions.
    #'
    get_pvalue_supports = function(unique = FALSE) {
      if(!unique) {
        idx_scns <- unlist(private$support_indices)
        idx_lens <- sapply(private$support_indices, length)
        return(rep(private$pvalue_supports, idx_lens)[order(idx_scns)])
      } else return(private$pvalue_supports)
    },

    #' @description
    #' Returns the indices that indicate to which tested null hypothesis each
    #' unique support belongs.
    #'
    #' @return
    #' A list of numeric vectors. Each one contains the indices of the null
    #' hypotheses to which the respective support and/or unique parameter set
    #' belongs.
    #'
    get_support_indices = function(){
      return(private$support_indices)
    },

    #' @description
    #' Prints the computed p-values.
    #'
    #' @param inputs            single logical value that indicates if the
    #'                          input values (i.e. observations, statistics and
    #'                          parameters) are to be printed; defaults to
    #'                          `TRUE`.
    #' @param pvalue_details    single logical value that indicates if details
    #'                          about the p-value computation are to be printed;
    #'                          defaults to `TRUE`.
    #' @param supports          single logical value that indicates if the
    #'                          p-value supports are to be printed; defaults to
    #'                          `FALSE`.
    #' @param test_idx          integer vector giving the indices of the tests
    #'                          whose results are to be printed; if `NULL` (the
    #'                          default), results of every test up to the index
    #'                          specified by `limit` (see below) are printed.
    #' @param limit             single integer that indicates the maximum number
    #'                          of test results to be printed; if `limit = 0`,
    #'                          results of every test are printed; ignored if
    #'                          `test_idx` is not set to `NULL`
    #' @param ...               further arguments passed to
    #'                          [`print.default()`][base::print.default()].
    #'
    #' @return
    #' Prints a summary of the tested null hypotheses. The object itself is
    #' invisibly returned.
    #'
    print = function(
      inputs = TRUE,
      pvalue_details = TRUE,
      supports = FALSE,
      test_idx = NULL,
      limit = 10,
      ...
    ) {
      # check arguments for plausibility
      qassert(inputs, "B1")
      qassert(pvalue_details, "B1")
      qassert(supports, "B1")
      if(!is.null(test_idx) && !all(is.na(test_idx)))
        qassert(test_idx, "x+[0,)")
      qassert(limit, c("0", "x1", "X1[0,)"))
      assert_int(x = limit, na.ok = TRUE, lower = 0, null.ok = TRUE)

      # number of tests
      n <- length(private$p_values)

      # designations of tests (if present)
      names <- if(is.data.frame(private$inputs$observations))
        rownames(private$inputs$observations) else
          names(private$inputs$observations)

      if(inputs || pvalue_details || supports){
        pars      <- private$inputs
        idx_alt   <- which(names(pars$computation) == "alternative")
        idx_ex    <- which(names(pars$computation) == "exact")
        idx_dist  <- which(names(pars$computation) == "distribution")
        idx_add   <- which(startsWith(names(pars$computation), "distribution."))
        #idx_cont <- which(names(pars$computation) == "correct")
        idx       <- c(idx_alt, idx_ex, idx_dist, idx_add)#, idx_cont)
      }
      if(supports)
        supp <- self$get_pvalue_supports(unique = FALSE)

      cli_h1(private$test_name)
      cli_text("\n")
      cli_ul(id = "results")
      cli_li("Data: {.field {private$data_name}}")
      if(n > 1) cli_li(paste0("Number of tests: {.field {n}}"))

      # determine number of tests to print
      if(is.null(test_idx)) {
        limit <- ifelse(is.null(limit) || is.na(limit), n, limit)
        nums <- seq_len(min(limit, n))
      } else nums <- unique(pmin(na.omit(test_idx), n))

      # number of results to be printed
      len <- length(nums)

      for(i in nums) {
        if(n > 1) {
          cli_end("results")
          cli_text("\n")
          heading <- paste("Results of test", i)
          #cat("Test", i)
          if(!is.null(names) && names[i] != i) {
            # remove line breaks from name
            name <- gsub("[\r\n]", "_", names[i])
            # how many characters can be printed to console (minus reserved)
            chars_tst_line <- getOption("width") - nchar(heading) - 6L
            # how many characters can be used by tag (= row name)
            chars_name_max <- chars_tst_line - 3L
            # length of current row name
            chars_name <- nchar(name)
            # print row name in brackets behind test number
            heading <- paste0(
              heading,
              " (",
              ifelse(
                chars_name > chars_name_max,
                paste(substr(name, 1, chars_name_max - 3L), "..."),
                name
              ),
              ")"
            )
          }
          cli_h2(heading)
          cli_ul(id = "results")
        }

        if(inputs) {
          if(is.data.frame(pars$observations)) {
            # fixed number of values (depending on test, e.g. 4 for Fisher's)
            cli_li("{qty(names(pars$observations))} Observation{?s}:")
            # start sub-list
            cli_ul(id = "obs")
            # print vector of strings with names of components and their values
            cli_li(paste0(
              "{.field ",
              names(pars$observations),
              "}: {.val {as.numeric(format(",
              pars$observations[i, ],
              "))}}"
            ))
          } else {
            # observations consist of one or more samples
            cli_li("Observations:")
            # start sub-list
            cli_ul(id = "obs")
            # number of samples per observation
            number_samples <- ifelse(
              is.list(pars$observations[[1]]),
              length(pars$observations),
              1
            )
            # print information about sample(s)
            for(s in seq_len(number_samples)) {
              # current sample
              sample_obs <- if(number_samples > 1)
                pars$observations[[s]][[i]] else
                  pars$observations[[i]]
              # number of observations
              len_obs <- length(sample_obs)
              # number of sample (if more than one) and (some) values
              cli_li(paste(
                "{.field Sample} {ifelse(number_samples > 1,",
                "paste0(col_green(s), ':'), 'of')} {.val {len_obs}}",
                "{mode(sample_obs)} {qty(len_obs)} value{?s}",
                "({ansi_collapse(as.numeric(format(sample_obs)), trunc = 5)})"
              ))
            }
          }
          # end sub-list
          cli_end("obs")

          # print statistics (if any)
          if(
            !is.null(private$statistics) &&
            !all(is.na(private$statistics[i, ]))
          ) {
            cli_li("Statistics:")
            # start sub-list
            cli_ul(id = "statistics")
            # print statistics
            for(j in seq_len(ncol(private$statistics))) {
              stat_val <- private$statistics[i, j]
              if(!is.na(stat_val))
                cli_li(paste(
                  "{.field {names(private$statistics)[j]}}:",
                  if(is.numeric(stat_val))
                    "{.val {as.numeric(format(stat_val))}}" else
                      "{col_blue(stat_val)}"

                ))
            }
            # end sub-list
            cli_end("statistics")
          }

          # print parameters
          if(!is.null(pars$parameters)) {
            idx_par_valid <- which(!is.na(pars$parameters[i, ]))
            cli_li("Parameters:")
            cli_ul(id = "parameters")
            for(j in idx_par_valid) {
              par_val <- pars$parameters[i, j]
              cli_li(paste(
                "{.field {names(pars$parameters)[j]}}:",
                if(is.numeric(par_val))
                  "{.val {as.numeric(format(par_val))}}" else
                    if(is.logical(par_val))
                      paste0(
                        "{col_blue('", ifelse(par_val, "yes", "no"), "')}"
                      ) else
                        "{.val {par_val}}"
              ))
            }
            cli_end("parameters")
          }

          # print null and alternative hypotheses
          if(pars$computation$alternative[i] == "greater") {
            alt <- "greater than"
            null <- "less than or equal to"
          } else if(pars$computation$alternative[i] == "less") {
            alt <- "less than"
            null <- "greater than or equal to"
          } else {
            alt <- "not equal to"
            null <- "equal to"
          }

          cli_li("Hypotheses:")
          cli_ul(id = "hypotheses")
          cli_li(paste(
            "null: true {names(pars$nullvalues)[1]} is {null}",
            "{.val {pars$nullvalues[[1]][i]}}"
          ))
          cli_li(paste(
            "alternative: true {names(pars$nullvalues)[1]} is {alt}",
            "{.val {pars$nullvalues[[1]][i]}}"
          ))
          cli_end("hypotheses")
        }

        # print p-values
        if(pvalue_details) {
          # print p-values with computation details
          meth <- switch(
            EXPR    = pars$computation$alternative[i],
            minlike = "two-sided, via minimum likelihood",
            blaker  = "two-sided, via combined tails",
            absdist = "two-sided, via absolute distance from mean",
            central = "two-sided, via minimum tail doubling",
            less    = "one-sided, lower tail",
            greater = "one-sided, upper tail",
            "two-sided"
          )
          cli_li("p-Value:")
          cli_ul(id = "pv")
          cli_li("computation: {col_blue(meth)}")

          cli_ul(id = "computation")
          cli_li(paste(
           "exact: {ifelse(pars$computation$exact[i],",
           "col_green('yes'), col_red('no'))}"
          ))
          cli_li("distribution: {col_blue(pars$computation$distribution[i])}")
          if(length(idx_add) > 0) {
            cli_ul(id = "dist_pars")
            for(j in idx_add) {
              par_name <- sub("distribution.", "", names(pars$computation)[j])
              par_val <- pars$computation[i, j]
              if(!is.na(par_val))
                cli_li(paste(
                  "{.field {par_name}}:",
                  if(is.numeric(par_val))
                    "{.val {as.numeric(format(par_val))}}" else
                      "{.val {par_val}}"
                ))
            }
            cli_end("dist_pars")
          }

          len_comp <- ncol(pars$computation)
          if(len_comp > length(idx)) {
            idx_comp <- setdiff(seq_len(len_comp), idx)
            if(!all(is.na(pars$computation[i, idx_comp]))) {
              for(j in idx_comp) {
                par_val <- pars$computation[i, j]
                if(!is.na(par_val))
                  cli_li(paste(
                    "{names(pars$computation)[j]}:",
                    if(is.numeric(par_val))
                      ifelse(
                        is.integer(par_val),
                        paste0(
                          "{col_blue(format(",
                            "par_val, format = 'fg', big.mark = ','",
                          "))}"
                        ),
                        "{.val {as.numeric(format(par_val))}}"
                      ) else if(is.logical(par_val))
                          ifelse(
                            par_val,
                            "{col_green('yes')}",
                            "{col_red('no')}"
                          ) else
                            "{col_blue(par_val)}"
                  ))
              }
            }
          }
          cli_end("computation")
          cli_li("result: {.val {as.numeric(format(private$p_values[i]))}}")
        } else {
          # print p-values only
          if(supports) {
            cli_li("p-Value:")
            cli_ul(id = "pv")
            cli_li("result: {.val {as.numeric(format(private$p_values[i]))}}")
          } else
            cli_li("p-Value: {.val {as.numeric(format(private$p_values[i]))}}")
        }

        if(supports) {
          cli_li(paste(
            "support: {.val {length(supp[[i]])}} observable p-value{?s}",
            "({ansi_collapse(as.numeric(format(supp[[i]])), trunc = 5)})"
          ))
        }

        if(pvalue_details || supports) cli_end("pv")
      }
      cli_end("results")

      if(is.null(test_idx) && limit > 0 && limit < n) {
        cli_text("\n")
        cli_text("\n")

        cli_text(
          paste(
            "[ print limit reached -- {n - limit} result{?s} omitted --",
            "use print parameter 'limit' for more results ]"
          )
        )
      }
      if(!is.null(test_idx) && n > 1) {
        cli_text("\n")
        cli_text("\n")

        cli_text("[ {length(nums)} out of {n} results printed ]")
      }

      invisible(self)
    }
  ),

  ## private ----

  private = list(
    ## @field test_name          single character string with the name of the
    ##                           test(s), e.g. "Fisher's Exact Test"
    test_name = character(),

    ## @field inputs             named list containing the observations and the
    ##                           tests' parameters
    inputs = list(),

    ## @field statistics         data frame containing the tests' statistics
    statistics = data.frame(),

    ## @field p_values           numeric vector of the p-values calculated for
    ##                           each discrete test setting
    p_values = numeric(),

    ## @field pvalue_supports    list of UNIQUE numeric vectors containing all
    ##                           p-values that can be observed in the respective
    ##                           discrete test setting
    pvalue_supports = list(),

    ## @field support_indices    list of numeric vectors containing the test
    ##                           indices that indicate to which individual test
    ##                           setting each unique support belongs
    support_indices = list(),

    ## @field data_name          single character string with the name of the
    ##                           variable that contains the observed data
    data_name = character()
  )
)

#' @name wilcox_test_pv
#'
#' @title
#' Wilcoxon's signed-rank test
#'
#' @description
#' `wilcox_test_pv()` performs an exact or approximate Wilcoxon signed-rank test
#' about the location of a population on a single sample or the differences
#' between two paired groups when the data is not necessarily normally
#' distributed. In contrast to [`stats::wilcox.test()`], it is vectorised and
#' only calculates *p*-values. Furthermore, it is capable of returning the
#' discrete *p*-value supports, i.e. all observable *p*-values under a null
#' hypothesis. Multiple tests can be evaluated simultaneously.
#'
#' @param x               numerical vector forming the sample to be tested or a
#'                        list of numerical vectors for multiple tests.
#' @param y               numerical vector forming the second sample to be
#'                        tested or a list of numerical vectors for multiple
#'                        tests; if `y = NULL` (the default), the one-sample
#'                        version is performed; for two-sample tests, all sample
#'                        pairs must have the same length.
#' @param mu              numerical vector or single number of hypothesised
#'                        location(s) for one-sample tests or location shift(s)
#'                        for two-sample tests.
#' @param zero_method     character vector or single string specifying how zero
#'                        differences are handled; must either be `"pratt"` (the
#'                        default) or `"wilcoxon"`. With `"pratt"`, zero
#'                        differences are included when computing ranks, but
#'                        excluded from the test statistic; with `"wilcoxon"`,
#'                        zero differences are discarded entirely before ranking
#'                        (the original Wilcoxon approach).
#'
#' @template param
#' @templateVar alternative TRUE
#' @templateVar exact_w TRUE
#' @templateVar correct_w TRUE
#' @templateVar simple_output TRUE
#' @templateVar digits_rank TRUE
#'
#' @details
#' The parameters `x`, `mu`, `alternative` and `zero_method` are vectorised. If
#' `x` is a list, they are replicated automatically to have the same lengths. In
#'  case `x` is not a list, it is added to one, which is then replicated to the
#' appropriate length. This allows multiple hypotheses to be tested
#' simultaneously.
#'
#' By setting `exact = NULL`, exact computation is performed only if the sample
#' size in a test setting is smaller than or equal to 200. Otherwise,
#' \eqn{p}-values are computed by normal approximation.
#'
#' The used test statistics `W` is also known as \eqn{T+} and is defined as the
#' sum of ranks of all strictly positive values of the sample `x`.
#'
#' If `digits_rank = Inf` (the default), [`rank()`][`base::rank()`] is used to
#' compute ranks for the tests statistics instead of
#' [`rank`][`base::rank()`]([`signif(., digits_rank)`][`base::signif()`])
#'
#' @template return
#'
#' @seealso
#' [`stats::wilcox.test()`], [`mann_whitney_test_pv()`]
#'
#' @references
#' Hollander, M. & Wolfe, D. (1973). *Nonparametric Statistical Methods*. Third
#'   Edition. New York: Wiley. pp. 40-55. \doi{10.1002/9781119196037}
#'
#' @examples
#' # Constructing
#' set.seed(1)
#' r1 <- rnorm(200)
#' r2 <- rnorm(200, 1)
#' r3 <- rnorm(200, 2)
#'
#' ## One-sample tests
#' #  Exact two-sided p-values and their supports
#' results_ex_1s <- wilcox_test_pv(r1)
#' print(results_ex_1s)
#' results_ex_1s$get_pvalues()
#' results_ex_1s$get_pvalue_supports()
#'
#' #  Multiple normal-approximated one-sided tests ("greater")
#' results_ap_1s <- wilcox_test_pv(list(r1, r2), alternative = "greater", exact = FALSE)
#' print(results_ap_1s)
#' results_ap_1s$get_pvalues()
#' results_ap_1s$get_pvalue_supports()
#'
#' ## Two-sample-tests
#' #  Normal-approximated one-sided p-values ("less") and their supports
#' results_ap_2s <- wilcox_test_pv(r1, r2, alternative = "less", exact = FALSE)
#' print(results_ap_2s)
#' results_ap_2s$get_pvalues()
#' results_ap_2s$get_pvalue_supports()
#'
#' #  Multiple exact two-sided tests ("greater")
#' results_ex_2s <- wilcox_test_pv(list(r1, r3), r2)
#' print(results_ex_2s)
#' results_ex_2s$get_pvalues()
#' results_ex_2s$get_pvalue_supports()
#'
#' @importFrom checkmate qassert qassertr
#' @importFrom cli cli_abort cli_warn
#' @export
wilcox_test_pv <- function(
  x,
  y = NULL,
  mu = 0,
  alternative = "two.sided",
  exact = NULL,
  correct = TRUE,
  digits_rank = Inf,
  zero_method = "pratt",
  simple_output = FALSE
) {
  # plausibility checks of input parameters
  qassert(x, c("N+", "L+"))
  if(!is.list(x)) x <- list(x) else qassertr(x, "N+")
  len_x <- length(x)

  one_sample <- is.null(y)
  if(!one_sample) {
    qassert(y, c("N+", "L+"))
    if(!is.list(y)) y <- list(y) else qassertr(y, "N+")
  }
  len_y <- length(y)

  qassert(mu, "N+()")
  len_m <- length(mu)

  len_a <- length(alternative)
  for(i in seq_len(len_a)){
    alternative[i] <- match.arg(
      tolower(alternative[i]),
      c("two.sided", "less", "greater")
    )
  }

  qassert(exact,   c("B1", "0"))

  qassert(correct, c("B1", "X1[0, 3]"))
  if(!is.logical(correct) && (is.null(exact) || (!is.null(exact) && !exact))) {
    edgeworth <- round(correct)
    correct   <- TRUE
  } else {
    if(!is.null(exact) && exact && is.numeric(correct)) correct <- TRUE
    edgeworth <- 0
  }

  qassert(digits_rank, "N1")

  len_z <- length(zero_method)
  for(i in seq_len(len_z)){
    zero_method[i] <- match.arg(
      tolower(zero_method[i]),
      c("pratt", "wilcoxon")
    )
  }

  qassert(simple_output, "B1")

  # replicate inputs to same length
  len_g                         <- max(len_x, len_y, len_m, len_a, len_z)
  if(len_x < len_g) x           <- rep_len(x, len_g)
  if(len_m < len_g) mu          <- rep_len(mu, len_g)
  if(len_a < len_g) alternative <- rep_len(alternative, len_g)
  if(len_z < len_g) zero_method <- rep_len(zero_method, len_g)

  if(!one_sample) {
    if(len_y < len_g) y <- rep_len(y, len_g)
    res_obs <- list(x, y)

    for(i in seq_len(len_g)) {
      # check if lengths of all sample pairs are equal; stop if they are not
      if(length(x[[i]]) != length(y[[i]]))
        cli_abort('All paired samples must have the same length')
      # compute differences
      x[[i]] <- x[[i]] - y[[i]]
    }
  }

  # compute ranks and lengths
  n     <- integer(len_g)
  m     <- integer(len_g)
  W     <- numeric(len_g)
  means <- numeric(len_g)
  sds   <- numeric(len_g)
  ties  <- logical(len_g)
  zeros <- numeric(len_g)
  stats <- vector("list", len_g)
  for(i in seq_len(len_g)) {
    y <- x[[i]] - mu[i]
    m[i] <- length(y)

    is_zero  <- (y == 0)
    zeros[i] <- sum(is_zero)
    if(zero_method[i] == "wilcoxon") {
      y         <- y[!is_zero]
      is_zero   <- rep(FALSE, length(y))
      n[i]      <- m[i] - zeros[i]
    } else n[i] <- m[i]

    ranks <- if(is.finite(digits_rank))
      rank(abs(signif(y, digits_rank))) else
        rank(abs(y))

    ranks   <- ranks[!is_zero]
    pos_y   <- y[!is_zero] > 0
    ties[i] <- m[i] - zeros[i] != length(unique(ranks))

    # test statistics
    W[i] <- sum(ranks[pos_y])
    if(ties[i] || (zeros[i] & zero_method[i] == "pratt") ||
       is.null(exact) && n[i] > 200 || !is.null(exact) && !exact)
      stats[[i]] <- as.integer(round(2 * ranks))

    # parameters for normal approximation
    means[i] <- n[i] * (n[i] + 1) / 4
    if(zeros[i] && zero_method[i] == "pratt")
      means[i] <- means[i] - zeros[i] * (zeros[i] + 1) / 4
    if(is.null(exact) && n[i] > 200 || !is.null(exact) && !exact) {
      sds[i] <- means[i] * (2 * n[i] + 1) / 6
      # correct for zeros depending on `zero_method`
      if(zeros[i] && zero_method[i] == "pratt") {
        sds[i] <- sds[i] - zeros[i] * (zeros[i] + 1) * (2 * zeros[i] + 1) / 24
      }
      # correct for possible ties
      if(ties[i]) {
        t <- table(ranks)
        sds[i] <- sds[i] - sum(t^3 - t) / 48
      }
    }
  }
  sds <- sqrt(sds)

  ex <- if(is.null(exact)) n <= 200 else rep(exact, len_g)
  ew <- edgeworth & !ex
  ew[ex] <- NA

  # compute Edgeworth coefficients for normal approximations, if desired
  idx_ew <- which(ew)
  if(length(idx_ew)) {
    ew_coefs <- matrix(NA_real_, len_g, edgeworth)
    for(i in idx_ew) {
      if(edgeworth >= 1) {
        w <- stats[[i]]/2
        s <- sqrt(sum(w^2) / 4)
        ew_coefs[i, 1] <- -sum(w^4) / s^4 / 192
      }
      if(edgeworth >= 2)
        ew_coefs[i, 2] <- sum(w^6) / s^6 / 2880
      if(edgeworth == 3)
        ew_coefs[i, 3] <- ew_coefs[i, 1]^2 / 2
    }
  }

  # determine unique parameter sets
  params    <- data.frame(
    alternative, n, means, sds, ew, ties, zeros, zero_method
  )
  params_ex <- unique(
    subset(params, ex & !ties & (!zeros | zero_method == "wilcoxon"), 1:2)
  )
  params_tz <- subset(params, ex & (ties | zeros & zero_method == "pratt"))
  params_ap <- unique(subset(params, !ex, -2))
  idx_ex    <- as.numeric(rownames(params_ex))
  idx_tz    <- as.numeric(rownames(params_tz))
  idx_ap    <- as.numeric(rownames(params_ap))
  rows      <- c(idx_ex, idx_tz, idx_ap)
  params_u  <- params[rows, ]
  if(any(ew, na.rm = TRUE)) ew_coefs_u <- ew_coefs[rows, , drop = FALSE]

  len_ex <- length(idx_ex)
  len_tz <- length(idx_tz)
  len_ap <- length(idx_ap)
  idx_ex <- seq_len(len_ex)
  idx_ap <- len_ex + len_tz + seq_len(len_ap)
  len_u  <- len_ex + len_tz + len_ap

  alts_u      <- params_u$alternative
  n_u         <- params_u$n
  mean_u      <- params_u$means
  sd_u        <- params_u$sds
  ew_u        <- params_u$ew
  ties_u      <- params_u$ties
  zeros_u     <- params_u$zeros
  zero_meth_u <- params_u$zero_method

  # prepare output
  res <- numeric(len_g)
  if(!simple_output) {
    supports <- vector("list", len_u)
    indices  <- vector("list", len_u)
  }

  # pre-compute exact distributions (if any)
  sizes_ex <- unique(n_u[idx_ex])
  d <- generate_sign_rank_probs(sizes_ex)

  # begin exact computations (if any)
  for(i in idx_ex) {
    idx_supp <- which(
      alts_u[i] == alternative & n_u[i] == n &
        ex & !ties & (!zeros | zero_method == "wilcoxon")
    )

    idx_d <- which(sizes_ex == n_u[i])
    d_i <- numerical_adjust(d[[idx_d]])

    if(simple_output) {
      # compute p-values directly
      res[idx_supp] <- switch(
        EXPR = alts_u[i],
        less = p_from_d(W[idx_supp], d_i),
        greater = p_from_d(W[idx_supp] - 1, d_i, FALSE),
        two.sided = {
          idx_l <- which(W[idx_supp] < mean_u[i])
          idx_u <- which(W[idx_supp] >= mean_u[i])
          pv <- numeric(length(idx_supp))
          if(length(idx_l))
            pv[idx_l] <- p_from_d(W[idx_supp][idx_l], d_i)
          if(length(idx_u))
            pv[idx_u] <- p_from_d(W[idx_supp][idx_u] - 1, d_i, FALSE)
          pmin(1, 2 * pv)
        }
      )
    } else {
      # compute p-value support
      pv_supp <- support_exact(
        alternative = alts_u[i],
        probs = d_i,
        expectations = abs(seq_along(d) - 1 - mean_u[i])
      )

      # store results and support
      res[idx_supp] <- pv_supp[W[idx_supp] + 1]
      supports[[i]] <- unique(sort(pv_supp))
      indices[[i]]  <- idx_supp
    }
  }

  # begin exact computations for settings with ties or zeros (if any)
  idx_out <- len_ex + 1
  for(i in idx_tz) {
    # compute exact distribution with ties
    dist <- numerical_adjust(sign_rank_probs_ties_int(stats[[i]]))
    R <- 2 * W[i]

    if(simple_output) {
      # compute p-values directly
      res[i] <- switch(
        EXPR = alternative[i],
        less = p_from_d(R, dist),
        greater = p_from_d(R - 1, dist, FALSE),
        two.sided = pmin(1, 2 * p_from_d(
          R - W[i] > means[i], dist, W[i] <= means[i]
        ))
      )
    } else {
      # compute p-value support
      pv_supp <- support_exact(
        alternative = alternative[i],
        probs = dist
      )

      # store results and support
      res[i]              <- pv_supp[R + 1]
      supports[[idx_out]] <- unique(sort(pv_supp))
      indices[[idx_out]]  <- i
      idx_out             <- idx_out + 1
    }
  }

  # begin approximation computations (if any)
  for(i in idx_ap) {
    idx_supp <- which(
      alts_u[i] == alternative &
      !ex &
      ties_u[i] == ties &
      zeros_u[i] == zeros &
      zero_meth_u[i] == zero_method &
      mean_u[i] == means &
      sd_u[i] == sds
    )

    ew_ok <- !is.na(ew_u[i]) && ew_u[i]
    e <- if(ew_ok) ew_coefs_u[i, ]

    if(simple_output) {
      res[idx_supp] <- switch(
        EXPR = alts_u[i],
        less = pnorm_MW_edgeworth(
          W[idx_supp], mean_u[i], sd_u[i], TRUE, correct, e
        ),
        greater = pnorm_MW_edgeworth(
          W[idx_supp], mean_u[i], sd_u[i], FALSE, correct, e
        ),
        two.sided = pmin(1, 2 * if(ew_ok)
          pmin(
            pnorm_MW_edgeworth(
              W[idx_supp], mean_u[i], sd_u[i], TRUE, correct, e
            ),
            pnorm_MW_edgeworth(
              W[idx_supp], mean_u[i], sd_u[i], FALSE, correct, e
            )
          ) else pmin(1,
            pnorm(-abs(W[idx_supp] - mean_u[i]), -correct * 0.5, sd_u[i])
          )
        )
      )
    } else {
      # compute p-value support
      z <- if(!any(ties[idx_supp])) 0L:(n_u[i] * (n_u[i] + 1L)) else
        0L:(n_u[i] * (n_u[i] + 1L)) / 2
      pv_supp <- switch(
        EXPR = alts_u[i],
        less = pnorm_MW_edgeworth(z, mean_u[i], sd_u[i], TRUE, correct, e),
        greater = pnorm_MW_edgeworth(z, mean_u[i], sd_u[i], FALSE, correct, e),
        two.sided = pmin(1, 2 * if(ew_ok)
          pmin(
            pnorm_MW_edgeworth(z, mean_u[i], sd_u[i], TRUE, correct, e),
            pnorm_MW_edgeworth(z, mean_u[i], sd_u[i], FALSE, correct, e)
          ) else pnorm(-abs(z - mean_u[i]), -correct * 0.5, sd_u[i])
        )
      )

      # store results and support
      idx_stat <- 1 + if(!any(ties[idx_supp]))
        W[idx_supp] else round(2 * W[idx_supp])
      res[idx_supp] <- pv_supp[idx_stat]
      if(!simple_output) {
        supports[[i]] <- unique(sort(pv_supp))
        indices[[i]]  <- idx_supp
      }
    }
  }

  # remove empty supports and indices
  if(!simple_output) {
    ord <- order(sapply(indices, "[", 1))
    supports <- supports[ord]
    indices  <- indices[ord]
  }

  # create output object
  out <- if(!simple_output) {
    arg_names  <- sapply(match.call(), deparse1)

    DiscreteTestResults$new(
      test_name = "Wilcoxon signed-rank test",
      inputs = list(
        observations = if(one_sample) x else res_obs,
        parameters = NULL,
        nullvalues = if(one_sample) data.frame(location = mu) else
          data.frame(`location shift` = mu, check.names = FALSE),
        computation = Filter(
          function(df) !all(is.na(df)),
          data.frame(
            alternative = alternative,
            exact = ex,
            distribution = ifelse(
              ex,
              paste0(
                "Wilcoxon's signed-rank",
                ifelse(ties | zeros & zero_method == "pratt", " (", ""),
                ifelse(ties, "tie-", ""),
                ifelse(ties & zeros & zero_method == "pratt", " and ", ""),
                ifelse(zeros & zero_method == "pratt", "zero-", ""),
                ifelse(ties | zeros & zero_method == "pratt", "adjusted)", "")
              ),
              "normal"
            ),
            #distribution.mean = ifelse(!ex, means, NA_real_),
            #distribution.sd = ifelse(!ex, sds, NA_real_),
            `continuity correction` = ifelse(ex, NA, correct),
            `Edgeworth expansion` = ew,
            `Edgeworth series terms` = ifelse(ew, ew * edgeworth, NA),
            `sample size` = m,
            zeros = ifelse(zeros, zeros, FALSE),
            `zero handling` = ifelse(
              zero_method == "pratt", "Pratt", "Wilcoxon"
            ),
            `(non-zero) ties` = ties,
            check.names = FALSE
          )
        )
      ),
      statistics = data.frame(W),
      p_values = res,
      pvalue_supports = supports,
      support_indices = indices,
      data_name = arg_names[if(one_sample) "x" else c("x", "y")]
    )
  } else res

  # return results
  return(out)
}

#' @name binom_test_pv
#'
#' @title
#' Binomial Tests
#'
#' @description
#' `binom_test_pv()` performs an exact or approximate binomial test about the
#' probability of success in a Bernoulli experiment. In contrast to
#' [`stats::binom.test()`], it is vectorised, only calculates *p*-values and
#' offers a normal approximation of their computation. Furthermore, it is
#' capable of returning the discrete *p*-value supports, i.e. all observable
#' *p*-values under a null hypothesis. Multiple tests can be evaluated
#' simultaneously. In two-sided tests, several procedures of obtaining the
#' respective *p*-values are implemented.
#'
#' **Note**: Please do not use the older `binom.test.pv()` anymore! It is now
#' defunct and will be removed in a future version.\cr
#' `r lifecycle::badge('superseded')`
#'
#' @param x   integer vector giving the number of successes.
#' @param n   integer vector giving the number of trials.
#' @param p   numerical vector of hypothesised probabilities of success.
#'
#' @template param
#' @templateVar alternative TRUE
#' @templateVar ts.method TRUE
#' @templateVar exact TRUE
#' @templateVar correct TRUE
#' @templateVar simple.output TRUE
#'
#' @details
#' The parameters `x`, `n`, `p` and `alternative` are vectorised. They are
#' replicated automatically to have the same lengths. This allows multiple
#' hypotheses to be tested simultaneously.
#'
#' If `p = NULL`, it is tested if the probability of success is 0.5 with
#' the alternative being specified by `alternative`.
#'
#' @template details_two_sided
#'
#' @template return
#'
#' @seealso
#' [`stats::binom.test()`]
#'
#' @references
#' Agresti, A. (2002). *Categorical data analysis*. Second Edition. New York:
#'   John Wiley & Sons. pp. 14-15. \doi{10.1002/0471249688}
#'
#' Blaker, H. (2000). Confidence curves and improved exact confidence intervals
#'   for discrete distributions. *Canadian Journal of Statistics*,
#'   **28**(4), pp. 783-798. \doi{10.2307/3315916}
#'
#' Hirji, K. F. (2006). *Exact analysis of discrete data*. New York: Chapman
#'   and Hall/CRC. pp. 55-83. \doi{10.1201/9781420036190}
#'
#' @examples
#' # Constructing
#' k <- c(4, 2, 2, 14, 6, 9, 4, 0, 1)
#' n <- c(18, 12, 10)
#' p <- c(0.5, 0.2, 0.3)
#'
#' # Exact two-sided p-values ("blaker") and their supports
#' results_ex  <- binom_test_pv(k, n, p, ts_method = "blaker")
#' print(results_ex)
#' results_ex$get_pvalues()
#' results_ex$get_pvalue_supports()
#'
#' # Normal-approximated one-sided p-values ("less") and their supports
#' results_ap  <- binom_test_pv(k, n, p, "less", exact = FALSE)
#' print(results_ap)
#' results_ap$get_pvalues()
#' results_ap$get_pvalue_supports()
#'
NULL

binom_test_int <- function(x, n, p, alternative, exact, correct, simple) {
  # determine unique parameter sets
  params <- unique(data.frame(n, p, alternative))
  n_u    <- params$n
  p_u    <- params$p
  alt_u  <- params$alternative
  len_u  <- length(n_u)

  # prepare output
  res <- numeric(length(x))
  if(!simple) {
    supports <- vector("list", len_u)
    indices  <- vector("list", len_u)
  }

  # begin computations
  for(i in seq_len(len_u)) {
    idx_supp <- which(n == n_u[i] & p == p_u[i] & alternative == alt_u[i])

    if(exact) {
      # compute p-value support
      pv_supp <- support_exact(
        alternative = alt_u[i],
        probs = generate_binom_probs(n_u[i], p_u[i]),
        expectations = if(alt_u[i] == "absdist")
          abs(0:n_u[i] - n_u[i] * p_u[i])
      )
    } else {
      if(p_u[i] == 0)
        pv_supp <- switch(
          EXPR      = alt_u[i],
          less      = c(rep(1, n_u[i] + 1)),
          greater   = c(1, rep(0, n_u[i])),
          two.sided = c(1, rep(0, n_u[i]))
        )
      else if(p_u[i] == 1)
        pv_supp <- switch(
          EXPR      = alt_u[i],
          less      = c(rep(0, n_u[i]), 1),
          greater   = rep(1, n_u[i] + 1),
          two.sided = c(rep(0, n_u[i]), 1)
        )
      else {
        # compute p-value support
        pv_supp <- support_normal(
          alternative = alt_u[i],
          x = 0:n_u[i],
          mean = n_u[i] * p_u[i],
          sd = sqrt(n_u[i] * p_u[i] * (1 - p_u[i])),
          correct = correct
        )
      }
    }

    # store results and support
    res[idx_supp] <- pv_supp[x[idx_supp] + 1]
    if(!simple) {
      supports[[i]] <- unique(sort(pv_supp))
      indices[[i]]  <- idx_supp
    }
  }

  if(simple)
    return(res) else
      return(list(pv = res, sup = supports, idx = indices))
}


#' @rdname binom_test_pv
#' @importFrom checkmate assert_integerish qassert
#' @importFrom cli cli_abort
#' @export
binom_test_pv <- function(
  x,
  n,
  p = 0.5,
  alternative = "two.sided",
  ts_method = "minlike",
  exact = TRUE,
  correct = TRUE,
  simple_output = FALSE
) {
  # plausibility checks of input parameters
  qassert(x, "A+")
  x <- assert_integerish(x, lower = 0, min.len = 1, coerce = TRUE)
  len_x <- length(x)

  qassert(n, "A+")
  n <- assert_integerish(n, lower = 0, min.len = 1, coerce = TRUE)
  len_n <- length(n)

  len_p <- length(p)
  if(!len_p) {
    p <- 0.5
  } else {
    qassert(p, "A+")
    qassert(p, "N+[0, 1]")
  }

  qassert(exact, "B1")
  if(!exact) qassert(correct, "B1")

  ts_method <- match.arg(
    tolower(ts_method),
    c("minlike", "blaker", "absdist", "central")
  )

  len_a <- length(alternative)
  for(i in seq_len(len_a)){
    alternative[i] <- match.arg(
      tolower(alternative[i]),
      c("two.sided", "less", "greater")
    )
    if(exact && alternative[i] == "two.sided")
      alternative[i] <- ts_method
  }

  qassert(simple_output, "B1")

  # maximum length of test parameters
  len_g <- max(len_x, len_n, len_p, len_a)

  # replicate inputs to same length
  if(len_x < len_g) x <- rep_len(x, len_g)
  if(len_n < len_g) n <- rep_len(n, len_g)
  if(len_p < len_g) p <- rep_len(p, len_g)
  if(len_a < len_g) alternative <- rep_len(alternative, len_g)

  # abort if any observed 'x' exceeds its 'n'
  if(any(x > n))
    cli_abort("All values of 'x' must not exceed 'n'.")

  # determine p-values (and their supports)
  res <- binom_test_int(x, n, p, alternative, exact, correct, simple_output)

  # create output object
  out <- if(!simple_output) {
    dnames <- sapply(match.call(), deparse1)

    DiscreteTestResults$new(
      test_name = "Binomial test",
      inputs = list(
        observations = data.frame(
          `number of successes` = x,
          check.names = FALSE
        ),
        parameters = data.frame(
          `number of trials` = n,
          check.names = FALSE
        ),
        nullvalues = data.frame(
          `probability of success` = p,
          check.names = FALSE
        ),
        computation = Filter(
          function(df) !all(is.na(df)),
          data.frame(
            alternative = alternative,
            exact = exact,
            distribution = ifelse(exact, "binomial", "normal"),
            #distribution.mean = if(exact) rep(NA_real_, len_g) else n * p,
            #distribution.sd = if(exact) rep(NA_real_, len_g) else sqrt(n * p * (1 - p)),
            `continuity correction` = if(exact) NA else correct,
            check.names = FALSE
          )
        )
      ),
      statistics = NULL,
      p_values = res$pv,
      pvalue_supports = res$sup,
      support_indices = res$idx,
      data_name = dnames[c("x", "n")]
    )
  } else res

  # return results
  return(out)
}

#' @rdname binom_test_pv
#' @importFrom lifecycle deprecate_stop
#' @export
binom.test.pv <- function(
    x,
    n,
    p = 0.5,
    alternative = "two.sided",
    ts.method = "minlike",
    exact = TRUE,
    correct = TRUE,
    simple.output = FALSE
) {
  deprecate_stop("0.2", "binom.test.pv()", "binom_test_pv()")
}

#' @name sign_test_pv
#'
#' @title
#' Sign Tests
#'
#' @description
#' `sign_test_pv()` performs an exact or approximate sign test about the median
#' of a distribution. It supports both one-sample and two-sample paired tests.
#' In contrast to other implementations, it is vectorised, only calculates
#' *p*-values, and offers a normal approximation of their computation.
#' Furthermore, it is capable of returning the discrete *p*-value supports, i.e.
#' all observable *p*-values under a null hypothesis. Multiple tests can be
#' evaluated simultaneously. In two-sided tests, several procedures of obtaining
#' the respective *p*-values are implemented.
#'
#' @param x       numerical vector forming the sample to be tested or a list of
#'                numerical vectors for multiple tests.
#' @param y       numerical vector forming the second sample to be tested or a
#'                list of numerical vectors for multiple tests; if `y = NULL`
#'                (the default), the one-sample version is performed; for
#'                two-sample tests, all sample pairs must have the same length.
#' @param mu      numerical vector of hypothesised median(s) for one-sample
#'                tests or median(s) of differences for two-sample tests.
#'
#' @template param
#' @templateVar alternative TRUE
#' @templateVar exact TRUE
#' @templateVar correct TRUE
#' @templateVar simple_output TRUE
#'
#' @details
#' For the **one-sample** case, the sign test counts the number of observations
#' in `x` that exceed the hypothesised median `mu`. Observations exactly equal
#' to `mu` (ties) are discarded. Under the null hypothesis, the number of
#' positive signs follows a Binomial(\eqn{n}, 0.5) distribution, where \eqn{n}
#' is the number of non-tied observations.
#'
#' For the **two-sample paired** case (when `y` is supplied), the pairwise
#' differences \eqn{d_i = x_i - y_i - \mu} are computed first. The sign test is
#' then applied to these differences.
#'
#' If `x` is a list, multiple tests are evaluated simultaneously. The parameters
#' `x`, `y` (if supplied and a list), `mu`, and `alternative` are vectorised.
#' They are replicated automatically to have the same lengths. This allows
#' multiple hypotheses to be tested simultaneously.
#'
#' The sign test is a special case of the binomial test. In contrast to
#' [`binom_test_pv()`], `sign_test_pv()` does not allow specifying exact
#' two-sided *p*-value calculation procedures. The reason is that the exact sign
#' test always tests for a probability of 0.5, in which case all these exact
#' two-sided *p*-value computation methods yield exactly the same results.
#'
#' @template return
#'
#' @seealso
#' [binom_test_pv()], [stats::binom.test()]
#'
#' @references
#' Dixon, W. J. and Mood, A. M. (1946). The statistical sign test.
#'   *Journal of the American Statistical Association*, **41**(236),
#'   pp. 557–566. \doi{10.1080/01621459.1946.10501898}
#'
#' @examples
#' # One-sample sign test: test whether median equals 3
#' x <- c(1, 5, 2, 7, 6, 8, 4)
#' results_1s <- sign_test_pv(x, mu = 3)
#' print(results_1s)
#' results_1s$get_pvalues()
#' results_1s$get_pvalue_supports()
#'
#' # Paired two-sample sign test: test whether difference of medians equals 1
#' x2 <- c(6, 8, 2, 4, 5)
#' y2 <- c(3, 5, 4, 2, 6)
#' results_2s <- sign_test_pv(x2, y2, mu = 1)
#' print(results_2s)
#' results_2s$get_pvalues()
#'
#' # Multiple tests simultaneously, one-sided p-values (one-sample, list input)
#' xs <- list(c(1, 5, 2, 7, 6, 8, 4), c(2, 4, 6, 1, 9))
#' results_l <- sign_test_pv(xs, mu = c(3, 5), alternative = "greater")
#' print(results_l)
#' results_l$get_pvalues()
#'
#' # Normal-approximated (one-sample, list input)
#' results_a <- sign_test_pv(xs, mu = c(3, 5), exact = FALSE)
#' print(results_a)
#' results_a$get_pvalues()
#'
#' @importFrom checkmate qassert qassertr
#' @importFrom cli cli_abort cli_warn
#' @export
sign_test_pv <- function(
  x,
  y = NULL,
  mu = 0,
  alternative = "two.sided",
  exact = TRUE,
  correct = TRUE,
  simple_output = FALSE
) {
  # plausibility checks of input parameters
  qassert(x, c("N+", "L+"))
  if(!is.list(x)) x <- list(x) else qassertr(x, "N+")
  len_x <- length(x)

  one_sample <- is.null(y)
  if(!one_sample) {
    qassert(y, c("N+", "L+"))
    if(!is.list(y)) y <- list(y) else qassertr(y, "N+")
  }
  len_y <- length(y)

  qassert(mu, "N+()")
  len_m <- length(mu)

  qassert(exact, c("B1", "0"))
  if(!exact) qassert(correct, "B1")

  len_a <- length(alternative)
  for(i in seq_len(len_a)){
    alternative[i] <- match.arg(
      tolower(alternative[i]),
      c("two.sided", "less", "greater")
    )
  }

  qassert(simple_output, "B1")

  # replicate inputs to same length
  len_g <- max(len_x, len_y, len_m, len_a)
  if(len_x < len_g) x <- rep_len(x, len_g)
  if(len_m < len_g) mu <- rep_len(mu, len_g)
  if(len_a < len_g) alternative <- rep_len(alternative, len_g)

  if(!one_sample) {
    if(len_y < len_g) y <- rep_len(y, len_g)

    for(i in seq_len(len_g)) {
      # check if lengths of all sample pairs are equal; stop if they are not
      if(length(x[[i]]) != length(y[[i]]))
        cli_abort('All paired samples must have the same length')
    }
  }

  # compute sign counts
  s <- integer(len_g)
  n <- integer(len_g)
  for(i in seq_len(len_g)) {
    d    <- if(one_sample) x[[i]] - mu[i] else x[[i]] - y[[i]] - mu[i]
    s[i] <- sum(d > 0)
    n[i] <- sum(d != 0)
  }

  # warn if any test has no non-tied observations
  if(any(n == 0L))
    cli_warn(
      c(
        "Some tests have no non-tied observations; p-values set to 1.",
        i = "Check whether 'x' (and 'y') contain only values equal to 'mu'."
      )
    )

  # determine p-values (and their supports)
  p <- rep(0.5, len_g)
  res <- binom_test_int(s, n, p, alternative, exact, correct, simple_output)

  # create output object
  out <- if(!simple_output) {
    dnames <- sapply(match.call(), deparse1)

    DiscreteTestResults$new(
      test_name = if(one_sample) "One-sample sign test" else
        "Paired two-sample sign test",
      inputs = list(
        observations = if(one_sample) x else list(x, y),
        parameters = data.frame(
          `(non-tied) observations` = n,
          check.names = FALSE
        ),
        nullvalues = if(one_sample) data.frame(median = mu) else
          data.frame(`difference of medians` = mu, check.names = FALSE),
        computation = Filter(
          function(df) !all(is.na(df)),
          data.frame(
            alternative = alternative,
            exact = exact,
            distribution = ifelse(exact, "binomial", "normal"),
            `continuity correction` = if(exact) NA else correct,
            check.names = FALSE
          )
        )
      ),
      statistics = data.frame(
        `positive signs` = s,
        #`negative signs` = n - s,
        check.names = FALSE
      ),
      p_values = res$pv,
      pvalue_supports = res$sup,
      support_indices = res$idx,
      data_name = dnames[if(one_sample) "x" else c("x", "y")]
    )
  } else res

  # return results
  return(out)
}

#' @name fisher_test_pv
#'
#' @title
#' Fisher's Exact Test for Count Data
#'
#' @description
#' `fisher_test_pv()` performs Fisher's exact test or a chi-square approximation
#' to assess if rows and columns of a 2-by-2 contingency table with fixed
#' marginals are independent. In contrast to [`stats::fisher.test()`], it is
#' vectorised, only calculates *p*-values and offers a normal approximation of
#' their computation. Furthermore, it is capable of returning the discrete
#' *p*-value supports, i.e. all observable *p*-values under a null hypothesis.
#' Multiple tables can be analysed simultaneously. In two-sided tests, several
#' procedures of obtaining the respective *p*-values are implemented.
#'
#' **Note**: Please do not use the older `fisher.test.pv()` anymore! It is now
#' defunct and will be removed in a future version.\cr
#' `r lifecycle::badge('superseded')`
#'
#' @param x   integer vector with four elements, a 2-by-2 matrix or an integer
#'            matrix (or data frame) with four columns, where each line
#'            represents a 2-by-2 table to be tested.
#'
#' @template param
#' @templateVar alternative TRUE
#' @templateVar ts.method TRUE
#' @templateVar exact TRUE
#' @templateVar correct TRUE
#' @templateVar simple.output TRUE
#'
#' @details
#' The parameters `x` and `alternative` are vectorised. They are replicated
#' automatically, such that the number of `x`'s rows is the same as the length
#' of `alternative`. This allows multiple null hypotheses to be tested
#' simultaneously. Since `x` is (if necessary) coerced to a matrix with four
#' columns, it is replicated row-wise.
#'
#' If `exact = TRUE`, Fisher's exact test is performed (the specific hypothesis
#' depends on the value of `alternative`). Otherwise, if `exact = FALSE`, a
#' chi-square approximation is used for two-sided hypotheses or a normal
#' approximation for one-sided tests, based on the square root of the
#' chi-squared statistic. This is possible because the degrees of freedom of
#' chi-squared tests on 2-by-2 tables are limited to 1.
#'
#' @template details_two_sided
#'
#' @template return
#'
#' @seealso
#' [`stats::fisher.test()`]
#'
#' @references
#' Fisher, R. A. (1935). The logic of inductive inference.
#'   *Journal of the Royal Statistical Society Series A*, **98**, pp.
#'   39–54. \doi{10.2307/2342435}
#'
#' Agresti, A. (2002). *Categorical data analysis*. Second Edition. New York:
#'   John Wiley & Sons. pp. 91–97. \doi{10.1002/0471249688}
#'
#' Blaker, H. (2000). Confidence curves and improved exact confidence intervals
#'   for discrete distributions. *Canadian Journal of Statistics*,
#'   **28**(4), pp. 783-798. \doi{10.2307/3315916}
#'
#' Hirji, K. F. (2006). *Exact analysis of discrete data*. New York: Chapman
#'   and Hall/CRC. pp. 55-83. \doi{10.1201/9781420036190}
#'
#' @examples
#' # Constructing
#' S1 <- c(4, 2, 2, 14, 6, 9, 4, 0, 1)
#' S2 <- c(0, 0, 1, 3, 2, 1, 2, 2, 2)
#' N1 <- rep(148, 9)
#' N2 <- rep(132, 9)
#' F1 <- N1 - S1
#' F2 <- N2 - S2
#' df <- data.frame(S1, F1, S2, F2)
#'
#' # Fisher's exact p-values and their supports
#' results_ex <- fisher_test_pv(df)
#' print(results_ex)
#' results_ex$get_pvalues()
#' results_ex$get_pvalue_supports()
#'
#' # Chi-square-approximated p-values and their supports
#' results_ap <- fisher_test_pv(df, exact = FALSE)
#' print(results_ap)
#' results_ap$get_pvalues()
#' results_ap$get_pvalue_supports()
#'
#' @importFrom stats dhyper pchisq
#' @importFrom checkmate assert_integerish qassert
#' @importFrom cli cli_abort cli_warn
#' @export
fisher_test_pv <- function(
  x,
  alternative = "two.sided",
  ts_method = "minlike",
  exact = TRUE,
  correct = TRUE,
  simple_output = FALSE
) {
  # plausibility checks of input parameters

  # define error message for malformed x
  error_msg_x <- paste("'x' must either be a 2-by-2 matrix,",
                       "a four-element vector or a four-column matrix")

  #  if x is an atomic vector, make it a matrix with one row
  if(is.vector(x) && is.atomic(x))
    x <- t(x)
  # if x is a data frame, make it a matrix
  if(is.data.frame(x))
    x <- as.matrix(x)
  # when x is a matrix, it must satisfy some conditions
  if(is.matrix(x)) {
    # stop immediately, if dimensions are violated
    if(any(dim(x) != 2) && ncol(x) != 4 && nrow(x) != 4)
      cli_abort(error_msg_x)
    # check if all values are non-negative and close to integer
    assert_integerish(x, lower = 0, min.len = 4)
    # coerce to integer
    x <- round(x)
    mode(x) <- "integer"
    # 2-by-2 matrices are transformed to single-row matrix
    if(all(dim(x) == 2)) {
      x <- matrix(as.vector(x), 1, 4,
        dimnames = list(NULL,
          make.names(paste(rep(colnames(x), rep(2, 2)), rownames(x)))
        )
      )
    }
  } else cli_abort(error_msg_x)
  len_x <- nrow(x)

  qassert(exact, "B1")
  if(!exact) qassert(correct, "B1")

  ts_method <- match.arg(
    tolower(ts_method),
    c("minlike", "blaker", "absdist", "central")
  )

  len_a <- length(alternative)
  for(i in seq_len(len_a)){
    alternative[i] <- match.arg(
      tolower(alternative[i]),
      c("two.sided", "less", "greater")
    )
    if(exact && alternative[i] == "two.sided")
      alternative[i] <- ts_method
  }

  qassert(simple_output, "B1")

  # determine parameters for R's hypergeometric distribution implementation
  m <- x[, 1] + x[, 2] # sums of 1st columns
  n <- x[, 3] + x[, 4] # sums of 2nd columns
  k <- x[, 1] + x[, 3] # sums of 1st rows
  q <- x[, 1]          # upper left elements

  # replicate inputs to same length
  len_g <- max(len_x, len_a)
  if(len_x < len_g){
    m <- rep_len(m, len_g)
    n <- rep_len(n, len_g)
    k <- rep_len(k, len_g)
    q <- rep_len(q, len_g)
  }
  if(len_a < len_g) alternative <- rep_len(alternative, len_g)

  # determine unique parameter sets and possible "q" value boundaries (support)
  params <- unique(data.frame(m, n, k, alternative))
  len_u <- nrow(params)
  m_u <- as.integer(params$m)
  n_u <- as.integer(params$n)
  k_u <- as.integer(params$k)
  alt_u <- params$alternative
  hi <- pmin(k_u, m_u)
  lo <- pmax(0, k_u - n_u)

  # prepare output
  res <- numeric(nrow(x))
  if(!simple_output) {
    supports <- vector("list", len_u)
    indices  <- vector("list", len_u)
  }
  if(!exact) {
    chi_out <- numeric(len_g)
    if(any(alternative != "two.sided")) delta_out <- rep(NA_real_)
  }

  # begin computations
  for(i in seq_len(len_u)) {
    # which hypotheses belong to the current unique parameter set
    idx_supp <- which(m == m_u[i] & n == n_u[i] & k == k_u[i] & alternative == alt_u[i])
    # possible "q" values
    support <- lo[i]:hi[i]

    if(exact){
      # compute all possible probability masses under fixed marginals
      d <- numerical_adjust(dhyper(support, m_u[i], n_u[i], k_u[i]))
      # p-value supports according to alternative and (maybe) two-sided method
      pv_supp <- support_exact(
        alternative = alt_u[i],
        probs = d,
        expectations = abs(support - m_u[i] * k_u[i] / (n_u[i] + m_u[i]))
      )
    } else {
      # observable tables under fixed marginals
      y <- rbind(
        support,
        m_u[i] - support,
        k_u[i] - support,
        n_u[i] + support - k_u[i]
      )
      # observable odds ratios
      obs_or <- y[1, ] * y[4, ] / (y[2, ] * y[3, ])
      # direction of deviance from homogeneity (odds ratio = 1)
      delta <- sign(obs_or - 1)
      # correct for NaN deltas (if observable odds ratio = NaN)
      delta[is.nan(delta)] <- 0

      # expected table under homogeneity
      f11 <- m_u[i] * k_u[i] / (n_u[i] + m_u[i])
      f10 <- k_u[i] - f11
      f01 <- m_u[i] - f11
      f00 <- n_u[i] - f10
      expected <- pmax(0, c(f11, f01, f10, f00))
      if(any(expected < 5))
        cli_warn("One or more Chi-squared approximations may be incorrect!\n")

      # chi-square values
      absdiff <- if(correct) {
        matrix(pmax(0, abs(y - expected) - 0.5), 4)^2
      } else (y - expected)^2
      chi <- absdiff / expected
      chi[is.nan(chi)] <- 0
      chi <- numerical_adjust(colSums(chi), FALSE)
      # degrees of freedom
      df <- as.integer(1 - any(expected == 0))
      # p-value supports according to alternative
      pv_supp <- switch(alt_u[i],
        less = {
          p <- pnorm_zero(delta * sqrt(chi), df)
          p[p == max(p)] <- 1
          p
        },
        greater = {
          p <- pnorm_zero(delta * sqrt(chi), df, FALSE)
          p[p == max(p)] <- 1
          p
        },
        two.sided = pchisq(chi, df, lower.tail = FALSE)
      )
    }

    # store results and support
    idx_obs <- sapply(
      seq_along(idx_supp), function(j) which(support == q[idx_supp[j]])
    )
    if(!exact && !simple_output) {
      chi_out[idx_supp] <- chi[idx_obs]
      if(alt_u[i] != "two.sided") delta_out[idx_supp] <- delta[idx_obs]
    }
    res[idx_supp] <- pv_supp[idx_obs]
    if(!simple_output) {
      supports[[i]] <- unique(sort(pv_supp))
      indices[[i]]  <- idx_supp
    }
  }

  # create output object
  out <- if(!simple_output) {
    dnames <- sapply(match.call(), deparse1)
    if(is.null(colnames(x)))
      colnames(x) <- paste0("x", c("[1, 1]", "[2, 1]", "[1, 2]", "[2, 2]"))
    if(len_x < len_g)
      x <- x[rep_len(seq_len(len_x), len_g), ]

    DiscreteTestResults$new(
      test_name = ifelse(
        exact,
        "Fisher's Exact test",
        "Chi-squared test for homogenity"
      ),
      inputs = list(
        observations = as.data.frame(x),
        parameters = NULL,
        nullvalues = data.frame(
          `odds ratio` = rep(1, len_g),
          check.names = FALSE
        ),
        computation = Filter(
          function(df) !all(is.na(df)),
          data.frame(
            alternative = alternative,
            exact = exact,
            distribution = ifelse(exact, "hypergeometric",
              ifelse(alternative == "two.sided", "chi-squared", "normal")
            ),
            #distribution.df = ifelse(
            #  !exact & alternative == "two.sided", df, NA_integer_
            #),
            #distribution.mean = ifelse(
            #  !exact & alternative != "two.sided", 0, NA_real_
            #),
            #distribution.sd = ifelse(
            #  !exact & alternative != "two.sided", 1, NA_real_
            #),
            `continuity correction` = if(exact) NA else correct,
            `first column sum` = m,
            `second column sum` = n,
            `first row sum` = k,
            `second row sum` = m + n - k,
            `total` = m + n,
            check.names = FALSE
          )
        )
      ),
      statistics = if(!exact) data.frame(
        `chi-squared` = ifelse(alternative == "two.sided", chi_out, NA_real_),
        z = ifelse(
          alternative == "two.sided", NA_real_, delta_out * sqrt(chi_out)
        ),
        check.names = FALSE
      ),
      p_values = res,
      pvalue_supports = supports,
      support_indices = indices,
      data_name = dnames["x"]
    )
  } else res

  # return results
  return(out)
}

#' @rdname fisher_test_pv
#' @export
#' @importFrom lifecycle deprecate_stop
fisher.test.pv <- function(
  x,
  alternative = "two.sided",
  ts.method = "minlike",
  exact = TRUE,
  correct = TRUE,
  simple.output = FALSE
) {
  deprecate_stop("0.2", "fisher.test.pv()", "fisher_test_pv()")
}

#' @title Conditional Two-Sample Homogeneity Test for Binomial Experiments
#'
#' @description
#' Performs an exact or approximate conditional test about the homogeneity of
#' two binomial samples, i.e. regarding the respective probabilities of success.
#' It is vectorised, only calculates *p*-values and offers a normal
#' approximation of their computation. Furthermore, it is capable of returning
#' the discrete *p*-value supports, i.e. all observable *p*-values under a null
#' hypothesis. Multiple tests can be evaluated simultaneously. In two-sided
#' tests, several procedures of obtaining the respective *p*-values are
#' implemented.
#'
#' @param x   integer vector with two elements or a matrix with two columns or a
#'            data frame with two columns giving the number of successes for the
#'            two experiments.
#' @param n   integer vector with two elements or a matrix with two columns or a
#'            data frame with two columns giving the number of trials for the
#'            two experiments.
#'
#' @template param
#' @templateVar alternative TRUE
#' @templateVar ts_method TRUE
#' @templateVar exact TRUE
#' @templateVar correct TRUE
#' @templateVar simple_output TRUE
#'
#' @details
#' The parameters `x`, `n` and `alternative` are vectorised. They are replicated
#' automatically, such that the number of `x`'s rows is the same as the length
#' of `alternative`. This allows multiple null hypotheses to be tested
#' simultaneously. Since `x` and `n` are coerced to matrices (if necessary) with
#' two columns, they are replicated row-wise.
#'
#' It can be shown that this test is a special case of Fisher's exact test,
#' because it is conditional on the numbers of trials **and** the sums of
#' successes and failures. Therefore, its computations are handled by
#' [`fisher_test_pv()`].
#'
#' @template details_two_sided
#'
#' @template return
#'
#' @seealso
#' [`fisher_test_pv()`]
#'
#' @references
#' Fisher, R. A. (1935). The logic of inductive inference.
#'   *Journal of the Royal Statistical Society Series A*, **98**, pp.
#'   39–54. \doi{10.2307/2342435}
#'
#' Agresti, A. (2002). *Categorical data analysis*. Second Edition. New York:
#'   John Wiley & Sons. pp. 91–97. \doi{10.1002/0471249688}
#'
#' Blaker, H. (2000) Confidence curves and improved exact confidence intervals
#'   for discrete distributions. *Canadian Journal of Statistics*,
#'   **28**(4), pp. 783-798. \doi{10.2307/3315916}
#'
#' Hirji, K. F. (2006). *Exact analysis of discrete data*. New York: Chapman
#'   and Hall/CRC. pp. 55-83. \doi{10.1201/9781420036190}
#'
#' @examples
#' # Constructing
#' set.seed(3)
#' p1 <- c(0.25, 0.5, 0.75)
#' p2 <- c(0.15, 0.5, 0.60)
#' n1 <- c(10, 20, 50)
#' n2 <- c(25, 75, 200)
#' x1 <- rbinom(3, n1, p1)
#' x2 <- rbinom(3, n2, p2)
#' x  <- cbind(x1 = x1, x2 = x2)
#' n  <- cbind(n1 = n1, n2 = n2)
#'
#' # Exact two-sided p-values ("blaker") and their supports
#' results_ex <- homogeneity_test_pv(x, n, ts_method = "blaker")
#' print(results_ex)
#' results_ex$get_pvalues()
#' results_ex$get_pvalue_supports()
#'
#' # Normal-approximated one-sided p-values ("less") and their supports
#' results_ap <- homogeneity_test_pv(x, n, "less", exact = FALSE)
#' print(results_ap)
#' results_ap$get_pvalues()
#' results_ap$get_pvalue_supports()
#'
#' @importFrom checkmate assert_integerish
#' @importFrom cli cli_abort
#' @export
homogeneity_test_pv <- function(
  x,
  n,
  alternative = "two.sided",
  ts_method = "minlike",
  exact = TRUE,
  correct = TRUE,
  simple_output = FALSE
) {
  # plausibility checks of input parameters

  # define error message for malformed x or n
  error_msg <- paste("must either be a two-element vector, a two-column matrix",
                     "or a two-column data frame")

  # make sure that x is a data frame, matrix or vector
  qassert(x, c("D+", "M+", "V2"))
  # make sure that n is a data frame, matrix or vector
  qassert(n, c("D+", "M+", "V2"))

  # if x is an atomic vector, make it a matrix with one row
  if(is.vector(x) && is.atomic(x))
    x <- t(x)
  # if n is an atomic vector, make it a matrix with one row
  if(is.vector(n) && is.atomic(n))
    n <- t(n)

  # if x is a data frame, make it a matrix
  if(is.data.frame(x))
    x <- as.matrix(x)
  # if n is a data frame, make it a matrix
  if(is.data.frame(n))
    n <- as.matrix(n)

  # when x is a matrix, it must satisfy some conditions
  if(is.matrix(x)) {
    # stop immediately, if dimensions are violated
    if(ncol(x) != 2 && nrow(x) != 2)
      cli_abort(paste("'x'", error_msg))
    # check if all values are non-negative and close to integer
    assert_integerish(x, lower = 0, min.len = 2)
    # coerce to integer
    x <- round(x)
    mode(x) <- "integer"
  } else cli_abort(paste("'x'", error_msg))
  # when n is a matrix, it must satisfy some conditions
  if(is.matrix(n)) {
    # stop immediately, if dimensions are violated
    if(ncol(n) != 2 && nrow(n) != 2)
      cli_abort(paste("'n'", error_msg))
    # check if all values are non-negative and close to integer
    assert_integerish(n, lower = 0, min.len = 2)
    # coerce to integer
    n <- round(n)
    mode(n) <- "integer"
  } else cli_abort(paste("'n'", error_msg))

  # determine largest number of rows
  len_x <- nrow(x)
  len_n <- nrow(n)
  len_g <- max(len_x, len_n)
  # enlarge x by row-replication, if necessary
  if(len_x < len_g)
    x <- x[rep_len(seq_len(len_x), len_g), ]
  # enlarge n by row-replication, if necessary
  if(len_n < len_g)
    n <- n[rep_len(seq_len(len_n), len_g), ]

  # create matrix of vectorised 2-by-2 tables
  y <- cbind(x, n - x);

  # compute test results
  res <- fisher_test_pv(y, alternative, ts_method, exact, correct, simple_output)

  # create output object
  out <- if(!simple_output) {
    dnames <- sapply(match.call(), deparse1)
    #if(is.null(colnames(x)))
      colnames(x) <- paste(c("first", "second"), "number of successes")
    #if(is.null(colnames(n)))
      colnames(n) <- paste(c("first", "second"), "number of trials")

    DiscreteTestResults$new(
      test_name = "Conditional two-sample binomial homogeneity test",
      inputs = c(
        list(
          observations = data.frame(x, check.names = FALSE),
          parameters = data.frame(n, check.names = FALSE),
          nullvalues = data.frame(
            `odds ratio` = rep(1, len_g),
            check.names = FALSE
          ),
          computation = Filter(
            function(df) !all(is.na(df)),
            data.frame(
              alternative = res$get_inputs()$computation$alternative,
              exact = res$get_inputs()$computation$exact,
              distribution = res$get_inputs()$computation$distribution,
              #distribution.df = ifelse(
              #  !exact & alternative == "two.sided", df, NA_integer_
              #),
              #distribution.mean = ifelse(
              #  !exact & alternative != "two.sided", 0, NA_real_
              #),
              #distribution.sd = ifelse(
              #  !exact & alternative != "two.sided", 1, NA_real_
              #),
              `continuity correction` = if(exact) NA else correct,
              `total number of successes` = rowSums(x[, 1:2, drop = FALSE]),
              `total number of trials` = rowSums(n),
              check.names = FALSE
            )
          )
        )
      ),
      statistics = res$get_statistics(),
      p_values = res$get_pvalues(),
      pvalue_supports = res$get_pvalue_supports(unique = TRUE),
      support_indices = res$get_support_indices(),
      data_name = dnames[c("x", "n")]
    )
  } else res

  # return results
  return(out)
}

#' @name perm_test_pv
#'
#' @title
#' Permutation Test
#'
#' @description
#' `perm_test_pv()` performs a permutation test for the comparison of two
#' independent samples using a user-supplied test statistic. It is vectorised
#' over multiple test problems: by passing lists of sample vectors for `x` and
#' `y`, several tests can be evaluated simultaneously. The function can compute
#' exact *p*-values (by enumerating all permutations) or approximate *p*-values
#' via Monte Carlo sampling. It is capable of returning the discrete *p*-value
#' supports, i.e. all observable *p*-values under the null hypothesis.
#'
#' @param x                 numerical vector or list of numerical vectors of
#'                          observations in the first group.
#' @param y                 numerical vector or list of numerical vectors of
#'                          observations in the second group.
#' @param statistic         single character giving the type of test statistic
#'                          to be used; must be one of `"diff_mean"`,
#'                          `"diff_median"`, `"diff_t"`, `"diff_welch"`,
#'                          `"diff_hl"`, `"ratio_var"` or `"ratio_sd"`
#'                          (see Details).
#' @param mu                numerical vector giving the hypothesised values
#'                          under the null; if `is.null(mu)` (the default) it
#'                          is set to 0 for test statistics that are based on
#'                          differences (`diff_`*) or 1 for ratio-based tests
#'                          statistics (`ratio_`*).
#' @param exact             logical value that indicates whether \eqn{p}-values
#'                          are to be calculated by exact computation (`TRUE`)
#'                          or by simulation (`FALSE`); if `NULL` (the default)
#'                          exact computation is performed if the number of
#'                          possible sample combinations (see Details) is below
#'                          or equal to `max_exact_combs`; otherwise the
#'                          respective \eqn{p}-values are obtained by
#'                          simulation.
#' @param max_exact_combs   maximum number of allowed combinations for exact
#'                          computation of the permutation distribution (see
#'                          details).
#' @param MC_sims           positive integer or `NULL`. The number of Monte
#'                          Carlo permutations to draw when `exact = FALSE`.
#'                          Ignored when `exact = TRUE`. Default is `10000L`.
#' @param seed              single integer or `NULL`. Random seed passed to
#'                          [`set.seed()`] before Monte Carlo sampling to ensure
#'                          reproducibility. Use `NULL` to skip seed setting.
#'                          Ignored when `exact = TRUE`.
#'
#' @template param
#' @templateVar alternative TRUE
#' @templateVar simple_output TRUE
#'
#' @details
#' Under the null hypothesis of exchangeability (i.e. both samples come from
#' the same distribution), all \eqn{\binom{n_x + n_y}{n_x}} partitions of the
#' pooled sample into groups of sizes \eqn{n_x} and \eqn{n_y} are equally
#' likely. `perm_test_pv()` exploits this by:
#'
#' \enumerate{
#'   \item Computing the observed test statistic \eqn{T_\text{obs}} from
#'         `x` and `y` using the function selected by `statistic` (see the
#'         *Test Statistics* section below).
#'   \item Enumerating (if `exact = TRUE`) or randomly sampling (if
#'         `exact = FALSE`) permutations of the pooled sample.
#'   \item Evaluating the selected test statistic on each permuted split.
#'   \item Deriving the *p*-value as the fraction of permutation statistics
#'         that are at least as extreme as \eqn{T_\text{obs}}, where *extreme*
#'         is defined by `alternative`:
#'         \describe{
#'           \item{`"less"`}{fraction with permuted statistic
#'                 \eqn{\le T_\text{obs}}}
#'           \item{`"greater"`}{fraction with permuted statistic
#'                 \eqn{\ge T_\text{obs}}}
#'           \item{`"two.sided"`}{fraction with
#'                 \eqn{|\text{permuted statistic}| \ge |T_\text{obs}|}}
#'         }
#' }
#'
#' Exact computation is only feasible for small pooled samples; the total
#' number of permutations grows as \eqn{\binom{n_x + n_y}{n_x}}. If
#' `exact = NULL`, exact computation is applied when the number of computations
#' is below or equal to `max_exact_combs` (default \eqn{10{,}000{,}000}).
#' Otherwise, the distribution of the test statistics is approximated by Monte
#' Carlo simulation. When `exact = TRUE` and the number of permutations exceeds
#' `max_exact_combs`, an error is raised. Set `exact = FALSE` to use Monte Carlo
#' sampling in such cases.
#'
#' The parameters `x`, `y`, `alternative`, and `MC_sims` are vectorised via
#' lists: if `x` and `y` are lists of the same length, each pair
#' `(x[[i]], y[[i]])` defines one test. Scalars and single vectors are
#' recycled to the required length.
#'
#' ## Test Statistics
#'
#' The `statistic` argument selects among seven built-in test statistics for
#' comparing two groups. Let \eqn{\bar{x}}, \eqn{\bar{y}} denote the group
#' means, \eqn{s_x^2}, \eqn{s_y^2} the sample variances, and \eqn{n_x},
#' \eqn{n_y} the group sizes.
#'
#' \describe{
#'   \item{`"diff_mean"` — Difference of means
#'         \eqn{T = \bar{x} - \bar{y} - \mu_0}}{
#'         The most common choice for location comparisons. Tests whether the
#'         population means differ by \eqn{\mu_0} (default: 0).\cr
#'         *Advantages*: Intuitive interpretation; optimal power under normality
#'         and equal variances (Pitman efficiency 1 vs. the two-sample
#'         \eqn{t}-test).\cr
#'         *Disadvantages*: Sensitive to outliers; less powerful than
#'         `"diff_hl"` or `"diff_t"` under heavy-tailed distributions.}
#'   \item{`"diff_median"` — Difference of medians
#'         \eqn{T = \text{median}(x) - \text{median}(y) - \mu_0}}{
#'         Tests for a shift in the median rather than the mean (default:
#'         \eqn{\mu_0 = 0)}.\cr
#'         *Advantages*: Robust to outliers and skewed distributions; directly
#'         interpretable in terms of the median.\cr
#'         *Disadvantages*: The sample median is a step function of the data,
#'         leading to a coarser permutation distribution with many ties; can
#'         have lower power than `"diff_hl"` for continuous data.}
#'   \item{`"diff_t"` — Pooled two-sample \eqn{t}-statistic
#'         \eqn{T = \frac{\bar{x} - \bar{y} - \mu_0}{s_p \sqrt{1/n_x + 1/n_y}}}
#'         where
#'         \eqn{s_p = \sqrt{\frac{(n_x-1)s_x^2 + (n_y-1)s_y^2}{n_x+n_y-2}}}}{
#'         Scales the mean difference by the pooled standard deviation,
#'         analogous to the classical Student \eqn{t}-test.\cr
#'         *Advantages*: Studentisation makes the statistic (approximately)
#'         scale-free; useful when group variances are expected to be equal;
#'         the permutation \eqn{p}-value is valid even without normality.\cr
#'         *Disadvantages*: Assumes equal variances (homoscedasticity);
#'         sensitive to outliers; no power gain over `"diff_mean"` in the
#'         permutation setting when sample sizes are equal.}
#'   \item{`"diff_welch"` — Welch \eqn{t}-statistic
#'         \eqn{T = \frac{\bar{x} - \bar{y} - \mu_0}{\sqrt{s_x^2/n_x + s_y^2/n_y}}}}{
#'         Scales the mean difference by the unpooled standard error, analogous
#'         to Welch's two-sample \eqn{t}-test. Unlike `"diff_t"`, the variances
#'         of the two groups are estimated separately rather than pooled.\cr
#'         *Advantages*: Does not assume equal variances; retains more power
#'         than `"diff_t"` when group variances are unequal and sample sizes
#'         differ; in the permutation setting the \eqn{p}-value remains exactly
#'         valid regardless of the variance ratio, just as with `"diff_t"`.\cr
#'         *Disadvantages*: No power gain over `"diff_t"` when variances are
#'         equal; estimating two separate variances instead of one pooled
#'         variance introduces additional estimation uncertainty, which can
#'         slightly reduce power compared to `"diff_t"` in balanced designs with
#'         equal variances.}
#'   \item{`"diff_hl"` — Hodges–Lehmann estimator
#'         \eqn{T = \text{median}_{i,j}\,(x_i - y_j) - \mu_0}}{
#'         Tests for a shift in the median of all \eqn{n_x \cdot n_y} pairwise
#'         differences between the two groups (default: \eqn{\mu_0 = 0}). It is
#'         the natural companion statistic to the Wilcoxon–Mann–Whitney test and
#'         estimates the location shift \eqn{\Delta} under a shift model.\cr
#'         *Advantages*: Highly robust to outliers; produces fewer ties in the
#'         permutation distribution than `"diff_median"`.\cr
#'         *Disadvantages*: \eqn{O(n_x \cdot n_y)} computation; the
#'         interpretation as a median shift requires a location-shift model.}
#'   \item{`"ratio_var"` — Ratio of variances
#'         \eqn{T = s_x^2 / (s_y^2 \cdot \mu_0)}}{
#'         Tests for differences in spread (scale) rather than location.
#'         Under the null \eqn{H_0\colon \sigma_x^2 / \sigma_y^2 = \mu_0}
#'         (default: \eqn{\mu_0 = 1}).\cr
#'         *Advantages*: Direct measure of relative variability; permutation
#'         validity does not require normality (unlike the classical
#'         \eqn{F}-test).\cr
#'         *Disadvantages*: Highly sensitive to outliers and non-normality
#'         (the variance itself is not robust); should be interpreted with
#'         caution if the distributions differ in shape as well as scale.}
#'   \item{`"ratio_sd"` — Ratio of standard deviations
#'         \eqn{T = s_x / (s_y \cdot \mu_0)}}{
#'         Equivalent to `"ratio_var"` on the standard deviation scale
#'         (\eqn{T = \sqrt{s_x^2 / (s_y^2 \cdot \mu_0^2)}}); tests the same null
#'         hypothesis \eqn{H_0 \colon \sigma_x / \sigma_y = \mu_0} (default:
#'         \eqn{\mu_0 = 1}).\cr
#'         *Advantages*: Same unit as the original data, which can aid
#'         interpretation; monotone transformation of `"ratio_var"`, so the
#'         \eqn{p}-values are identical (note that \eqn{\mu_0} needs to be
#'         squared for the variance-based statistic to be equivalent).\cr
#'         *Disadvantages*: Same sensitivity to outliers as `"ratio_var"`;
#'         the monotone relationship means it carries no additional statistical
#'         information compared to `"ratio_var"`.
#'   }
#' }
#'
#' @template return
#'
#' @seealso
#' [`mann_whitney_test_pv()`], [`stats::wilcox.test()`]
#'
#' @references
#' Pitman, E. J. G. (1937). Significance tests which may be applied to
#'   samples from any populations. *Supplement to the Journal of the Royal
#'   Statistical Society*, **4**(1), pp. 119–130. \doi{10.2307/2984124}
#'
#' Good, P. (2000). *Permutation Tests: A Practical Guide to Resampling
#'   Methods for Testing Hypotheses*. Second Edition. New York: Springer.
#'   \doi{10.1007/978-1-4757-3235-1}
#'
#' @examples
#' set.seed(42)
#' x1 <- rnorm(8)
#' y1 <- rnorm(10, mean = 1)
#'
#' # Two-sided test for difference of means = 0
#' results_ex <- perm_test_pv(x1, y1)
#' print(results_ex)
#' results_ex$get_pvalues()
#'
#' # Hodges Lehmann statistic with Monte Carlo approximation
#' results_hl <- perm_test_pv(x1, y1, "diff_hl", exact = FALSE, seed = 1L)
#' results_hl$print()
#' results_hl$get_pvalues()
#'
#' # Multiple tests simultaneously, one-sided alternative (mu = 0.5),
#' # using t-statistic, forced exact computation
#' xs <- list(rnorm(12), rnorm(9, 1))
#' ys <- list(rnorm(11, 1), rnorm(10, 2))
#' results_multi <- perm_test_pv(xs, ys, "diff_t", c(0.5, -1.5), "greater", TRUE)
#' results_multi$print()
#' results_multi$get_pvalues()
#'
#' @importFrom checkmate qassert qassertr
#' @importFrom cli cli_warn
#' @importFrom stats setNames
#' @export
perm_test_pv <- function(
  x,
  y,
  statistic = c(
    "diff_mean", "diff_median", "diff_t", "diff_welch", "diff_hl",
    "ratio_var", "ratio_sd"
  ),
  mu = NULL,
  alternative = "two.sided",
  exact = NULL,
  max_exact_combs = 10L^7,
  MC_sims = 10L^5,
  seed = NULL,
  simple_output = FALSE
) {
  # input checks
  qassert(x, c("N+", "L+"))
  if(!is.list(x)) x <- list(x) else qassertr(x, "N+")
  len_x <- length(x)

  qassert(y, c("N+", "L+"))
  if(!is.list(y)) y <- list(y) else qassertr(y, "N+")
  len_y <- length(y)

  statistic <- match.arg(
    tolower(statistic),
    c(
      "diff_mean", "diff_median", "diff_t", "diff_welch", "diff_hl",
      "ratio_var", "ratio_sd"
    )
  )

  if(startsWith(statistic, "diff")) {
    qassert(mu, c("n+()", "0"))
    if(!is.null(mu)) mu[is.na(mu)] <- 0 else mu <- 0
  } else {
    qassert(mu, c("n+(0,)", "0"))
    if(!is.null(mu)) mu[is.na(mu)] <- 1 else mu <- 1
  }
  len_m <- length(mu)

  len_a <- length(alternative)
  for(i in seq_len(len_a)) {
    alternative[i] <- match.arg(
      tolower(alternative[i]),
      c("two.sided", "less", "greater")
    )
  }

  qassert(exact, c("B1", "0"))

  qassert(max_exact_combs, "X1[1,)")
  max_exact_combs <- as.integer(max_exact_combs)

  qassert(MC_sims, "X1[1,)")
  MC_sims <- as.integer(MC_sims)

  assert_numeric(seed, any.missing = FALSE, null.ok = TRUE)

  qassert(simple_output, "B1")

  # replicate to common length
  len_g <- max(len_x, len_y, len_m, len_a)
  if(len_x < len_g) x <- rep_len(x, len_g)
  if(len_y < len_g) y <- rep_len(y, len_g)
  if(len_m < len_g) mu <- rep_len(mu, len_g)
  if(len_a < len_g) alternative <- rep_len(alternative, len_g)

  # per-test sizes and observed statistics
  T_obs <- numeric(len_g)
  use_exact <- logical(len_g)

  for(i in seq_len(len_g)) {
    nx <- length(x[[i]])
    ny <- length(y[[i]])
    n_total <- nx + ny

    # check whether exact computation is feasible
    n_perm <- choose(n_total, nx)
    use_exact[i] <- if(is.null(exact)) n_perm <= max_exact_combs else exact

    if(is.null(exact) && n_perm > max_exact_combs)
      cli_warn(
        c(
          paste0(
            "Test ", i, ": exact computation requires ",
            formatC(n_perm, format = "fg", big.mark = ","),
            " permutations, exceeding the limit given by 'max_exact_combs' of ",
            formatC(max_exact_combs, format = "fg", big.mark = ","), "."
          ),
          i = paste(
            "Using Monte Carlo sampling instead.",
            "Increase limit if exact calculation is desired."
          )
        )
      )
  }

  if(!is.null(seed) && any(!use_exact)) set.seed(seed)

  # prepare output containers
  res <- numeric(len_g)
  if(!simple_output) {
    supports <- vector("list", len_g)
    indices  <- vector("list", len_g)
  }

  # compute permutation p-values
  for(i in seq_len(len_g)) {
    comp_res <- perm_test_run(
      x[[i]], y[[i]], mu[i], statistic, use_exact[i], MC_sims
    )

    # observed statistic
    T_obs[i] <- comp_res$observed

    # observable statistics (unique and sorted!)
    T_stats <- comp_res$statistics

    # adjust statistics for slight numerical differences
    T_stats <- numerical_adjust(T_stats, FALSE)
    T_obs[i] <- T_stats[which.min(abs(T_stats - T_obs[i]))]

    # adjust test statistics if i-th test is two-sided
    if(alternative[i] == "two.sided") {
      T_stats  <- abs(T_stats)
      T_obs[i] <- abs(T_obs[i])
    }

    # probabilities of each statistic
    probs <- comp_res$probabilities

    # sort statistics (and probabilities accordingly)
    ord_T   <- order(T_stats)
    T_stats <- T_stats[ord_T]
    probs   <- probs[ord_T]

    # if necessary: remove duplicate statistics and combine probabilities
    dupl <- duplicated(T_stats)
    if(any(dupl)) {
      sf <- stepfun(T_stats, c(0, cumsum(probs)))
      T_stats <- T_stats[!dupl]
      probs <- diff(c(0, sf(T_stats)))
    }

    # adjust probabilities for slight numerical differences
    probs <- numerical_adjust(probs)

    # unique sorted p-value support from all permutation statistics
    pv_supp <- pmin(1,
      switch(
        alternative[i],
        less = cumsum(probs),
        greater = rev(cumsum(rev(probs))),
        two.sided = rev(cumsum(rev(probs)))
      )
    )

    idx_pv <- which(T_stats == T_obs[i])
    res[i] <- pv_supp[idx_pv]
    if(!simple_output) {
      supports[[i]] <- unique(sort(pv_supp))
      indices[[i]]  <- i
    }
  }

  # create output object
  out <- if(!simple_output) {
    dnames <- sapply(match.call(), deparse1)
    stat_name <- if(is.na(dnames["statistic"]))
      deparse1(substitute(statistic)) else dnames["statistic"]
    switch(
      statistic,
      diff_mean = {
        null_label <- "location shift"
        stat_label <- "difference of means"
      },
      diff_median = {
        null_label <- "location shift"
        stat_label <- "difference of medians"
      },
      diff_t = {
        null_label <- "location shift"
        stat_label <- "Student (t)"
      },
      diff_welch = {
        null_label <- "location shift"
        stat_label <- "Welch (t)"
      },
      diff_hl = {
        null_label <- "location shift"
        stat_label <- "Hodges-Lehmann"
      },
      ratio_var = {
        null_label <- "ratio of variances"
        stat_label <- "ratio of variances"
      },
      ratio_sd = {
        null_label <- "ratio of standard deviations"
        stat_label <- "ratio of standard deviations"
      }
    )

    DiscreteTestResults$new(
      test_name = "Permutation test",
      inputs = list(
        observations = list(x, y),
        parameters = NULL,
        nullvalues = data.frame(
          setNames(list(mu), null_label),
          check.names = FALSE
        ),
        computation = Filter(
          function(df) !all(is.na(df)),
          data.frame(
            alternative = alternative,
            exact = use_exact,
            distribution = ifelse(
              use_exact,
              "permutation (exact)",
              "permutation (Monte Carlo)"
            ),
            combinations = ifelse(
              use_exact, as.integer(choose(n_total, nx[i])), NA
            ),
            simulations = ifelse(use_exact, NA, MC_sims),
            check.names = FALSE
          )
        )
      ),
      statistics = data.frame(
        `test statistic type` = stat_label,
        `observed value` = T_obs,
        check.names = FALSE
      ),
      p_values = res,
      pvalue_supports = supports,
      support_indices = indices,
      data_name = dnames[c("x", "y")]
    )
  } else res

  # return results
  return(out)
}

#' @title
#' Discrete Test Results Summary Class
#'
#' @description
#' This is the class used by `DiscreteTests` for summarising
#' [`DiscreteTestResults`] objects. It contains the summarised objects itself,
#' as well as a summary [`tibble`][tibble::tibble()] object as private members.
#' Both can be extracted by public methods.
#'
#' @examples
#' # binomial tests
#' obj <- binom_test_pv(0:5, 5, 0.5)
#' # create DiscreteTestResultsSummary object
#' res <- DiscreteTestResultsSummary$new(obj)
#' # print summary
#' print(res)
#' # extract summary table
#' res$get_summary_table()
#'
#' @importFrom R6 R6Class
#' @importFrom checkmate assert_r6
#' @importFrom cli cli_text cli_verbatim
#' @importFrom tibble add_column as_tibble rowid_to_column
#' @importFrom withr with_options
#' @export
DiscreteTestResultsSummary <- R6Class(
  "summary.DiscreteTestResults",

  ## public ----

  public = list(
    #' @description
    #' Creates a new `summary.DiscreteTestResults` object.
    #'
    #' @param test_results   the [`DiscreteTestResults`] class object to be
    #'                       summarised.
    initialize = function(test_results) {
      # make sure that the results object is of class 'DiscreteTestResults'
      assert_r6(test_results, "DiscreteTestResults")

      # get test results
      inputs <- test_results$get_inputs(unique = FALSE)
      pvals  <- test_results$get_pvalues(named = TRUE)

      # if observations are samples, set names (and re-arrange if multi-samples)
      if(is.list(inputs$observations) && !is.data.frame(inputs$observations)) {
        # determine if one- or multi-sample test
        len_obs <- length(inputs$observations)
        if(len_obs > 1L && is.list(inputs$observations[[1]])) {
          # multi-samples
          names_obs <- paste("sample", seq_len(len_obs))
          # start summary table
          summary_table <- inputs$observations
        } else {
          # single samples
          names_obs <- "sample"
          # start summary table
          summary_table <- list(inputs$observations)
        }
      } else {
        # single observations
        names_obs <- names(inputs$observations)
        # start summary table
        summary_table <- as.list(inputs$observations)
      }

      # compile data as list, ensure desired order
      summary_table <- c(
        list(pvals),
        summary_table,
        c(
          as.list(inputs$parameters),
          as.list(inputs$nullvalues),
          as.list(inputs$computation)
        )
      )

      # get test designations (a.k.a names) if present
      test_names <- names(pvals)

      # create tibble
      summary_table <- as_tibble(summary_table, .name_repair = "minimal")

      # set column headers
      names(summary_table) <- c(
        "p-value",
        names_obs,
        names(inputs$parameters),
        names(inputs$nullvalues),
        names(inputs$computation)
      )

      # add ID column
      summary_table <- if(!is.null(test_names))
        add_column(summary_table, ID = test_names, .before = 1) else
          rowid_to_column(summary_table, "ID")

      # assign inputs
      private$test_results  <- test_results
      private$summary_table <- summary_table
    },

    #' @description
    #' Returns the underlying [DiscreteTestResults] object.
    #'
    #' @return
    #' A [DiscreteTestResults] R6 class object.
    get_test_results = function() {
      return(private$test_results)
    },

    #' @description
    #' Returns the summary table of the underlying [DiscreteTestResults] object.
    #' @return
    #' A data frame.
    get_summary_table = function() {
      return(private$summary_table)
    },

    #' @description
    #' Prints the summary.
    #'
    #' @param ...  further arguments passed to `print.data.frame`.
    #'
    #' @return
    #' Prints a summary table of the tested null hypotheses. The object itself
    #' is invisibly returned.
    print = function(...) {
      print(private$test_results, FALSE, FALSE, FALSE, limit = 0)
      cli_verbatim("\n")
      cli_h2("Summary")
      out_table <- with_options(
        list(crayon.enabled = TRUE),
        capture.output(print(private$summary_table, ...))
      )
      cli_verbatim(out_table)
      cat("\n")

      invisible(self)
    }
  ),

  ## private ----

  private = list(
    # DiscreteTestResults R6 class object
    test_results = NULL,

    # summary table (a data frame)
    summary_table = data.frame()
  )
)

#' @title
#' Summarizing Discrete Test Results
#'
#' @description
#' `summary` method for class [`DiscreteTestResults`].
#'
#' @param object   object of class [`DiscreteTestResults`] to be summarised;
#'                 usually created by using one of the packages test functions,
#'                 e.g. [binom_test_pv()], with `simple_output = FALSE`.
#' @param ...      further arguments passed to or from other methods.
#'
#' @return
#' A [`summary.DiscreteTestResults`][DiscreteTestResultsSummary] R6 class
#' object.
#'
#' @examples
#' # binomial tests
#' obj <- binom_test_pv(0:5, 5, 0.4)
#' # print summary
#' summary(obj)
#' # extract summary table
#' smry <- summary(obj)
#' smry$get_summary_table()
#'
#' @importFrom checkmate assert_r6
#' @export
## S3 method for class 'DiscreteTestResults'
summary.DiscreteTestResults <- function(object, ...) {
  assert_r6(object, "DiscreteTestResults")

  DiscreteTestResultsSummary$new(object)
}

#' @name mann_whitney_test_pv
#'
#' @title
#' Wilcoxon-Mann-Whitney *U* test
#'
#' @description
#' `mann_whitney_test_pv()` performs an exact or approximate
#' Wilcoxon-Mann-Whitney *U* test about the location shift between two
#' independent groups when the data is not necessarily normally distributed. In
#' contrast to [`stats::wilcox.test()`], it is vectorised and only calculates
#' *p*-values. Furthermore, it is capable of returning the discrete *p*-value
#' supports, i.e. all observable *p*-values under a null hypothesis. Multiple
#' tests can be evaluated simultaneously.
#'
#' @param x,y     numerical vectors forming the samples to be tested or lists
#'                of numerical vectors for multiple tests.
#' @param mu      numerical vector or single number of hypothesised location
#'                shift(s).
#'
#' @template param
#' @templateVar alternative TRUE
#' @templateVar exact_w TRUE
#' @templateVar correct_w TRUE
#' @templateVar simple_output TRUE
#' @templateVar digits_rank TRUE
#'
#' @details
#' We use a test statistic called the Wilcoxon Rank Sum Statistic, defined by
#' \deqn{U = \sum_{i = 1}^{n_X}{rank(X_i)} - \frac{n_X(n_X + 1)}{2},}
#' where \eqn{rank(X_i)} is the rank of \eqn{X_i} in the concatenated sample
#' of \eqn{X} and \eqn{Y}, and \eqn{n_X} and \eqn{n_Y} are the respective
#' sizes of the samples \eqn{X} and \eqn{Y}. Note that \eqn{U}
#' can range from \eqn{0} to \eqn{n_X \cdot n_Y}.
#' This is the same statistic used by [`stats::wilcox.test()`] and
#' whose distribution is accessible with [`pwilcox`].
#' This is also the statistic defined by the two given references.
#' Note, however, that it is not what is called the Mann-Whitney U Statistic
#' in the (English-language) Wikipedia article (as of February 12, 2026). The
#' latter is defined as, using our notation, \eqn{\min(U, n_X \cdot n_Y - U)}.
#' Using the Wikipedia notation, the Wilcoxon Rank Sum Statistic is \eqn{U_2}.
#'
#' The parameters `x`, `y`, `mu` and `alternative` are vectorised. If `x` and
#' `y` are lists, they are replicated automatically to have the same lengths. In
#' case `x` or `y` are not lists, they are added to new ones, which are then
#' replicated to the appropriate lengths. This allows multiple hypotheses to be
#' tested simultaneously.
#'
#' In the presence of ties, computation of the Edgeworth series (up to
#' `correct = 3`) is not possible. Therefore, numeric values of `correct`
#' are ignored and only a continuity correction is performed.
#'
#' By setting `exact = NULL`, exact computation is performed only if both
#' samples sizes in a test setting are lower than or equal to 200. Otherwise,
#' \eqn{p}-values are computed by normal approximation.
#'
#' If `digits_rank = Inf` (the default), [`rank()`][`base::rank()`] is used to
#' compute ranks for the tests statistics instead of
#' [`rank`][`base::rank()`]([`signif(., digits_rank)`][`base::signif()`])
#'
#' @template return
#'
#' @seealso
#' [`stats::wilcox.test()`], [`pwilcox`], [`wilcox_test_pv()`]
#'
#' @references
#' Mann, H. D. & Whitney, D. R. (1947). On a Test of Whether one of Two Random
#'   Variables is Stochastically Larger than the Other. *Ann. Math. Statist.*,
#'   *18*(1), pp. 50-60. \doi{10.1214/aoms/1177730491}
#'
#' Hollander, M. & Wolfe, D. (1973). *Nonparametric Statistical Methods*. Third
#'   Edition. New York: Wiley. pp. 115-135. \doi{10.1002/9781119196037}
#'
#' @examples
#' # Constructing
#' set.seed(1)
#' r1 <- rnorm(100)
#' r2 <- rnorm(100, 1)
#'
#' # Exact two-sided p-values and their supports
#' results_ex  <- mann_whitney_test_pv(r1, r2)
#' print(results_ex)
#' results_ex$get_pvalues()
#' results_ex$get_pvalue_supports()
#'
#' # Normal-approximated one-sided p-values ("less") and their supports
#' results_ap  <- mann_whitney_test_pv(r1, r2, alternative = "less", exact = FALSE)
#' print(results_ap)
#' results_ap$get_pvalues()
#' results_ap$get_pvalue_supports()
#'
#' @importFrom checkmate qassert qassertr
#' @importFrom cli cli_warn
#' @export
mann_whitney_test_pv <- function(
  x,
  y,
  mu = 0,
  alternative = "two.sided",
  exact = NULL,
  correct = TRUE,
  digits_rank = Inf,
  simple_output = FALSE
) {
  # plausibility checks of input parameters
  qassert(x, c("N+", "L+"))
  if(!is.list(x)) x <- list(x) else qassertr(x, "N+")
  len_x <- length(x)

  qassert(y, c("N+", "L+"))
  if(!is.list(y)) y <- list(y) else qassertr(y, "N+")
  len_y <- length(y)

  qassert(mu, "N+()")
  len_m <- length(mu)

  len_a <- length(alternative)
  for(i in seq_len(len_a)){
    alternative[i] <- match.arg(
      tolower(alternative[i]),
      c("two.sided", "less", "greater")
    )
  }

  qassert(exact,   c("B1", "0"))

  qassert(correct, c("B1", "X1[0, 3]"))
  if(!is.logical(correct) && (is.null(exact) || (!is.null(exact) && !exact))) {
    edgeworth <- round(correct)
    correct   <- TRUE
  } else {
    if(!is.null(exact) && exact && is.numeric(correct)) correct <- TRUE
    edgeworth <- 0
  }

  qassert(digits_rank, "N1")

  qassert(simple_output, "B1")

  # replicate inputs to same length
  len_g <- max(len_x, len_y, len_m, len_a)
  if(len_x < len_g) x <- rep_len(x, len_g)
  if(len_y < len_g) y <- rep_len(y, len_g)
  if(len_m < len_g) mu <- rep_len(mu, len_g)
  if(len_a < len_g) alternative <- rep_len(alternative, len_g)

  # compute ranks and lengths
  nx    <- integer(len_g)
  ny    <- integer(len_g)
  N     <- integer(len_g)
  nn    <- integer(len_g)
  U     <- numeric(len_g)
  means <- numeric(len_g)
  sds   <- numeric(len_g)
  ties  <- logical(len_g)
  stats <- vector("list", len_g)
  for(i in seq_len(len_g)) {
    nx[i] <- length(x[[i]])
    ny[i] <- length(y[[i]])
    N[i]  <- nx[i] + ny[i]
    nn[i] <- nx[i] * ny[i]

    ranks <- if(is.finite(digits_rank))
      rank(signif(c(x[[i]] - mu[i], y[[i]]), digits_rank)) else
        rank(c(x[[i]] - mu[i], y[[i]]))

    U[i] <- sum(ranks[seq_len(nx[i])]) - nx[i] * (nx[i] + 1) / 2
    ties[i] <- length(ranks) != length(unique(ranks))
    if(ties[i]) stats[[i]] <- as.integer(round(2 * ranks))

    means[i] <- nn[i] / 2
    if(
      is.null(exact) && (nx[i] > 200 || ny[i] > 200) ||
      !is.null(exact) && !exact
    ) {
      sds[i] <- means[i] * (N[i] + 1) / 6
      if(ties[i]) {
        t <- table(ranks)
        sds[i] <- sds[i] - means[i] * sum(t^3 - t)/(6 * N[i] * (N[i] - 1))
      }
    }
  }

  ex <- if(is.null(exact)) nx < 201 & ny < 201 else rep(exact, len_g)
  ew <- edgeworth & !ex & !ties
  ew[ex | ties] <- NA

  # compute Edgeworth coefficients for normal approximations, if desired
  idx_ew <- which(ew)
  if(length(idx_ew)) {
    ew_coefs <- matrix(NA_real_, len_g, edgeworth)
    if(edgeworth >= 1)
      ew_coefs[idx_ew, 1] <- -(N[idx_ew]^2 - nn[idx_ew] + N[idx_ew]) /
        (20 * nn[idx_ew] * (N[idx_ew] + 1))
    if(edgeworth >= 2)
      ew_coefs[idx_ew, 2] <- (
        2*(nx[idx_ew]^4 + ny[idx_ew]^4) +
        4*(
          nn[idx_ew]*(nx[idx_ew]^2 + ny[idx_ew]^2) + nx[idx_ew]^3 + ny[idx_ew]^3
        ) +
        6*nx[idx_ew]^2 * ny[idx_ew]^2 +
        N[idx_ew] * (7*nn[idx_ew] + N[idx_ew] - 1)
      ) / (210 * nx[idx_ew]^2 * ny[idx_ew]^2 * (N[idx_ew] + 1)^2)
    if(edgeworth == 3)
      ew_coefs[idx_ew, 3] <- ew_coefs[idx_ew, 1]^2 / 2
  }

  # determine unique parameter sets
  params    <- data.frame(alternative, nx, ny, means, sds, ew, ties)
  params_ex <- unique(subset(params, ex & !ties, 1:3))
  params_ti <- subset(params, ex & ties)
  params_ap <- unique(subset(params, !ex, -(2:3)))
  idx_ex    <- as.numeric(rownames(params_ex))
  idx_ti    <- as.numeric(rownames(params_ti))
  idx_ap    <- as.numeric(rownames(params_ap))
  rows      <- c(idx_ex, idx_ti, idx_ap)
  params_u  <- params[rows, ]
  if(any(ew, na.rm = TRUE)) ew_coefs_u <- ew_coefs[rows, , drop = FALSE]

  len_ex <- length(idx_ex)
  len_ti <- length(idx_ti)
  len_ap <- length(idx_ap)
  idx_ex <- seq_len(len_ex)
  idx_ap <- len_ex + len_ti + seq_len(len_ap)
  len_u  <- len_ex + len_ti + len_ap

  alts_u <- params_u$alternative
  nx_u   <- params_u$nx
  ny_u   <- params_u$ny
  mean_u <- params_u$means
  sd_u   <- params_u$sds
  ew_u   <- params_u$ew
  ties_u <- params_u$ties

  # prepare output
  res <- numeric(len_g)
  if(!simple_output) {
    supports <- vector("list", len_u)
    indices  <- vector("list", len_u)
  }

  # pre-compute exact distributions (if any)
  sizes_ex <- unique(data.frame(nx_u, ny_u)[idx_ex, ])
  d <- generate_mann_whitney_probs(sizes_ex[, 1], sizes_ex[, 2])

  # begin exact computations for settings without ties (if any)
  for(i in idx_ex) {
    idx_supp <- which(
      alts_u[i] == alternative & nx_u[i] == nx & ny_u[i] == ny & ex & !ties
    )

    idx_d <- which(sizes_ex[, 1] == nx_u[i] & sizes_ex[, 2] == ny_u[i])

    if(simple_output) {
      # compute p-values directly
      res[idx_supp] <- switch(
        EXPR = alts_u[i],
        less = p_from_d(U[idx_supp], d[[idx_d]]),
        greater = p_from_d(U[idx_supp] - 1, d[[idx_d]], FALSE),
        two.sided = {
          idx_l <- which(U[idx_supp] < mean_u[i])
          idx_u <- which(U[idx_supp] >= mean_u[i])
          pv <- numeric(length(idx_supp))
          if(length(idx_l))
            pv[idx_l] <- p_from_d(U[idx_supp][idx_l], d[[idx_d]])
          if(length(idx_u))
            pv[idx_u] <- p_from_d(U[idx_supp][idx_u] - 1, d[[idx_d]], FALSE)
          pmin(1, 2 * pv)
        }
      )
    } else {
      # compute p-value support
      pv_supp <- support_exact(
        alternative = alts_u[i],
        probs = d[[idx_d]]
      )

      # store results and support
      res[idx_supp] <- pv_supp[U[idx_supp] + 1]
      supports[[i]] <- unique(sort(pv_supp))
      indices[[i]]  <- idx_supp
    }
  }

  # begin exact computations for settings with ties (if any)
  idx_out <- len_ex + 1
  for(i in idx_ti) {
    # compute exact distribution with ties
    dist <- numerical_adjust(mann_whitney_probs_ties_int(stats[[i]], nx[i]))
    min_stat <- sum(sort(stats[[i]])[seq_len(nx[i])])
    R <- 2 * U[i] + nx[i] * (nx[i] + 1) - min_stat

    if(simple_output) {
      # compute p-values directly
      res[i] <- switch(
        EXPR = alternative[i],
        less = p_from_d(R, dist),
        greater = p_from_d(R - 1, dist, FALSE),
        two.sided = pmin(1, 2 * p_from_d(
          R - U[i] > means[i], dist, U[i] <= means[i]
        ))
      )
    } else {
      # compute p-value support
      pv_supp <- support_exact(
        alternative = alternative[i],
        probs = dist
      )

      # store results and support
      res[i]              <- pv_supp[R + 1]
      supports[[idx_out]] <- unique(sort(pv_supp))
      indices[[idx_out]]  <- i
      idx_out             <- idx_out + 1
    }
  }

  # begin approximation computations (if any)
  for(i in idx_ap) {
    idx_supp <- which(
      alts_u[i] == alternative & !ex & ties_u[i] == ties &
        mean_u[i] == means & sd_u[i] == sds
    )

    ew_ok <- !is.na(ew_u[i]) && ew_u[i] && !ties_u[i]
    e <- if(ew_ok) ew_coefs_u[i, ]

    if(simple_output) {
      res[idx_supp] <- switch(
        EXPR = alts_u[i],
        less = pnorm_MW_edgeworth(
          U[idx_supp], mean_u[i], sd_u[i], TRUE, correct, e
        ),
        greater = pnorm_MW_edgeworth(
          U[idx_supp], mean_u[i], sd_u[i], FALSE, correct, e
        ),
        two.sided = pmin(1, 2 * if(ew_ok)
          pmin(
            pnorm_MW_edgeworth(
              U[idx_supp], mean_u[i], sd_u[i], TRUE, correct, e
            ),
            pnorm_MW_edgeworth(
              U[idx_supp], mean_u[i], sd_u[i], FALSE, correct, e
            )
          ) else pmin(1,
            pnorm(-abs(U[idx_supp] - mean_u[i]), -correct * 0.5, sd_u[i])
          )
        )
      )
    } else {
      # compute p-value support
      z <- if(!any(ties[idx_supp])) 0L:(nx_u[i] * ny_u[i]) else
        seq(0, nx_u[i] * ny_u[i], 0.5)
      pv_supp <- switch(
        EXPR = alts_u[i],
        less = pnorm_MW_edgeworth(z, mean_u[i], sd_u[i], TRUE, correct, e),
        greater = pnorm_MW_edgeworth(z, mean_u[i], sd_u[i], FALSE, correct, e),
        two.sided = pmin(1, 2 * if(ew_ok)
          pmin(
            pnorm_MW_edgeworth(z, mean_u[i], sd_u[i], TRUE, correct, e),
            pnorm_MW_edgeworth(z, mean_u[i], sd_u[i], FALSE, correct, e)
          ) else pnorm(-abs(z - mean_u[i]), -correct * 0.5, sd_u[i])
        )
      )

      # store results and support
      idx_stat <- 1 + if(!any(ties[idx_supp]))
        U[idx_supp] else round(2 * U[idx_supp])
      res[idx_supp] <- pv_supp[idx_stat]
      if(!simple_output) {
        supports[[i]] <- unique(sort(pv_supp))
        indices[[i]]  <- idx_supp
      }
    }
  }

  # re-arrange supports and indices in original order of hypotheses
  if(!simple_output) {
    ord <- order(sapply(indices, "[", 1))
    supports <- supports[ord]
    indices  <- indices[ord]
  }

  # create output object
  out <- if(!simple_output) {
    DiscreteTestResults$new(
      test_name = "Wilcoxon-Mann-Whitney U test",
      inputs = list(
        observations = list(x, y),
        parameters = NULL,
        nullvalues = data.frame(`location shift` = mu, check.names = FALSE),
        computation = Filter(
          function(df) !all(is.na(df)),
          data.frame(
            alternative = alternative,
            exact = ex,
            distribution = ifelse(
              ex,
              paste0(
                "Wilcoxon-Mann-Whitney", ifelse(ties, " (tie-adjusted)", "")
              ),
              "normal"
            ),
            #distribution.mean = ifelse(!ex, means, NA_real_),
            #distribution.sd = ifelse(!ex, sds, NA_real_),
            `continuity correction` = ifelse(ex, NA, correct),
            `Edgeworth expansion` = ew,
            `Edgeworth series terms` = ifelse(ew, ew * edgeworth, NA),
            `size of first sample` = nx,
            `size of second sample` = ny,
            ties = ties,
            check.names = FALSE
          )
        )
      ),
      statistics = data.frame(U),
      p_values = res,
      pvalue_supports = supports,
      support_indices = indices,
      data_name = sapply(match.call(), deparse1)[c("x", "y")]
    )
  } else res

  # return results
  return(out)
}

#' @importFrom stats stepfun
pvalues_by_statistics <- function(statistics, probs, decreasing = FALSE, normalize = FALSE){
  stats_order <- order(statistics, decreasing = decreasing)
  statistics <- statistics[stats_order]
  probs <- probs[stats_order]
  probs_cumulated <- cumsum(probs)
  probs_cumulated[probs_cumulated > 1] <- 1
  probs_cumulated[length(probs_cumulated)] <- 1
  order_for_stepfun <- if(decreasing) order(statistics) else seq_along(statistics)
  probs_stepfun <- stepfun(statistics[order_for_stepfun],
                           c(0, probs_cumulated[order_for_stepfun]))
  pvalues <- probs_stepfun(statistics)

  if(normalize) pvalues <- pvalues/max(pvalues)
  return(pvalues[order(stats_order)])
}

numerical_adjust <- function(values, normalize = TRUE, rel.tol = .Machine$double.eps * 128) {
  values_order <- order(values)
  values <- values[values_order]
  idx_duplicates <- which(duplicated(values))
  values_unique <- unique(values)
  len_values_unique <- length(values_unique) - 1
  rel_diffs <- (values_unique[1:len_values_unique + 1] - values_unique[1:len_values_unique]) / values_unique[1:len_values_unique + 1]
  idx_diffs <- which(rel_diffs <= rel.tol & rel_diffs > 0)
  len_diffs <- length(idx_diffs)
  for(i in seq_len(len_diffs)) {
    j <- 1

    while(
      i < len_diffs && i + j <= len_diffs &&
      idx_diffs[i] + j <= len_values_unique &&
      idx_diffs[i + j] == idx_diffs[i] + j
    ) j <- j + 1

    values_unique[idx_diffs[i] + 0:j] <- mean(values_unique[idx_diffs[i] + 0:j])
  }
  if(length(idx_duplicates)) {
    values[-idx_duplicates] <- values_unique
    values[idx_duplicates] <- -Inf
    values <- cummax(values)
  } else values <- values_unique
  values <- values[order(values_order)]

  if(normalize) {
    sum_values <- sum(values)
    if(sum_values <= 0){
      values <- exp(values)
      sum_values <- sum(values)
    }
    counter <- 1
    while(sum_values != 1 && (sum_values > 1 || counter <= 10)) {
      values <- values/sum_values
      sum_values <- sum(values)
      counter <- counter + 1
    }
    #if(sum_values > 1 && counter > 10)
    #  warning("Sum of probabilities slightly exceeds 1. Normalisation attempt failed.\n")
  }

  return(values)
}

#' @importFrom stats dbinom
generate_binom_probs <- function(size, prob, log = FALSE){
  probability_masses <- numerical_adjust(dbinom(0:size, size, prob))

  if(log) return(log(probability_masses)) else return(probability_masses)
}

#' @importFrom stats dpois qpois
generate_poisson_probs <- function(lambda, log = FALSE){
  # search for last observation with P(X = limit) > 0
  limit <- ifelse(lambda == 0, 0, qpois(2^-1074, lambda, FALSE))
  probability_masses <- numerical_adjust(dpois(0:limit, lambda))

  if(log) return(log(probability_masses)) else return(probability_masses)
}

#' @importFrom stats dsignrank
generate_sign_rank_probs <- function(n, log = FALSE) {
  len <- length(n)
  if(len == 0) return(list())

  ord_n <- order(n)
  n <- n[ord_n]
  max_n <- n[len]

  res <- sign_rank_probs_int(n)[order(ord_n)]

  for(i in seq_len(len)) {
    res[[i]] <- if(log)
      log(numerical_adjust(res[[i]])) else
        numerical_adjust(res[[i]])
  }

  return(res)
}

generate_mann_whitney_probs <- function(m, n, log = FALSE) {
  len <- length(m)
  if(len == 0) return(list())
  sizes <- cbind(pmax(m, n), pmin(m, n))

  ord_n <- order(sizes[, 2])
  sizes <- sizes[ord_n, , drop = FALSE]
  ord_m <- order(sizes[, 1])
  sizes <- sizes[ord_m, , drop = FALSE]
  m <- as.integer(sizes[, 1])
  n <- as.integer(sizes[, 2])

  res <- mann_whitney_probs_int(m, n)[order(ord_m)][order(ord_n)]

  for(i in seq_len(len)) {
    res[[i]] <- if(log)
      log(numerical_adjust(res[[i]])) else
        numerical_adjust(res[[i]])
  }

  return(res)
}

support_exact <- function(alternative, probs, expectations = NULL){
  support <- pmin(
    1,
    switch(
      EXPR    = alternative,
      less    = c(cumsum(probs[-length(probs)]), 1),
      greater = c(1, rev(cumsum(rev(probs[-1])))),
      minlike = pvalues_by_statistics(statistics = probs, probs = probs),
      blaker  = pvalues_by_statistics(
        statistics = pmin(cumsum(probs), rev(cumsum(rev(probs)))),
        probs = probs
      ),
      absdist = pvalues_by_statistics(
        statistics = expectations,
        probs = probs,
        decreasing = TRUE
      ),
      # default for "two.sided" or "central", i.e. double of smaller tail
      2 * pmin(cumsum(probs), rev(cumsum(rev(probs))))
    )
  )
  return(support)
}

#' @importFrom stats pnorm
support_normal <- function(alternative, x, mean, sd, correct) {
  z <- (x - mean) / sd
  support <- pmin(
    1,
    switch(
      EXPR      = alternative,
      less      = rev(c(1, pnorm(rev(z)[-1] + correct * 0.5 / sd))),
      greater   = c(1, pnorm(z[-1] - correct * 0.5 / sd, lower.tail = FALSE)),
      # defaulting for "two.sided"
      2 * pnorm(-abs(z) + correct * 0.5 / sd)
    )
  )

  return(support)
}

# modification of pnorm that ensures that P(X >= q) is computed for sd = 0 (instead of P(X > q))
#' @importFrom stats pnorm
pnorm_zero <- function(q, sd = 1, lower_tail = TRUE){
  # number of input values
  len <- length(q)
  # vector of results
  res <- numeric(len)
  # make sure 'sd' has the same size as 'q'
  sd <- rep_len(sd, len)
  # 'sd = 0' means we have a single-point distribution at 0
  idx0 <- which(sd == 0)
  # standard normal distribution
  idx1 <- which(sd != 0)
  if(length(idx0)){
    if(lower_tail)
      res[idx0] <- as.numeric(q[idx0] >= 0) else
        res[idx0] <- as.numeric(q[idx0] <= 0)
  }
  if(length(idx1))
    res[idx1] <- pnorm(q[idx1], sd = sd[idx1], lower.tail = lower_tail)

  # return results
  return(res)
}

p_from_d <- function(q, pmf, lower.tail = TRUE) {
  q_pos <- q >= 0
  res <- numeric(length(q))
  if(any(q_pos)) q <- q[q_pos]

  if(lower.tail) {
    if(!all(q_pos)) res[!q_pos] <- 0
    if(any(q_pos)) res[q_pos] <- cumsum(pmf[seq_len(max(q) + 1)])[q + 1]
  } else {
    if(!all(q_pos)) res[!q_pos] <- 1
    if(any(q_pos))
      res[q_pos] <- rev(cumsum(rev(pmf[(min(q) + 2):length(pmf)])))[q - min(q) + 1]
  }
  return(res)
}

#' @importFrom stats dnorm
pnorm_MW_edgeworth <- function(
  x,
  mean = 0,
  sd = 1,
  lower.tail = TRUE,
  correct = TRUE,
  constants = NULL
) {
  level <- length(constants)
  s <- ifelse(correct || level > 0, (-1)^!lower.tail, 0)
  z <- (x - mean + ifelse(correct, s * 0.5, 0)) / sd
  r <- pnorm(z, lower.tail = lower.tail)
  if(level > 0) {
    a <- constants[1]
    if(level > 1) b <- constants[2]
    if(level > 2) c <- constants[3]

    H <- dnorm(z) * switch(
      level,
      a*(z^3 - 3*z),
      b*z^5 - (10*b - a)*z^3 + (15*b - 3*a)*z,
      c*z^7 - (21*c - b)*z^5 + (105*c - 10*b + a)*z^3 - (105*c - 15*b + 3*a)*z
    )

    r <- pmax(0, pmin(1, r - s*H))
  }

  return(r)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

mann_whitney_probs_int <- function(m, n) {
    .Call('_DiscreteTests_mann_whitney_probs_int', PACKAGE = 'DiscreteTests', m, n)
}

mann_whitney_probs_ties_int <- function(ranks, size_x) {
    .Call('_DiscreteTests_mann_whitney_probs_ties_int', PACKAGE = 'DiscreteTests', ranks, size_x)
}

perm_test_run <- function(x, y, mu, method = "diff_mean", exact = TRUE, sims = 100000L) {
    .Call('_DiscreteTests_perm_test_run', PACKAGE = 'DiscreteTests', x, y, mu, method, exact, sims)
}

sign_rank_probs_int <- function(n) {
    .Call('_DiscreteTests_sign_rank_probs_int', PACKAGE = 'DiscreteTests', n)
}

sign_rank_probs_ties_int <- function(scores) {
    .Call('_DiscreteTests_sign_rank_probs_ties_int', PACKAGE = 'DiscreteTests', scores)
}

#' @title
#' Vectorised Computation of P-Values and Their Supports for Several Discrete
#' Statistical Tests
#'
#' @docType package
#' @import Rcpp
#' @useDynLib DiscreteTests
#' @name DiscreteTests-package
#'
#' @description
#' This package provides vectorised functions for computing p-values of various
#' discrete statistical tests. Exact and approximate computation methods are
#' provided. For exact p-values, several procedures of determining two-sided
#' p-values are included.
#'
#' Additionally, these functions are capable of returning the discrete p-value
#' supports, i.e. all observable p-values under a null hypothesis. These
#' supports can be used for multiple testing procedures in the
#' [`DiscreteFDR`][DiscreteFDR::DiscreteFDR-package] and
#' [`FDX`][FDX::FDX-package] packages.
#'
#' @references
#' Fisher, R. A. (1935). The logic of inductive inference.
#'   *Journal of the Royal Statistical Society Series A*, **98**, pp.
#'   39–54. \doi{10.2307/2342435}
#'
#' Agresti, A. (2002). *Categorical data analysis* (2nd ed.). New York: John
#'   Wiley & Sons. \doi{10.1002/0471249688}
#'
#' Blaker, H. (2000) Confidence curves and improved exact confidence intervals
#'   for discrete distributions. *Canadian Journal of Statistics*,
#'   **28**(4), pp. 783-798. \doi{10.2307/3315916}
#'
#' Dixon, W. J. and Mood, A. M. (1946). The statistical sign test.
#'   *Journal of the American Statistical Association*, **41**(236),
#'   pp. 557–566. \doi{10.1080/01621459.1946.10501898}
#'
#' Good, P. (2000). *Permutation Tests: A Practical Guide to Resampling
#'   Methods for Testing Hypotheses*. Second Edition. New York: Springer.
#'   \doi{10.1007/978-1-4757-3235-1}
#'
#' Hirji, K. F. (2006). *Exact analysis of discrete data*. New York: Chapman
#'   and Hall/CRC. pp. 55-83. \doi{10.1201/9781420036190}
#'
#' Hollander, M. & Wolfe, D. (1973). *Nonparametric Statistical Methods*. Third
#'   Edition. New York: Wiley. \doi{10.1002/9781119196037}
#'
#' Mann, H. D. & Whitney, D. R. (1947). On a Test of Whether one of Two Random
#'   Variables is Stochastically Larger than the Other. *Ann. Math. Statist.*,
#'   *18*(1), pp. 50-60. \doi{10.1214/aoms/1177730491}
#'
#' Pitman, E. J. G. (1937). Significance tests which may be applied to
#'   samples from any populations. *Supplement to the Journal of the Royal
#'   Statistical Society*, **4**(1), pp. 119–130. \doi{10.2307/2984124}
#'
"_PACKAGE"

## usethis namespace: start
#' @importFrom lifecycle deprecated
## usethis namespace: end
NULL

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DiscreteTests-package.R
\docType{package}
\name{DiscreteTests-package}
\alias{DiscreteTests}
\alias{DiscreteTests-package}
\title{Vectorised Computation of P-Values and Their Supports for Several Discrete
Statistical Tests}
\description{
This package provides vectorised functions for computing p-values of various
discrete statistical tests. Exact and approximate computation methods are
provided. For exact p-values, several procedures of determining two-sided
p-values are included.

Additionally, these functions are capable of returning the discrete p-value
supports, i.e. all observable p-values under a null hypothesis. These
supports can be used for multiple testing procedures in the
\code{\link[DiscreteFDR:DiscreteFDR-package]{DiscreteFDR}} and
\code{\link[FDX:FDX-package]{FDX}} packages.
}
\references{
Agresti, A. (2002). \emph{Categorical data analysis} (2nd ed.). New York: John
Wiley & Sons. \doi{10.1002/0471249688}

Blaker, H. (2000) Confidence curves and improved exact confidence intervals
for discrete distributions. \emph{Canadian Journal of Statistics},
\strong{28}(4), pp. 783-798. \doi{10.2307/3315916}

Dixon, W. J. and Mood, A. M. (1946). The statistical sign test.
\emph{Journal of the American Statistical Association}, \strong{41}(236),
pp. 557–566. \doi{10.1080/01621459.1946.10501898}

Fisher, R. A. (1935). The logic of inductive inference.
\emph{Journal of the Royal Statistical Society Series A}, \strong{98}, pp.
39–54. \doi{10.2307/2342435}

Good, P. (2000). \emph{Permutation Tests: A Practical Guide to Resampling
Methods for Testing Hypotheses}. Second Edition. New York: Springer.
\doi{10.1007/978-1-4757-3235-1}

Hirji, K. F. (2006). \emph{Exact analysis of discrete data}. New York: Chapman
and Hall/CRC. pp. 55-83. \doi{10.1201/9781420036190}

Hollander, M. & Wolfe, D. (1973). \emph{Nonparametric Statistical Methods}. Third
Edition. New York: Wiley. \doi{10.1002/9781119196037}

Mann, H. D. & Whitney, D. R. (1947). On a Test of Whether one of Two Random
Variables is Stochastically Larger than the Other. \emph{Ann. Math. Statist.},
\emph{18}(1), pp. 50-60. \doi{10.1214/aoms/1177730491}

Pitman, E. J. G. (1937). Significance tests which may be applied to
samples from any populations. \emph{Supplement to the Journal of the Royal
Statistical Society}, \strong{4}(1), pp. 119–130. \doi{10.2307/2984124}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/DISOhda/DiscreteTests}
  \item Report bugs at \url{https://github.com/DISOhda/DiscreteTests/issues}
}

}
\author{
\strong{Maintainer}: Florian Junge \email{diso.fbmn@h-da.de} (\href{https://orcid.org/0009-0001-6856-6938}{ORCID})

Authors:
\itemize{
  \item Florian Junge \email{diso.fbmn@h-da.de} (\href{https://orcid.org/0009-0001-6856-6938}{ORCID})
  \item Christina Kihn
}

Other contributors:
\itemize{
  \item Sebastian Döhler (\href{https://orcid.org/0000-0002-0321-6355}{ORCID}) [contributor]
  \item Guillermo Durand (\href{https://orcid.org/0000-0003-4056-5631}{ORCID}) [contributor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permutation.r
\name{perm_test_pv}
\alias{perm_test_pv}
\title{Permutation Test}
\usage{
perm_test_pv(
  x,
  y,
  statistic = c("diff_mean", "diff_median", "diff_t", "diff_welch", "diff_hl",
    "ratio_var", "ratio_sd"),
  mu = NULL,
  alternative = "two.sided",
  exact = NULL,
  max_exact_combs = 10L^7,
  MC_sims = 10L^5,
  seed = NULL,
  simple_output = FALSE
)
}
\arguments{
\item{x}{numerical vector or list of numerical vectors of
observations in the first group.}

\item{y}{numerical vector or list of numerical vectors of
observations in the second group.}

\item{statistic}{single character giving the type of test statistic
to be used; must be one of \code{"diff_mean"},
\code{"diff_median"}, \code{"diff_t"}, \code{"diff_welch"},
\code{"diff_hl"}, \code{"ratio_var"} or \code{"ratio_sd"}
(see Details).}

\item{mu}{numerical vector giving the hypothesised values
under the null; if \code{is.null(mu)} (the default) it
is set to 0 for test statistics that are based on
differences (\code{diff_}\emph{) or 1 for ratio-based tests
statistics (\code{ratio_}}).}

\item{alternative}{character vector that indicates the alternative hypotheses; each value must be one of \code{"two.sided"} (the default), \code{"less"} or \code{"greater"}.}

\item{exact}{logical value that indicates whether \eqn{p}-values
are to be calculated by exact computation (\code{TRUE})
or by simulation (\code{FALSE}); if \code{NULL} (the default)
exact computation is performed if the number of
possible sample combinations (see Details) is below
or equal to \code{max_exact_combs}; otherwise the
respective \eqn{p}-values are obtained by
simulation.}

\item{max_exact_combs}{maximum number of allowed combinations for exact
computation of the permutation distribution (see
details).}

\item{MC_sims}{positive integer or \code{NULL}. The number of Monte
Carlo permutations to draw when \code{exact = FALSE}.
Ignored when \code{exact = TRUE}. Default is \code{10000L}.}

\item{seed}{single integer or \code{NULL}. Random seed passed to
\code{\link[=set.seed]{set.seed()}} before Monte Carlo sampling to ensure
reproducibility. Use \code{NULL} to skip seed setting.
Ignored when \code{exact = TRUE}.}

\item{simple_output}{logical value that indicates whether an R6 class object, including the tests' parameters and support sets, i.e. all observable p-values under each null hypothesis, is to be returned (see below).}
}
\value{
If \code{simple.output = TRUE}, a vector of computed p-values is returned.
Otherwise, the output is a \code{\link{DiscreteTestResults}} R6 class object, which
also includes the p-value supports and testing parameters. These have to be
accessed by public methods, e.g. \verb{$get_pvalues()}.
}
\description{
\code{perm_test_pv()} performs a permutation test for the comparison of two
independent samples using a user-supplied test statistic. It is vectorised
over multiple test problems: by passing lists of sample vectors for \code{x} and
\code{y}, several tests can be evaluated simultaneously. The function can compute
exact \emph{p}-values (by enumerating all permutations) or approximate \emph{p}-values
via Monte Carlo sampling. It is capable of returning the discrete \emph{p}-value
supports, i.e. all observable \emph{p}-values under the null hypothesis.
}
\details{
Under the null hypothesis of exchangeability (i.e. both samples come from
the same distribution), all \eqn{\binom{n_x + n_y}{n_x}} partitions of the
pooled sample into groups of sizes \eqn{n_x} and \eqn{n_y} are equally
likely. \code{perm_test_pv()} exploits this by:

\enumerate{
\item Computing the observed test statistic \eqn{T_\text{obs}} from
\code{x} and \code{y} using the function selected by \code{statistic} (see the
\emph{Test Statistics} section below).
\item Enumerating (if \code{exact = TRUE}) or randomly sampling (if
\code{exact = FALSE}) permutations of the pooled sample.
\item Evaluating the selected test statistic on each permuted split.
\item Deriving the \emph{p}-value as the fraction of permutation statistics
that are at least as extreme as \eqn{T_\text{obs}}, where \emph{extreme}
is defined by \code{alternative}:
\describe{
\item{\code{"less"}}{fraction with permuted statistic
\eqn{\le T_\text{obs}}}
\item{\code{"greater"}}{fraction with permuted statistic
\eqn{\ge T_\text{obs}}}
\item{\code{"two.sided"}}{fraction with
\eqn{|\text{permuted statistic}| \ge |T_\text{obs}|}}
}
}

Exact computation is only feasible for small pooled samples; the total
number of permutations grows as \eqn{\binom{n_x + n_y}{n_x}}. If
\code{exact = NULL}, exact computation is applied when the number of computations
is below or equal to \code{max_exact_combs} (default \eqn{10{,}000{,}000}).
Otherwise, the distribution of the test statistics is approximated by Monte
Carlo simulation. When \code{exact = TRUE} and the number of permutations exceeds
\code{max_exact_combs}, an error is raised. Set \code{exact = FALSE} to use Monte Carlo
sampling in such cases.

The parameters \code{x}, \code{y}, \code{alternative}, and \code{MC_sims} are vectorised via
lists: if \code{x} and \code{y} are lists of the same length, each pair
\verb{(x[[i]], y[[i]])} defines one test. Scalars and single vectors are
recycled to the required length.
\subsection{Test Statistics}{

The \code{statistic} argument selects among seven built-in test statistics for
comparing two groups. Let \eqn{\bar{x}}, \eqn{\bar{y}} denote the group
means, \eqn{s_x^2}, \eqn{s_y^2} the sample variances, and \eqn{n_x},
\eqn{n_y} the group sizes.

\describe{
\item{\code{"diff_mean"} — Difference of means
\eqn{T = \bar{x} - \bar{y} - \mu_0}}{
The most common choice for location comparisons. Tests whether the
population means differ by \eqn{\mu_0} (default: 0).\cr
\emph{Advantages}: Intuitive interpretation; optimal power under normality
and equal variances (Pitman efficiency 1 vs. the two-sample
\eqn{t}-test).\cr
\emph{Disadvantages}: Sensitive to outliers; less powerful than
\code{"diff_hl"} or \code{"diff_t"} under heavy-tailed distributions.}
\item{\code{"diff_median"} — Difference of medians
\eqn{T = \text{median}(x) - \text{median}(y) - \mu_0}}{
Tests for a shift in the median rather than the mean (default:
\eqn{\mu_0 = 0)}.\cr
\emph{Advantages}: Robust to outliers and skewed distributions; directly
interpretable in terms of the median.\cr
\emph{Disadvantages}: The sample median is a step function of the data,
leading to a coarser permutation distribution with many ties; can
have lower power than \code{"diff_hl"} for continuous data.}
\item{\code{"diff_t"} — Pooled two-sample \eqn{t}-statistic
\eqn{T = \frac{\bar{x} - \bar{y} - \mu_0}{s_p \sqrt{1/n_x + 1/n_y}}}
where
\eqn{s_p = \sqrt{\frac{(n_x-1)s_x^2 + (n_y-1)s_y^2}{n_x+n_y-2}}}}{
Scales the mean difference by the pooled standard deviation,
analogous to the classical Student \eqn{t}-test.\cr
\emph{Advantages}: Studentisation makes the statistic (approximately)
scale-free; useful when group variances are expected to be equal;
the permutation \eqn{p}-value is valid even without normality.\cr
\emph{Disadvantages}: Assumes equal variances (homoscedasticity);
sensitive to outliers; no power gain over \code{"diff_mean"} in the
permutation setting when sample sizes are equal.}
\item{\code{"diff_welch"} — Welch \eqn{t}-statistic
\eqn{T = \frac{\bar{x} - \bar{y} - \mu_0}{\sqrt{s_x^2/n_x + s_y^2/n_y}}}}{
Scales the mean difference by the unpooled standard error, analogous
to Welch's two-sample \eqn{t}-test. Unlike \code{"diff_t"}, the variances
of the two groups are estimated separately rather than pooled.\cr
\emph{Advantages}: Does not assume equal variances; retains more power
than \code{"diff_t"} when group variances are unequal and sample sizes
differ; in the permutation setting the \eqn{p}-value remains exactly
valid regardless of the variance ratio, just as with \code{"diff_t"}.\cr
\emph{Disadvantages}: No power gain over \code{"diff_t"} when variances are
equal; estimating two separate variances instead of one pooled
variance introduces additional estimation uncertainty, which can
slightly reduce power compared to \code{"diff_t"} in balanced designs with
equal variances.}
\item{\code{"diff_hl"} — Hodges–Lehmann estimator
\eqn{T = \text{median}_{i,j}\,(x_i - y_j) - \mu_0}}{
Tests for a shift in the median of all \eqn{n_x \cdot n_y} pairwise
differences between the two groups (default: \eqn{\mu_0 = 0}). It is
the natural companion statistic to the Wilcoxon–Mann–Whitney test and
estimates the location shift \eqn{\Delta} under a shift model.\cr
\emph{Advantages}: Highly robust to outliers; produces fewer ties in the
permutation distribution than \code{"diff_median"}.\cr
\emph{Disadvantages}: \eqn{O(n_x \cdot n_y)} computation; the
interpretation as a median shift requires a location-shift model.}
\item{\code{"ratio_var"} — Ratio of variances
\eqn{T = s_x^2 / (s_y^2 \cdot \mu_0)}}{
Tests for differences in spread (scale) rather than location.
Under the null \eqn{H_0\colon \sigma_x^2 / \sigma_y^2 = \mu_0}
(default: \eqn{\mu_0 = 1}).\cr
\emph{Advantages}: Direct measure of relative variability; permutation
validity does not require normality (unlike the classical
\eqn{F}-test).\cr
\emph{Disadvantages}: Highly sensitive to outliers and non-normality
(the variance itself is not robust); should be interpreted with
caution if the distributions differ in shape as well as scale.}
\item{\code{"ratio_sd"} — Ratio of standard deviations
\eqn{T = s_x / (s_y \cdot \mu_0)}}{
Equivalent to \code{"ratio_var"} on the standard deviation scale
(\eqn{T = \sqrt{s_x^2 / (s_y^2 \cdot \mu_0^2)}}); tests the same null
hypothesis \eqn{H_0 \colon \sigma_x / \sigma_y = \mu_0} (default:
\eqn{\mu_0 = 1}).\cr
\emph{Advantages}: Same unit as the original data, which can aid
interpretation; monotone transformation of \code{"ratio_var"}, so the
\eqn{p}-values are identical (note that \eqn{\mu_0} needs to be
squared for the variance-based statistic to be equivalent).\cr
\emph{Disadvantages}: Same sensitivity to outliers as \code{"ratio_var"};
the monotone relationship means it carries no additional statistical
information compared to \code{"ratio_var"}.
}
}
}
}
\examples{
set.seed(42)
x1 <- rnorm(8)
y1 <- rnorm(10, mean = 1)

# Two-sided test for difference of means = 0
results_ex <- perm_test_pv(x1, y1)
print(results_ex)
results_ex$get_pvalues()

# Hodges Lehmann statistic with Monte Carlo approximation
results_hl <- perm_test_pv(x1, y1, "diff_hl", exact = FALSE, seed = 1L)
results_hl$print()
results_hl$get_pvalues()

# Multiple tests simultaneously, one-sided alternative (mu = 0.5),
# using t-statistic, forced exact computation
xs <- list(rnorm(12), rnorm(9, 1))
ys <- list(rnorm(11, 1), rnorm(10, 2))
results_multi <- perm_test_pv(xs, ys, "diff_t", c(0.5, -1.5), "greater", TRUE)
results_multi$print()
results_multi$get_pvalues()

}
\references{
Pitman, E. J. G. (1937). Significance tests which may be applied to
samples from any populations. \emph{Supplement to the Journal of the Royal
Statistical Society}, \strong{4}(1), pp. 119–130. \doi{10.2307/2984124}

Good, P. (2000). \emph{Permutation Tests: A Practical Guide to Resampling
Methods for Testing Hypotheses}. Second Edition. New York: Springer.
\doi{10.1007/978-1-4757-3235-1}
}
\seealso{
\code{\link[=mann_whitney_test_pv]{mann_whitney_test_pv()}}, \code{\link[stats:wilcox.test]{stats::wilcox.test()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-results.R
\name{DiscreteTestResults}
\alias{DiscreteTestResults}
\title{Discrete Test Results Class}
\description{
This is the class used by the statistical test functions of this package for
returning not only p-values, but also the supports of their distributions and
the parameters of the respective tests. Objects of this class are obtained by
setting the \code{simple.output} parameter of a test function to \code{FALSE} (the
default). All data members of this class are private to avoid inconsistencies
by deliberate or inadvertent changes by the user. However, the results can be
read by public methods.
}
\examples{
## one-sided binomial test
#  parameters
x <- 2:4
n <- 5
p <- 0.4
m <- length(x)
#  support (same for all three tests) and p-values
support <- sapply(0:n, function(k) binom.test(k, n, p, "greater")$p.value)
pv <- support[x + 1]
#  DiscreteTestResults object
res <- DiscreteTestResults$new(
  # string with name of the test
  test_name = "Exact binomial test",
  # list of data frames
  inputs = list(
    observations = data.frame(
      `number of successes` = x,
      # no name check of column header to have a speaking name for 'print'
      check.names = FALSE
    ),
    parameters = data.frame(
      # parameter 'n', needs to be replicated to length of 'x'
      `number of trials` = rep(n, m),
      # no name check of column header to have a speaking name for 'print'
      check.names = FALSE
    ),
    nullvalues = data.frame(
      # here: only one null value, 'p'; needs to be replicated to length of 'x'
      `probability of success` = rep(p, m),
      # no name check of column header to have a speaking name for 'print'
      check.names = FALSE
    ),
    computation = data.frame(
      # mandatory parameter 'alternative', needs to be replicated to the length of 'x'
      alternative = rep("greater", m),
      # mandatory exactness information, replicated to the length of 'alternative'
      exact = rep(TRUE, m),
      # mandatory distribution information, replicated to the length of 'alternative'
      distribution = rep("binomial", m)
    )
  ),
  # test statistics (not needed, since observation itself is the statistic)
  statistics = NULL,
  # numerical vector of p-values
  p_values = pv,
  # list of supports (here: only one support); values must be sorted and unique
  pvalue_supports = list(unique(sort(support))),
  # list of indices that indicate which p-value/hypothesis each support belongs to
  support_indices = list(1:m),
  # name of input data variables
  data_name = "x, n and p"
)

#  print results without supports
print(res)
#  print results with supports
print(res, supports = TRUE)

}
\section{Methods}{
\subsection{Public methods}{
  \itemize{
    \item \href{#method-DiscreteTestResults-initialize}{\code{DiscreteTestResults$new()}}
    \item \href{#method-DiscreteTestResults-get_pvalues}{\code{DiscreteTestResults$get_pvalues()}}
    \item \href{#method-DiscreteTestResults-get_inputs}{\code{DiscreteTestResults$get_inputs()}}
    \item \href{#method-DiscreteTestResults-get_statistics}{\code{DiscreteTestResults$get_statistics()}}
    \item \href{#method-DiscreteTestResults-get_pvalue_supports}{\code{DiscreteTestResults$get_pvalue_supports()}}
    \item \href{#method-DiscreteTestResults-get_support_indices}{\code{DiscreteTestResults$get_support_indices()}}
    \item \href{#method-DiscreteTestResults-print}{\code{DiscreteTestResults$print()}}
    \item \href{#method-DiscreteTestResults-clone}{\code{DiscreteTestResults$clone()}}
  }
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DiscreteTestResults-initialize"></a>}}
\if{latex}{\out{\hypertarget{method-DiscreteTestResults-initialize}{}}}
\subsection{\code{DiscreteTestResults$new()}}{
  Creates a new \code{DiscreteTestResults} object.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DiscreteTestResults$new(
  test_name,
  inputs,
  statistics,
  p_values,
  pvalue_supports,
  support_indices,
  data_name
)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{test_name}}{single character string with the name of the
test(s).}
      \item{\code{inputs}}{named list of \strong{exactly four named} elements
containing the observations, test parameters and
hypothesised null values \strong{as data frames or}
\strong{lists}; the names of these list fields must
be \code{observations}, \code{parameters}, \code{nullvalues}
and \code{computation}. See details for further
information about the requirements for these
fields.}
      \item{\code{statistics}}{data frame containing the tests' statistics;
\code{NULL} is allowed and recommended, e.g. if the
observed values themselves are the statistics.}
      \item{\code{p_values}}{numeric vector of the p-values calculated by
each hypothesis test.}
      \item{\code{pvalue_supports}}{list of \strong{unique} numeric vectors containing
all p-values that are observable under the
respective hypothesis; each value of \code{p_values}
must occur in its respective p-value support.}
      \item{\code{support_indices}}{list of numeric vectors containing the test
indices that indicates to which individual
testing scenario each unique parameter set and
each unique support belongs.}
      \item{\code{data_name}}{single character string with the name of the
variable that contains the observed data.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Details}{
    The fields of the \code{inputs} have the following requirements:
\describe{
\item{\verb{$observations}}{data frame or list of vectors that comprises of
the observed data; if it is a matrix, it must be
converted to a data frame; must not be \code{NULL},
only numerical and character values are
allowed.}
\item{\verb{$nullvalues}}{data frame that holds the hypothesised values
of the tests, e.g. the rate parameters for Poisson
tests; must not be \code{NULL}, only numerical values
are allowed.}
\item{\verb{$parameters}}{data frame that may contain additional parameters
of each test (e.g. numbers of Bernoulli trials for
binomial tests). Only numerical, character or
logical values are permitted; \code{NULL} is allowed,
too, e.g. if there are no additional parameters.}
\item{\verb{$computation}}{data frame that consists of details about the
p-value computation, e.g. if they were calculated
exactly, the used distribution etc. It \strong{must}
include mandatory columns named \code{exact},
\code{alternative} and \code{distribution}. Any additional
information may be added, like the marginals for
Fisher's exact test etc., but only numerical,
character or logical values are allowed.}
}

All data frames must have the same number of rows. Their column names are
used by the \code{print()} method for producing text output, therefore they
should be informative, i.e. short and (if necessary) non-syntactic,
like e.g. \code{`number of success`}.

The mandatory column \code{exact} of the data frame \code{computation} must be
logical, while the values of \code{alternative} must be one of \code{"greater"},
\code{"less"}, \code{"two.sided"}, \code{"minlike"}, \code{"blaker"}, \code{"absdist"} or
\code{"central"}. The \code{distribution} column must hold character strings that
identify the distribution under the null hypothesis, e.g. \code{"normal"}. All
the columns of this data frame are used by the \code{print()} method, so their
names should also be informative and (if necessary) non-syntactic.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DiscreteTestResults-get_pvalues"></a>}}
\if{latex}{\out{\hypertarget{method-DiscreteTestResults-get_pvalues}{}}}
\subsection{\code{DiscreteTestResults$get_pvalues()}}{
  Returns the computed p-values.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DiscreteTestResults$get_pvalues(named = TRUE)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{named}}{single logical value that indicates whether the vector is
to be returned as a named vector (if names are present)}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A numeric vector of the p-values of all null hypotheses.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DiscreteTestResults-get_inputs"></a>}}
\if{latex}{\out{\hypertarget{method-DiscreteTestResults-get_inputs}{}}}
\subsection{\code{DiscreteTestResults$get_inputs()}}{
  Return the list of the test inputs.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DiscreteTestResults$get_inputs(unique = FALSE)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{unique}}{single logical value that indicates whether only unique
combinations of parameter sets and null values are to be
returned. If \code{unique = FALSE} (the default), the returned
data frames may contain duplicate sets.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A list of four elements. The first one contains a data frame with the
observations for each tested null hypothesis, while the second is another
data frame with additional parameters (if any, e.g. \code{n} in case of a
binomial test) that were passed to the respective test's function. The
third list field holds the hypothesised null values (e.g. \code{p} for
binomial tests). The last list element contains computational details,
e.g. test \code{alternative}s, the used \code{distribution} etc. If
\code{unique = TRUE}, only unique combinations of parameters, null values and
computation specifics are returned, but observations remain unchanged
(i.e. they are never unique).
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DiscreteTestResults-get_statistics"></a>}}
\if{latex}{\out{\hypertarget{method-DiscreteTestResults-get_statistics}{}}}
\subsection{\code{DiscreteTestResults$get_statistics()}}{
  Returns the test statistics.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DiscreteTestResults$get_statistics()}
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A numeric \code{data.frame} with one column containing the test statistics.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DiscreteTestResults-get_pvalue_supports"></a>}}
\if{latex}{\out{\hypertarget{method-DiscreteTestResults-get_pvalue_supports}{}}}
\subsection{\code{DiscreteTestResults$get_pvalue_supports()}}{
  Returns the \emph{p}-value supports, i.e. all observable p-values under the
respective null hypothesis of each test.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DiscreteTestResults$get_pvalue_supports(unique = FALSE)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{unique}}{single logical value that indicates whether only unique
p-value supports are to be returned. If \code{unique = FALSE}
(the default), the returned supports may be duplicated.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A list of numeric vectors containing the supports of the p-value null
distributions.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DiscreteTestResults-get_support_indices"></a>}}
\if{latex}{\out{\hypertarget{method-DiscreteTestResults-get_support_indices}{}}}
\subsection{\code{DiscreteTestResults$get_support_indices()}}{
  Returns the indices that indicate to which tested null hypothesis each
unique support belongs.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DiscreteTestResults$get_support_indices()}
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A list of numeric vectors. Each one contains the indices of the null
hypotheses to which the respective support and/or unique parameter set
belongs.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DiscreteTestResults-print"></a>}}
\if{latex}{\out{\hypertarget{method-DiscreteTestResults-print}{}}}
\subsection{\code{DiscreteTestResults$print()}}{
  Prints the computed p-values.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DiscreteTestResults$print(
  inputs = TRUE,
  pvalue_details = TRUE,
  supports = FALSE,
  test_idx = NULL,
  limit = 10,
  ...
)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{inputs}}{single logical value that indicates if the
input values (i.e. observations, statistics and
parameters) are to be printed; defaults to
\code{TRUE}.}
      \item{\code{pvalue_details}}{single logical value that indicates if details
about the p-value computation are to be printed;
defaults to \code{TRUE}.}
      \item{\code{supports}}{single logical value that indicates if the
p-value supports are to be printed; defaults to
\code{FALSE}.}
      \item{\code{test_idx}}{integer vector giving the indices of the tests
whose results are to be printed; if \code{NULL} (the
default), results of every test up to the index
specified by \code{limit} (see below) are printed.}
      \item{\code{limit}}{single integer that indicates the maximum number
of test results to be printed; if \code{limit = 0},
results of every test are printed; ignored if
\code{test_idx} is not set to \code{NULL}}
      \item{\code{...}}{further arguments passed to
\code{\link[base:print.default]{print.default()}}.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    Prints a summary of the tested null hypotheses. The object itself is
invisibly returned.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DiscreteTestResults-clone"></a>}}
\if{latex}{\out{\hypertarget{method-DiscreteTestResults-clone}{}}}
\subsection{\code{DiscreteTestResults$clone()}}{
  The objects of this class are cloneable with this method.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{DiscreteTestResults$clone(deep = FALSE)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{deep}}{Whether to make a deep clone.}
    }
    \if{html}{\out{</div>}}
  }
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/homogeneity.R
\name{homogeneity_test_pv}
\alias{homogeneity_test_pv}
\title{Conditional Two-Sample Homogeneity Test for Binomial Experiments}
\usage{
homogeneity_test_pv(
  x,
  n,
  alternative = "two.sided",
  ts_method = "minlike",
  exact = TRUE,
  correct = TRUE,
  simple_output = FALSE
)
}
\arguments{
\item{x}{integer vector with two elements or a matrix with two columns or a
data frame with two columns giving the number of successes for the
two experiments.}

\item{n}{integer vector with two elements or a matrix with two columns or a
data frame with two columns giving the number of trials for the
two experiments.}

\item{alternative}{character vector that indicates the alternative hypotheses; each value must be one of \code{"two.sided"} (the default), \code{"less"} or \code{"greater"}.}

\item{ts_method, }{single character string that indicates the two-sided p-value computation method (if any value in \code{alternative} equals \code{"two.sided"}) and must be one of \code{"minlike"} (the default), \code{"blaker"}, \code{"absdist"} or \code{"central"} (see details). Ignored, if \code{exact = FALSE}.}

\item{exact}{logical value that indicates whether \eqn{p}-values are to be calculated by exact computation (\code{TRUE}; the default) or by a continuous approximation (\code{FALSE}).}

\item{correct}{logical value that indicates if a continuity correction is to be applied (\code{TRUE}; the default) or not (\code{FALSE}). Ignored, if \code{exact = TRUE}.}

\item{simple_output, }{logical value that indicates whether an R6 class object, including the tests' parameters and support sets, i.e. all observable p-values under each null hypothesis, is to be returned (see below).}
}
\value{
If \code{simple.output = TRUE}, a vector of computed p-values is returned.
Otherwise, the output is a \code{\link{DiscreteTestResults}} R6 class object, which
also includes the p-value supports and testing parameters. These have to be
accessed by public methods, e.g. \verb{$get_pvalues()}.
}
\description{
Performs an exact or approximate conditional test about the homogeneity of
two binomial samples, i.e. regarding the respective probabilities of success.
It is vectorised, only calculates \emph{p}-values and offers a normal
approximation of their computation. Furthermore, it is capable of returning
the discrete \emph{p}-value supports, i.e. all observable \emph{p}-values under a null
hypothesis. Multiple tests can be evaluated simultaneously. In two-sided
tests, several procedures of obtaining the respective \emph{p}-values are
implemented.
}
\details{
The parameters \code{x}, \code{n} and \code{alternative} are vectorised. They are replicated
automatically, such that the number of \code{x}'s rows is the same as the length
of \code{alternative}. This allows multiple null hypotheses to be tested
simultaneously. Since \code{x} and \code{n} are coerced to matrices (if necessary) with
two columns, they are replicated row-wise.

It can be shown that this test is a special case of Fisher's exact test,
because it is conditional on the numbers of trials \strong{and} the sums of
successes and failures. Therefore, its computations are handled by
\code{\link[=fisher_test_pv]{fisher_test_pv()}}.

For exact computation, various procedures of determining two-sided p-values
are implemented.
\describe{
\item{\code{"minlike"}}{The standard approach in \code{\link[stats:fisher.test]{stats::fisher.test()}} and
\code{\link[stats:binom.test]{stats::binom.test()}}. The probabilities of the
likelihoods that are equal or less than the observed one
are summed up. In Hirji (2006), it is referred to as the
\emph{Probability-based} approach.}
\item{\code{"blaker"}}{The minima of the observations' lower and upper tail
probabilities are combined with the opposite tail not
greater than these minima. More details can be found in
Blaker (2000) or Hirji (2006), where it is referred to as
the \emph{Combined Tails} method.}
\item{\code{"absdist"}}{The probabilities of the absolute distances from the
expected value that are greater than or equal to the
observed one are summed up. In Hirji (2006), it is
referred to as the \emph{Distance from Center} approach.}
\item{\code{"central"}}{The smaller values of the observations' simply doubles
the minimum of lower and upper tail probabilities. In
Hirji (2006), it is referred to as the \emph{Twice the
Smaller Tail} method.}
}
For non-exact (i.e. continuous approximation) approaches, \code{ts_method} is
ignored, since all its methods would yield the same p-values. More
specifically, they all converge to the doubling approach as in
\code{ts_mthod = "central"}.
}
\examples{
# Constructing
set.seed(3)
p1 <- c(0.25, 0.5, 0.75)
p2 <- c(0.15, 0.5, 0.60)
n1 <- c(10, 20, 50)
n2 <- c(25, 75, 200)
x1 <- rbinom(3, n1, p1)
x2 <- rbinom(3, n2, p2)
x  <- cbind(x1 = x1, x2 = x2)
n  <- cbind(n1 = n1, n2 = n2)

# Exact two-sided p-values ("blaker") and their supports
results_ex <- homogeneity_test_pv(x, n, ts_method = "blaker")
print(results_ex)
results_ex$get_pvalues()
results_ex$get_pvalue_supports()

# Normal-approximated one-sided p-values ("less") and their supports
results_ap <- homogeneity_test_pv(x, n, "less", exact = FALSE)
print(results_ap)
results_ap$get_pvalues()
results_ap$get_pvalue_supports()

}
\references{
Fisher, R. A. (1935). The logic of inductive inference.
\emph{Journal of the Royal Statistical Society Series A}, \strong{98}, pp.
39–54. \doi{10.2307/2342435}

Agresti, A. (2002). \emph{Categorical data analysis}. Second Edition. New York:
John Wiley & Sons. pp. 91–97. \doi{10.1002/0471249688}

Blaker, H. (2000) Confidence curves and improved exact confidence intervals
for discrete distributions. \emph{Canadian Journal of Statistics},
\strong{28}(4), pp. 783-798. \doi{10.2307/3315916}

Hirji, K. F. (2006). \emph{Exact analysis of discrete data}. New York: Chapman
and Hall/CRC. pp. 55-83. \doi{10.1201/9781420036190}
}
\seealso{
\code{\link[=fisher_test_pv]{fisher_test_pv()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mann-whitney.R
\name{mann_whitney_test_pv}
\alias{mann_whitney_test_pv}
\title{Wilcoxon-Mann-Whitney \emph{U} test}
\usage{
mann_whitney_test_pv(
  x,
  y,
  mu = 0,
  alternative = "two.sided",
  exact = NULL,
  correct = TRUE,
  digits_rank = Inf,
  simple_output = FALSE
)
}
\arguments{
\item{x, y}{numerical vectors forming the samples to be tested or lists
of numerical vectors for multiple tests.}

\item{mu}{numerical vector or single number of hypothesised location
shift(s).}

\item{alternative}{character vector that indicates the alternative hypotheses; each value must be one of \code{"two.sided"} (the default), \code{"less"} or \code{"greater"}.}

\item{exact}{single logical value that indicates whether \eqn{p}-values are to be calculated by exact computation (\code{TRUE}) or by a continuous approximation (\code{FALSE}). \code{NULL} (the default) is allowed (see details).}

\item{correct}{either a single logical value that indicates if a continuity correction in the normal approximation is to be applied (\code{TRUE}; the default) or not (\code{FALSE}), or a single integer between 0 and 3 specifying both that a continuity correction should be used \emph{and} the number of terms of an Edgeworth expansion for a more accurate normal approximation. Ignored, if \code{exact = TRUE}.}

\item{digits_rank}{single number giving the significant digits used to compute ranks for the test statistics.}

\item{simple_output}{logical value that indicates whether an R6 class object, including the tests' parameters and support sets, i.e. all observable p-values under each null hypothesis, is to be returned (see below).}
}
\value{
If \code{simple.output = TRUE}, a vector of computed p-values is returned.
Otherwise, the output is a \code{\link{DiscreteTestResults}} R6 class object, which
also includes the p-value supports and testing parameters. These have to be
accessed by public methods, e.g. \verb{$get_pvalues()}.
}
\description{
\code{mann_whitney_test_pv()} performs an exact or approximate
Wilcoxon-Mann-Whitney \emph{U} test about the location shift between two
independent groups when the data is not necessarily normally distributed. In
contrast to \code{\link[stats:wilcox.test]{stats::wilcox.test()}}, it is vectorised and only calculates
\emph{p}-values. Furthermore, it is capable of returning the discrete \emph{p}-value
supports, i.e. all observable \emph{p}-values under a null hypothesis. Multiple
tests can be evaluated simultaneously.
}
\details{
We use a test statistic called the Wilcoxon Rank Sum Statistic, defined by
\deqn{U = \sum_{i = 1}^{n_X}{rank(X_i)} - \frac{n_X(n_X + 1)}{2},}
where \eqn{rank(X_i)} is the rank of \eqn{X_i} in the concatenated sample
of \eqn{X} and \eqn{Y}, and \eqn{n_X} and \eqn{n_Y} are the respective
sizes of the samples \eqn{X} and \eqn{Y}. Note that \eqn{U}
can range from \eqn{0} to \eqn{n_X \cdot n_Y}.
This is the same statistic used by \code{\link[stats:wilcox.test]{stats::wilcox.test()}} and
whose distribution is accessible with \code{\link{pwilcox}}.
This is also the statistic defined by the two given references.
Note, however, that it is not what is called the Mann-Whitney U Statistic
in the (English-language) Wikipedia article (as of February 12, 2026). The
latter is defined as, using our notation, \eqn{\min(U, n_X \cdot n_Y - U)}.
Using the Wikipedia notation, the Wilcoxon Rank Sum Statistic is \eqn{U_2}.

The parameters \code{x}, \code{y}, \code{mu} and \code{alternative} are vectorised. If \code{x} and
\code{y} are lists, they are replicated automatically to have the same lengths. In
case \code{x} or \code{y} are not lists, they are added to new ones, which are then
replicated to the appropriate lengths. This allows multiple hypotheses to be
tested simultaneously.

In the presence of ties, computation of the Edgeworth series (up to
\code{correct = 3}) is not possible. Therefore, numeric values of \code{correct}
are ignored and only a continuity correction is performed.

By setting \code{exact = NULL}, exact computation is performed only if both
samples sizes in a test setting are lower than or equal to 200. Otherwise,
\eqn{p}-values are computed by normal approximation.

If \code{digits_rank = Inf} (the default), \code{\link[base:rank]{rank()}} is used to
compute ranks for the tests statistics instead of
\code{\link[base:rank]{rank}}(\code{\link[base:signif]{signif(., digits_rank)}})
}
\examples{
# Constructing
set.seed(1)
r1 <- rnorm(100)
r2 <- rnorm(100, 1)

# Exact two-sided p-values and their supports
results_ex  <- mann_whitney_test_pv(r1, r2)
print(results_ex)
results_ex$get_pvalues()
results_ex$get_pvalue_supports()

# Normal-approximated one-sided p-values ("less") and their supports
results_ap  <- mann_whitney_test_pv(r1, r2, alternative = "less", exact = FALSE)
print(results_ap)
results_ap$get_pvalues()
results_ap$get_pvalue_supports()

}
\references{
Mann, H. D. & Whitney, D. R. (1947). On a Test of Whether one of Two Random
Variables is Stochastically Larger than the Other. \emph{Ann. Math. Statist.},
\emph{18}(1), pp. 50-60. \doi{10.1214/aoms/1177730491}

Hollander, M. & Wolfe, D. (1973). \emph{Nonparametric Statistical Methods}. Third
Edition. New York: Wiley. pp. 115-135. \doi{10.1002/9781119196037}
}
\seealso{
\code{\link[stats:wilcox.test]{stats::wilcox.test()}}, \code{\link{pwilcox}}, \code{\link[=wilcox_test_pv]{wilcox_test_pv()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fisher.R
\name{fisher_test_pv}
\alias{fisher_test_pv}
\alias{fisher.test.pv}
\title{Fisher's Exact Test for Count Data}
\usage{
fisher_test_pv(
  x,
  alternative = "two.sided",
  ts_method = "minlike",
  exact = TRUE,
  correct = TRUE,
  simple_output = FALSE
)

fisher.test.pv(
  x,
  alternative = "two.sided",
  ts.method = "minlike",
  exact = TRUE,
  correct = TRUE,
  simple.output = FALSE
)
}
\arguments{
\item{x}{integer vector with four elements, a 2-by-2 matrix or an integer
matrix (or data frame) with four columns, where each line
represents a 2-by-2 table to be tested.}

\item{alternative}{character vector that indicates the alternative hypotheses; each value must be one of \code{"two.sided"} (the default), \code{"less"} or \code{"greater"}.}

\item{ts_method, ts.method}{single character string that indicates the two-sided p-value computation method (if any value in \code{alternative} equals \code{"two.sided"}) and must be one of \code{"minlike"} (the default), \code{"blaker"}, \code{"absdist"} or \code{"central"} (see details). Ignored, if \code{exact = FALSE}.}

\item{exact}{logical value that indicates whether \eqn{p}-values are to be calculated by exact computation (\code{TRUE}; the default) or by a continuous approximation (\code{FALSE}).}

\item{correct}{logical value that indicates if a continuity correction is to be applied (\code{TRUE}; the default) or not (\code{FALSE}). Ignored, if \code{exact = TRUE}.}

\item{simple_output, simple.output}{logical value that indicates whether an R6 class object, including the tests' parameters and support sets, i.e. all observable p-values under each null hypothesis, is to be returned (see below).}
}
\value{
If \code{simple.output = TRUE}, a vector of computed p-values is returned.
Otherwise, the output is a \code{\link{DiscreteTestResults}} R6 class object, which
also includes the p-value supports and testing parameters. These have to be
accessed by public methods, e.g. \verb{$get_pvalues()}.
}
\description{
\code{fisher_test_pv()} performs Fisher's exact test or a chi-square approximation
to assess if rows and columns of a 2-by-2 contingency table with fixed
marginals are independent. In contrast to \code{\link[stats:fisher.test]{stats::fisher.test()}}, it is
vectorised, only calculates \emph{p}-values and offers a normal approximation of
their computation. Furthermore, it is capable of returning the discrete
\emph{p}-value supports, i.e. all observable \emph{p}-values under a null hypothesis.
Multiple tables can be analysed simultaneously. In two-sided tests, several
procedures of obtaining the respective \emph{p}-values are implemented.

\strong{Note}: Please do not use the older \code{fisher.test.pv()} anymore! It is now
defunct and will be removed in a future version.\cr
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#superseded}{\figure{lifecycle-superseded.svg}{options: alt='[Superseded]'}}}{\strong{[Superseded]}}
}
\details{
The parameters \code{x} and \code{alternative} are vectorised. They are replicated
automatically, such that the number of \code{x}'s rows is the same as the length
of \code{alternative}. This allows multiple null hypotheses to be tested
simultaneously. Since \code{x} is (if necessary) coerced to a matrix with four
columns, it is replicated row-wise.

If \code{exact = TRUE}, Fisher's exact test is performed (the specific hypothesis
depends on the value of \code{alternative}). Otherwise, if \code{exact = FALSE}, a
chi-square approximation is used for two-sided hypotheses or a normal
approximation for one-sided tests, based on the square root of the
chi-squared statistic. This is possible because the degrees of freedom of
chi-squared tests on 2-by-2 tables are limited to 1.

For exact computation, various procedures of determining two-sided p-values
are implemented.
\describe{
\item{\code{"minlike"}}{The standard approach in \code{\link[stats:fisher.test]{stats::fisher.test()}} and
\code{\link[stats:binom.test]{stats::binom.test()}}. The probabilities of the
likelihoods that are equal or less than the observed one
are summed up. In Hirji (2006), it is referred to as the
\emph{Probability-based} approach.}
\item{\code{"blaker"}}{The minima of the observations' lower and upper tail
probabilities are combined with the opposite tail not
greater than these minima. More details can be found in
Blaker (2000) or Hirji (2006), where it is referred to as
the \emph{Combined Tails} method.}
\item{\code{"absdist"}}{The probabilities of the absolute distances from the
expected value that are greater than or equal to the
observed one are summed up. In Hirji (2006), it is
referred to as the \emph{Distance from Center} approach.}
\item{\code{"central"}}{The smaller values of the observations' simply doubles
the minimum of lower and upper tail probabilities. In
Hirji (2006), it is referred to as the \emph{Twice the
Smaller Tail} method.}
}
For non-exact (i.e. continuous approximation) approaches, \code{ts_method} is
ignored, since all its methods would yield the same p-values. More
specifically, they all converge to the doubling approach as in
\code{ts_mthod = "central"}.
}
\examples{
# Constructing
S1 <- c(4, 2, 2, 14, 6, 9, 4, 0, 1)
S2 <- c(0, 0, 1, 3, 2, 1, 2, 2, 2)
N1 <- rep(148, 9)
N2 <- rep(132, 9)
F1 <- N1 - S1
F2 <- N2 - S2
df <- data.frame(S1, F1, S2, F2)

# Fisher's exact p-values and their supports
results_ex <- fisher_test_pv(df)
print(results_ex)
results_ex$get_pvalues()
results_ex$get_pvalue_supports()

# Chi-square-approximated p-values and their supports
results_ap <- fisher_test_pv(df, exact = FALSE)
print(results_ap)
results_ap$get_pvalues()
results_ap$get_pvalue_supports()

}
\references{
Fisher, R. A. (1935). The logic of inductive inference.
\emph{Journal of the Royal Statistical Society Series A}, \strong{98}, pp.
39–54. \doi{10.2307/2342435}

Agresti, A. (2002). \emph{Categorical data analysis}. Second Edition. New York:
John Wiley & Sons. pp. 91–97. \doi{10.1002/0471249688}

Blaker, H. (2000). Confidence curves and improved exact confidence intervals
for discrete distributions. \emph{Canadian Journal of Statistics},
\strong{28}(4), pp. 783-798. \doi{10.2307/3315916}

Hirji, K. F. (2006). \emph{Exact analysis of discrete data}. New York: Chapman
and Hall/CRC. pp. 55-83. \doi{10.1201/9781420036190}
}
\seealso{
\code{\link[stats:fisher.test]{stats::fisher.test()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/poisson.R
\name{poisson_test_pv}
\alias{poisson_test_pv}
\alias{poisson.test.pv}
\title{Poisson Test}
\usage{
poisson_test_pv(
  x,
  lambda = 1,
  alternative = "two.sided",
  ts_method = "minlike",
  exact = TRUE,
  correct = TRUE,
  simple_output = FALSE
)

poisson.test.pv(
  x,
  lambda = 1,
  alternative = "two.sided",
  ts.method = "minlike",
  exact = TRUE,
  correct = TRUE,
  simple.output = FALSE
)
}
\arguments{
\item{x}{integer vector giving the number of events.}

\item{lambda}{non-negative numerical vector of hypothesised rate(s).}

\item{alternative}{character vector that indicates the alternative hypotheses; each value must be one of \code{"two.sided"} (the default), \code{"less"} or \code{"greater"}.}

\item{ts_method, ts.method}{single character string that indicates the two-sided p-value computation method (if any value in \code{alternative} equals \code{"two.sided"}) and must be one of \code{"minlike"} (the default), \code{"blaker"}, \code{"absdist"} or \code{"central"} (see details). Ignored, if \code{exact = FALSE}.}

\item{exact}{single logical value that indicates whether \eqn{p}-values are to be calculated by exact computation (\code{TRUE}; the default) or by a continuous approximation (\code{FALSE}).}

\item{correct}{single logical value that indicates if a continuity correction in the normal approximation is to be applied (\code{TRUE}; the default) or not (\code{FALSE}). Ignored, if \code{exact = TRUE}.}

\item{simple_output, simple.output}{logical value that indicates whether an R6 class object, including the tests' parameters and support sets, i.e. all observable p-values under each null hypothesis, is to be returned (see below).}
}
\value{
If \code{simple.output = TRUE}, a vector of computed p-values is returned.
Otherwise, the output is a \code{\link{DiscreteTestResults}} R6 class object, which
also includes the p-value supports and testing parameters. These have to be
accessed by public methods, e.g. \verb{$get_pvalues()}.
}
\description{
\code{poisson_test_pv()} performs an exact or approximate Poisson test about the
rate parameter of a Poisson distribution. In contrast to
\code{\link[stats:poisson.test]{stats::poisson.test()}}, it is vectorised, only calculates \emph{p}-values and
offers a normal approximation of their computation. Furthermore, it is
capable of returning the discrete \emph{p}-value supports, i.e. all observable
\emph{p}-values under a null hypothesis. Multiple tests can be evaluated
simultaneously. In two-sided tests, several procedures of obtaining the
respective \emph{p}-values are implemented.

\strong{Note}: Please do not use the older \code{poisson.test.pv()} anymore! It is now
defunct and will be removed in a future version.\cr
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#superseded}{\figure{lifecycle-superseded.svg}{options: alt='[Superseded]'}}}{\strong{[Superseded]}}
}
\details{
The parameters \code{x}, \code{lambda} and \code{alternative} are vectorised. They are
replicated automatically to have the same lengths. This allows multiple null
hypotheses to be tested simultaneously.

Since the Poisson distribution itself has an infinite support, so do the
\emph{p}-values of exact Poisson tests. Therefore, supports only contain
\emph{p}-values that are not rounded off to 0.

For exact computation, various procedures of determining two-sided p-values
are implemented.
\describe{
\item{\code{"minlike"}}{The standard approach in \code{\link[stats:fisher.test]{stats::fisher.test()}} and
\code{\link[stats:binom.test]{stats::binom.test()}}. The probabilities of the
likelihoods that are equal or less than the observed one
are summed up. In Hirji (2006), it is referred to as the
\emph{Probability-based} approach.}
\item{\code{"blaker"}}{The minima of the observations' lower and upper tail
probabilities are combined with the opposite tail not
greater than these minima. More details can be found in
Blaker (2000) or Hirji (2006), where it is referred to as
the \emph{Combined Tails} method.}
\item{\code{"absdist"}}{The probabilities of the absolute distances from the
expected value that are greater than or equal to the
observed one are summed up. In Hirji (2006), it is
referred to as the \emph{Distance from Center} approach.}
\item{\code{"central"}}{The smaller values of the observations' simply doubles
the minimum of lower and upper tail probabilities. In
Hirji (2006), it is referred to as the \emph{Twice the
Smaller Tail} method.}
}
For non-exact (i.e. continuous approximation) approaches, \code{ts_method} is
ignored, since all its methods would yield the same p-values. More
specifically, they all converge to the doubling approach as in
\code{ts_mthod = "central"}.
}
\examples{
# Constructing
k <- c(4, 2, 2, 14, 6, 9, 4, 0, 1)
lambda <- c(3, 2, 1)

# Exact two-sided p-values ("blaker") and their supports
results_ex  <- poisson_test_pv(k, lambda, ts_method = "blaker")
print(results_ex)
results_ex$get_pvalues()
results_ex$get_pvalue_supports()

# Normal-approximated one-sided p-values ("less") and their supports
results_ap  <- poisson_test_pv(k, lambda, "less", exact = FALSE)
print(results_ap)
results_ap$get_pvalues()
results_ap$get_pvalue_supports()

}
\references{
Blaker, H. (2000). Confidence curves and improved exact confidence intervals
for discrete distributions. \emph{Canadian Journal of Statistics},
\strong{28}(4), pp. 783-798. \doi{10.2307/3315916}

Hirji, K. F. (2006). \emph{Exact analysis of discrete data}. New York: Chapman
and Hall/CRC. pp. 55-83. \doi{10.1201/9781420036190}
}
\seealso{
\code{\link[stats:poisson.test]{stats::poisson.test()}}, \code{\link[=binom_test_pv]{binom_test_pv()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-summary.R
\name{DiscreteTestResultsSummary}
\alias{DiscreteTestResultsSummary}
\title{Discrete Test Results Summary Class}
\description{
This is the class used by \code{DiscreteTests} for summarising
\code{\link{DiscreteTestResults}} objects. It contains the summarised objects itself,
as well as a summary \code{\link[tibble:tibble]{tibble}} object as private members.
Both can be extracted by public methods.
}
\examples{
# binomial tests
obj <- binom_test_pv(0:5, 5, 0.5)
# create DiscreteTestResultsSummary object
res <- DiscreteTestResultsSummary$new(obj)
# print summary
print(res)
# extract summary table
res$get_summary_table()

}
\section{Methods}{
\subsection{Public methods}{
  \itemize{
    \item \href{#method-summary.DiscreteTestResults-initialize}{\code{summary.DiscreteTestResults$new()}}
    \item \href{#method-summary.DiscreteTestResults-get_test_results}{\code{summary.DiscreteTestResults$get_test_results()}}
    \item \href{#method-summary.DiscreteTestResults-get_summary_table}{\code{summary.DiscreteTestResults$get_summary_table()}}
    \item \href{#method-summary.DiscreteTestResults-print}{\code{summary.DiscreteTestResults$print()}}
    \item \href{#method-summary.DiscreteTestResults-clone}{\code{summary.DiscreteTestResults$clone()}}
  }
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-summary.DiscreteTestResults-initialize"></a>}}
\if{latex}{\out{\hypertarget{method-summary.DiscreteTestResults-initialize}{}}}
\subsection{\code{summary.DiscreteTestResults$new()}}{
  Creates a new \code{summary.DiscreteTestResults} object.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{summary.DiscreteTestResults$new(test_results)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{test_results}}{the \code{\link{DiscreteTestResults}} class object to be
summarised.}
    }
    \if{html}{\out{</div>}}
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-summary.DiscreteTestResults-get_test_results"></a>}}
\if{latex}{\out{\hypertarget{method-summary.DiscreteTestResults-get_test_results}{}}}
\subsection{\code{summary.DiscreteTestResults$get_test_results()}}{
  Returns the underlying \link{DiscreteTestResults} object.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{summary.DiscreteTestResults$get_test_results()}
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A \link{DiscreteTestResults} R6 class object.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-summary.DiscreteTestResults-get_summary_table"></a>}}
\if{latex}{\out{\hypertarget{method-summary.DiscreteTestResults-get_summary_table}{}}}
\subsection{\code{summary.DiscreteTestResults$get_summary_table()}}{
  Returns the summary table of the underlying \link{DiscreteTestResults} object.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{summary.DiscreteTestResults$get_summary_table()}
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    A data frame.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-summary.DiscreteTestResults-print"></a>}}
\if{latex}{\out{\hypertarget{method-summary.DiscreteTestResults-print}{}}}
\subsection{\code{summary.DiscreteTestResults$print()}}{
  Prints the summary.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{summary.DiscreteTestResults$print(...)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{...}}{further arguments passed to \code{print.data.frame}.}
    }
    \if{html}{\out{</div>}}
  }
  \subsection{Returns}{
    Prints a summary table of the tested null hypotheses. The object itself
is invisibly returned.
  }
}

\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-summary.DiscreteTestResults-clone"></a>}}
\if{latex}{\out{\hypertarget{method-summary.DiscreteTestResults-clone}{}}}
\subsection{\code{summary.DiscreteTestResults$clone()}}{
  The objects of this class are cloneable with this method.
  \subsection{Usage}{
    \if{html}{\out{<div class="r">}}
    \preformatted{summary.DiscreteTestResults$clone(deep = FALSE)}
    \if{html}{\out{</div>}}
  }
  \subsection{Arguments}{
    \if{html}{\out{<div class="arguments">}}
    \describe{
      \item{\code{deep}}{Whether to make a deep clone.}
    }
    \if{html}{\out{</div>}}
  }
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wilcoxon.R
\name{wilcox_test_pv}
\alias{wilcox_test_pv}
\title{Wilcoxon's signed-rank test}
\usage{
wilcox_test_pv(
  x,
  y = NULL,
  mu = 0,
  alternative = "two.sided",
  exact = NULL,
  correct = TRUE,
  digits_rank = Inf,
  zero_method = "pratt",
  simple_output = FALSE
)
}
\arguments{
\item{x}{numerical vector forming the sample to be tested or a
list of numerical vectors for multiple tests.}

\item{y}{numerical vector forming the second sample to be
tested or a list of numerical vectors for multiple
tests; if \code{y = NULL} (the default), the one-sample
version is performed; for two-sample tests, all sample
pairs must have the same length.}

\item{mu}{numerical vector or single number of hypothesised
location(s) for one-sample tests or location shift(s)
for two-sample tests.}

\item{alternative}{character vector that indicates the alternative hypotheses; each value must be one of \code{"two.sided"} (the default), \code{"less"} or \code{"greater"}.}

\item{exact}{single logical value that indicates whether \eqn{p}-values are to be calculated by exact computation (\code{TRUE}) or by a continuous approximation (\code{FALSE}). \code{NULL} (the default) is allowed (see details).}

\item{correct}{either a single logical value that indicates if a continuity correction in the normal approximation is to be applied (\code{TRUE}; the default) or not (\code{FALSE}), or a single integer between 0 and 3 specifying both that a continuity correction should be used \emph{and} the number of terms of an Edgeworth expansion for a more accurate normal approximation. Ignored, if \code{exact = TRUE}.}

\item{digits_rank}{single number giving the significant digits used to compute ranks for the test statistics.}

\item{zero_method}{character vector or single string specifying how zero
differences are handled; must either be \code{"pratt"} (the
default) or \code{"wilcoxon"}. With \code{"pratt"}, zero
differences are included when computing ranks, but
excluded from the test statistic; with \code{"wilcoxon"},
zero differences are discarded entirely before ranking
(the original Wilcoxon approach).}

\item{simple_output}{logical value that indicates whether an R6 class object, including the tests' parameters and support sets, i.e. all observable p-values under each null hypothesis, is to be returned (see below).}
}
\value{
If \code{simple.output = TRUE}, a vector of computed p-values is returned.
Otherwise, the output is a \code{\link{DiscreteTestResults}} R6 class object, which
also includes the p-value supports and testing parameters. These have to be
accessed by public methods, e.g. \verb{$get_pvalues()}.
}
\description{
\code{wilcox_test_pv()} performs an exact or approximate Wilcoxon signed-rank test
about the location of a population on a single sample or the differences
between two paired groups when the data is not necessarily normally
distributed. In contrast to \code{\link[stats:wilcox.test]{stats::wilcox.test()}}, it is vectorised and
only calculates \emph{p}-values. Furthermore, it is capable of returning the
discrete \emph{p}-value supports, i.e. all observable \emph{p}-values under a null
hypothesis. Multiple tests can be evaluated simultaneously.
}
\details{
The parameters \code{x}, \code{mu}, \code{alternative} and \code{zero_method} are vectorised. If
\code{x} is a list, they are replicated automatically to have the same lengths. In
case \code{x} is not a list, it is added to one, which is then replicated to the
appropriate length. This allows multiple hypotheses to be tested
simultaneously.

By setting \code{exact = NULL}, exact computation is performed only if the sample
size in a test setting is smaller than or equal to 200. Otherwise,
\eqn{p}-values are computed by normal approximation.

The used test statistics \code{W} is also known as \eqn{T+} and is defined as the
sum of ranks of all strictly positive values of the sample \code{x}.

If \code{digits_rank = Inf} (the default), \code{\link[base:rank]{rank()}} is used to
compute ranks for the tests statistics instead of
\code{\link[base:rank]{rank}}(\code{\link[base:signif]{signif(., digits_rank)}})
}
\examples{
# Constructing
set.seed(1)
r1 <- rnorm(200)
r2 <- rnorm(200, 1)
r3 <- rnorm(200, 2)

## One-sample tests
#  Exact two-sided p-values and their supports
results_ex_1s <- wilcox_test_pv(r1)
print(results_ex_1s)
results_ex_1s$get_pvalues()
results_ex_1s$get_pvalue_supports()

#  Multiple normal-approximated one-sided tests ("greater")
results_ap_1s <- wilcox_test_pv(list(r1, r2), alternative = "greater", exact = FALSE)
print(results_ap_1s)
results_ap_1s$get_pvalues()
results_ap_1s$get_pvalue_supports()

## Two-sample-tests
#  Normal-approximated one-sided p-values ("less") and their supports
results_ap_2s <- wilcox_test_pv(r1, r2, alternative = "less", exact = FALSE)
print(results_ap_2s)
results_ap_2s$get_pvalues()
results_ap_2s$get_pvalue_supports()

#  Multiple exact two-sided tests ("greater")
results_ex_2s <- wilcox_test_pv(list(r1, r3), r2)
print(results_ex_2s)
results_ex_2s$get_pvalues()
results_ex_2s$get_pvalue_supports()

}
\references{
Hollander, M. & Wolfe, D. (1973). \emph{Nonparametric Statistical Methods}. Third
Edition. New York: Wiley. pp. 40-55. \doi{10.1002/9781119196037}
}
\seealso{
\code{\link[stats:wilcox.test]{stats::wilcox.test()}}, \code{\link[=mann_whitney_test_pv]{mann_whitney_test_pv()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sign.r
\name{sign_test_pv}
\alias{sign_test_pv}
\title{Sign Tests}
\usage{
sign_test_pv(
  x,
  y = NULL,
  mu = 0,
  alternative = "two.sided",
  exact = TRUE,
  correct = TRUE,
  simple_output = FALSE
)
}
\arguments{
\item{x}{numerical vector forming the sample to be tested or a list of
numerical vectors for multiple tests.}

\item{y}{numerical vector forming the second sample to be tested or a
list of numerical vectors for multiple tests; if \code{y = NULL}
(the default), the one-sample version is performed; for
two-sample tests, all sample pairs must have the same length.}

\item{mu}{numerical vector of hypothesised median(s) for one-sample
tests or median(s) of differences for two-sample tests.}

\item{alternative}{character vector that indicates the alternative hypotheses; each value must be one of \code{"two.sided"} (the default), \code{"less"} or \code{"greater"}.}

\item{exact}{single logical value that indicates whether \eqn{p}-values are to be calculated by exact computation (\code{TRUE}; the default) or by a continuous approximation (\code{FALSE}).}

\item{correct}{single logical value that indicates if a continuity correction in the normal approximation is to be applied (\code{TRUE}; the default) or not (\code{FALSE}). Ignored, if \code{exact = TRUE}.}

\item{simple_output}{logical value that indicates whether an R6 class object, including the tests' parameters and support sets, i.e. all observable p-values under each null hypothesis, is to be returned (see below).}
}
\value{
If \code{simple.output = TRUE}, a vector of computed p-values is returned.
Otherwise, the output is a \code{\link{DiscreteTestResults}} R6 class object, which
also includes the p-value supports and testing parameters. These have to be
accessed by public methods, e.g. \verb{$get_pvalues()}.
}
\description{
\code{sign_test_pv()} performs an exact or approximate sign test about the median
of a distribution. It supports both one-sample and two-sample paired tests.
In contrast to other implementations, it is vectorised, only calculates
\emph{p}-values, and offers a normal approximation of their computation.
Furthermore, it is capable of returning the discrete \emph{p}-value supports, i.e.
all observable \emph{p}-values under a null hypothesis. Multiple tests can be
evaluated simultaneously. In two-sided tests, several procedures of obtaining
the respective \emph{p}-values are implemented.
}
\details{
For the \strong{one-sample} case, the sign test counts the number of observations
in \code{x} that exceed the hypothesised median \code{mu}. Observations exactly equal
to \code{mu} (ties) are discarded. Under the null hypothesis, the number of
positive signs follows a Binomial(\eqn{n}, 0.5) distribution, where \eqn{n}
is the number of non-tied observations.

For the \strong{two-sample paired} case (when \code{y} is supplied), the pairwise
differences \eqn{d_i = x_i - y_i - \mu} are computed first. The sign test is
then applied to these differences.

If \code{x} is a list, multiple tests are evaluated simultaneously. The parameters
\code{x}, \code{y} (if supplied and a list), \code{mu}, and \code{alternative} are vectorised.
They are replicated automatically to have the same lengths. This allows
multiple hypotheses to be tested simultaneously.

The sign test is a special case of the binomial test. In contrast to
\code{\link[=binom_test_pv]{binom_test_pv()}}, \code{sign_test_pv()} does not allow specifying exact
two-sided \emph{p}-value calculation procedures. The reason is that the exact sign
test always tests for a probability of 0.5, in which case all these exact
two-sided \emph{p}-value computation methods yield exactly the same results.
}
\examples{
# One-sample sign test: test whether median equals 3
x <- c(1, 5, 2, 7, 6, 8, 4)
results_1s <- sign_test_pv(x, mu = 3)
print(results_1s)
results_1s$get_pvalues()
results_1s$get_pvalue_supports()

# Paired two-sample sign test: test whether difference of medians equals 1
x2 <- c(6, 8, 2, 4, 5)
y2 <- c(3, 5, 4, 2, 6)
results_2s <- sign_test_pv(x2, y2, mu = 1)
print(results_2s)
results_2s$get_pvalues()

# Multiple tests simultaneously, one-sided p-values (one-sample, list input)
xs <- list(c(1, 5, 2, 7, 6, 8, 4), c(2, 4, 6, 1, 9))
results_l <- sign_test_pv(xs, mu = c(3, 5), alternative = "greater")
print(results_l)
results_l$get_pvalues()

# Normal-approximated (one-sample, list input)
results_a <- sign_test_pv(xs, mu = c(3, 5), exact = FALSE)
print(results_a)
results_a$get_pvalues()

}
\references{
Dixon, W. J. and Mood, A. M. (1946). The statistical sign test.
\emph{Journal of the American Statistical Association}, \strong{41}(236),
pp. 557–566. \doi{10.1080/01621459.1946.10501898}
}
\seealso{
\code{\link[=binom_test_pv]{binom_test_pv()}}, \code{\link[stats:binom.test]{stats::binom.test()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-summary.R
\name{summary.DiscreteTestResults}
\alias{summary.DiscreteTestResults}
\title{Summarizing Discrete Test Results}
\usage{
\method{summary}{DiscreteTestResults}(object, ...)
}
\arguments{
\item{object}{object of class \code{\link{DiscreteTestResults}} to be summarised;
usually created by using one of the packages test functions,
e.g. \code{\link[=binom_test_pv]{binom_test_pv()}}, with \code{simple_output = FALSE}.}

\item{...}{further arguments passed to or from other methods.}
}
\value{
A \code{\link[=DiscreteTestResultsSummary]{summary.DiscreteTestResults}} R6 class
object.
}
\description{
\code{summary} method for class \code{\link{DiscreteTestResults}}.
}
\examples{
# binomial tests
obj <- binom_test_pv(0:5, 5, 0.4)
# print summary
summary(obj)
# extract summary table
smry <- summary(obj)
smry$get_summary_table()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcnemar.R
\name{mcnemar_test_pv}
\alias{mcnemar_test_pv}
\alias{mcnemar.test.pv}
\title{McNemar's Test for Count Data}
\usage{
mcnemar_test_pv(
  x,
  alternative = "two.sided",
  exact = TRUE,
  correct = TRUE,
  simple_output = FALSE
)

mcnemar.test.pv(
  x,
  alternative = "two.sided",
  exact = TRUE,
  correct = TRUE,
  simple.output = FALSE
)
}
\arguments{
\item{x}{integer vector with four elements, a 2-by-2 matrix or an integer
matrix (or data frame) with four columns where each line
represents a 2-by-2 table to be tested.}

\item{alternative}{character vector that indicates the alternative hypotheses; each value must be one of \code{"two.sided"} (the default), \code{"less"} or \code{"greater"}.}

\item{exact}{single logical value that indicates whether \eqn{p}-values are to be calculated by exact computation (\code{TRUE}; the default) or by a continuous approximation (\code{FALSE}).}

\item{correct}{single logical value that indicates if a continuity correction in the normal approximation is to be applied (\code{TRUE}; the default) or not (\code{FALSE}). Ignored, if \code{exact = TRUE}.}

\item{simple_output, simple.output}{logical value that indicates whether an R6 class object, including the tests' parameters and support sets, i.e. all observable p-values under each null hypothesis, is to be returned (see below).}
}
\value{
If \code{simple.output = TRUE}, a vector of computed p-values is returned.
Otherwise, the output is a \code{\link{DiscreteTestResults}} R6 class object, which
also includes the p-value supports and testing parameters. These have to be
accessed by public methods, e.g. \verb{$get_pvalues()}.
}
\description{
Performs McNemar's chi-square test or an exact variant to assess the symmetry
of rows and columns in a 2-by-2 contingency table. In contrast to
\code{\link[stats:mcnemar.test]{stats::mcnemar.test()}}, it is vectorised, only calculates \emph{p}-values and
offers their exact computation. Furthermore, it is capable of returning the
discrete \emph{p}-value supports, i.e. all observable \emph{p}-values under a null
hypothesis. Multiple tables can be analysed simultaneously. In two-sided
tests, several procedures of obtaining the respective \emph{p}-values are
implemented. It is a special case of the \link[=binom_test_pv]{binomial test}.

\strong{Note}: Please do not use the older \code{mcnemar.test.pv()} anymore! It is now
defunct and will be removed in a future version.\cr
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#superseded}{\figure{lifecycle-superseded.svg}{options: alt='[Superseded]'}}}{\strong{[Superseded]}}
}
\details{
The parameters \code{x} and \code{alternative} are vectorised. They are replicated
automatically, such that the number of \code{x}'s rows is the same as the length
of \code{alternative}. This allows multiple null hypotheses to be tested
simultaneously. Since \code{x} is coerced to a matrix (if necessary) with four
columns, it is replicated row-wise.

It can be shown that McNemar's test is a special case of the binomial test.
In contrast to \code{\link[=binom_test_pv]{binom_test_pv()}}, \code{mcnemar_test_pv()} does not allow
specifying exact two-sided p-value calculation procedures. The reason is that
McNemar's exact test always tests for a probability of 0.5, in which case all
these exact two-sided p-value computation methods yield exactly the same
results.
}
\examples{
# Constructing
S1 <- c(4, 2, 2, 14, 6, 9, 4, 0, 1)
S2 <- c(0, 0, 1, 3, 2, 1, 2, 2, 2)
N1 <- rep(148, 9)
N2 <- rep(132, 9)
F1 <- N1 - S1
F2 <- N2 - S2
df <- data.frame(S1, F1, S2, F2)

# Exact p-values and their supports
results_ex  <- mcnemar_test_pv(df)
print(results_ex)
results_ex$get_pvalues()
results_ex$get_pvalue_supports()

# Chi-square-approximated p-values and their supports
results_ap  <- mcnemar_test_pv(df, exact = FALSE)
print(results_ap)
results_ap$get_pvalues()
results_ap$get_pvalue_supports()

}
\references{
Agresti, A. (2002). \emph{Categorical data analysis}. Second Edition. New York:
John Wiley & Sons. pp. 411–413. \doi{10.1002/0471249688}
}
\seealso{
\code{\link[stats:mcnemar.test]{stats::mcnemar.test()}}, \code{\link[=binom_test_pv]{binom_test_pv()}}
}
